/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: CustomerImpl.java,v 1.1 2004/08/16 09:33:11 slobodan Exp $
 */

package com.lutris.airsent.business.customer;


import com.lutris.airsent.spec.AirSentException;

import com.lutris.airsent.business.*;

import com.lutris.airsent.spec.customer.Customer;

import com.lutris.airsent.business.address.*;

import com.lutris.airsent.spec.address.Address;


import com.lutris.airsent.data.person.*;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.appserver.server.sql.ObjectIdException;
import com.lutris.dods.builder.generator.query.DataObjectException;


/**
 * Customer implementation
 */
public class CustomerImpl 
    implements Customer, java.io.Serializable {

    CustomerDO customerDO = null;
    PersonDO personDO = null;

    /**
     * Create a new customer
     *
     * @exception if an error occurs
     */
    CustomerImpl() 
        throws AirSentException {
	try {
            customerDO = CustomerDO.createVirgin();
            personDO = PersonDO.createVirgin();
            
            AddressManager af = HomeManagerImpl.getInstance().getAddressManager();
            
            AddressImpl addr = af.create();
            
            personDO.setAddressID(addr.getDataObject());
	    customerDO.setPersonID(personDO);
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    }

    /**
     * Create an existing customer
     *
     * @param customer customer data object
     * @exception if an error occurs
     */
    public CustomerImpl(CustomerDO customer) 
        throws AirSentException {
	try {
	    customerDO = customer;
	    personDO = customerDO.getPersonID();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    }

    /**
     * Gets the customer handle.
     * 
     * @return database id
     * @exception if an error occurs
     */
    public String getHandle() 
        throws AirSentException {
	try {
	    return customerDO.getHandle();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 
    
    /**
     * Gets the first name.
     *
     * @exception if an error occurs
     */
    public String getFirstName() 
        throws AirSentException {
	try {
	    return personDO.getFirstName();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Sets the first name.
     *
     * @exception if an error occurs
     */
    public void setFirstName(String first) 
        throws AirSentException, AirSentBusinessException {
        if (first.length() > Customer.MAX_FIRST_NAME) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
	try {
	    personDO.setFirstName(first);
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Gets the last name.
     *
     * @return last name
     * @exception if an error occurs
     */
    public String getLastName() 
        throws AirSentException {
	try {
	    return personDO.getLastName();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Set the last name.
     *
     * @param last last name
     * @exception if an error occurs
     */
    public void setLastName(String last) 
        throws AirSentException, AirSentBusinessException {
        if (last.length() > Customer.MAX_LAST_NAME) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
	try {
	    personDO.setLastName(last);
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Gets the email.
     *
     * @return email address
     * @exception if an error occurs
     */
    public String getEmail() 
        throws AirSentException {
	try {
	    return personDO.getEmail();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Sets the email.
     *
     * @param email email address
     * @exception if an error occurs
     */
    public void setEmail(String email) 
        throws AirSentException, AirSentBusinessException {
        if (email.length() > Customer.MAX_EMAIL) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
	try {
	    personDO.setEmail(email);
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Gets the address.
     *
     * @return address
     * @exception if an error occurs
     */
    public Address getAddress() 
        throws AirSentException {
	try {
            AddressManager af = HomeManagerImpl.getInstance().getAddressManager();
            return af.create(personDO.getAddressID());
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Gets the business.
     *
     * @return business
     * @exception if an error occurs
     */
    public String getBusiness() 
        throws AirSentException {
	try {
	    return customerDO.getBusiness();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Sets the business.
     *
     * @param business business
     * @exception if an error occurs
     */
    public void setBusiness(String business) 
        throws AirSentException, AirSentBusinessException {
        if (business.length() > Customer.MAX_BUSINESS) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
	try {
	    customerDO.setBusiness(business);
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Gets the login.
     *
     * @return login name
     * @exception if an error occurs
     */
    public String getLogin() 
        throws AirSentException {
	try {
	    return customerDO.getLogin();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Sets the login.
     *
     * @param login login name
     * @exception if an error occurs
     */
    public void setLogin(String login) 
        throws AirSentException, AirSentBusinessException {
        if (login.length() > Customer.MAX_LOGIN) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
	try {
	    customerDO.setLogin(login);
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Gets the password.
     *
     * @return password
     * @exception if an error occurs
     */
    public String getPassword()
        throws AirSentException {
	try {
	    return customerDO.getPassword();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Sets the password.
     *
     * @param password password
     * @exception if an error occurs
     */
    public void setPassword(String password) 
        throws AirSentException, AirSentBusinessException {
        if (password.length() > Customer.MAX_PASSWORD) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
	try {
	    customerDO.setPassword(password);
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Saves the customer.
     *
     * @exception if an error occurs
     */
    public void save() 
        throws AirSentException {
	try {
	    customerDO.commit();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Deletes the customer.
     *
     * @exception if an error occurs
     */
    public void delete() 
        throws AirSentException {
	try {
	    customerDO.delete();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 
}

