/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: DeliveryImpl.java,v 1.2 2004/10/14 11:13:11 lola Exp $
 */

package com.lutris.airsent.business.delivery;


import com.lutris.airsent.spec.AirSentException;
import com.lutris.airsent.spec.delivery.Delivery;
import com.lutris.airsent.spec.delivery.Update;
import com.lutris.airsent.spec.messenger.Messenger;

import com.lutris.airsent.business.*;
import com.lutris.airsent.business.delivery.*;
import com.lutris.airsent.data.delivery.DeliveryDO;

import com.lutris.airsent.business.address.AddressManager;


import com.lutris.airsent.business.address.AddressImpl;
import com.lutris.airsent.business.address.AddressManager;

import com.lutris.airsent.spec.address.Address;
import com.lutris.airsent.spec.messenger.MessengerManager;

import com.lutris.airsent.spec.customer.Customer;

import com.lutris.airsent.business.customer.CustomerImpl;
import com.lutris.airsent.data.person.CustomerDO;
import com.lutris.airsent.business.messenger.*;
import com.lutris.airsent.data.person.MessengerDO;
import com.lutris.dods.builder.generator.query.DataObjectException;
import java.util.*;
import java.text.*;


/**
 * Delivery Implementation
 */
public class DeliveryImpl implements Delivery {

    DeliveryDO deliveryDO;
    private Update update;
    private AddressImpl pickUpAddress;
    private AddressImpl dropOffAddress;
    private Customer customer;
    private Messenger messenger;
    
    /**
     * Constructs DeliveryImpl object
     *
     * 
     * @exception if an error occurs
     */
    DeliveryImpl(Update update, Customer customer) 
        throws AirSentException {

        try {
            deliveryDO = DeliveryDO.createVirgin();
            AddressManager af = HomeManagerImpl.getInstance().getAddressManager();
            pickUpAddress = af.create();
            deliveryDO.setPickup(pickUpAddress.getDataObject());
            dropOffAddress = af.create();
            deliveryDO.setDropoff(dropOffAddress.getDataObject());
            setCustomer(customer);
            this.update = update;
        } catch (Exception e) {
            throw new AirSentException(e);
        }
    }
     
    /**
     * Constructs DeliveryImpl object
     *
     * 
     * @exception if an error occurs
     */
    DeliveryImpl(Update update, DeliveryDO d) 
        throws AirSentBusinessException {
        if (d == null) {
            throw new AirSentBusinessException("Error: data object is null");
        }
	try {
	    deliveryDO = d;
	    this.update = update;
	    this.customer = new CustomerImpl(deliveryDO.getCustomerID());
	} catch (Exception e) {
	    throw new AirSentBusinessException(e);
        }
    }
    
    /**
     * Constructs DeliveryImpl object
     *
     * 
     * @exception if an error occurs
     */
    DeliveryImpl(Update update, String id) 
        throws AirSentException, AirSentBusinessException {
        if (id == null) {
            throw new AirSentBusinessException("Error: data object is null");
        }
        try {
            deliveryDO = DeliveryDO.createExisting(id);
            this.update = update;
	    this.customer = new CustomerImpl(deliveryDO.getCustomerID());
        } catch (Exception e) {
            throw new AirSentBusinessException(e);
        }
    }
 
    /**
     * Gets the handle.
     *
     * @return handle
     * @exception if an error occurs
     */
    public String getHandle() 
        throws AirSentException {
	try {
	    return deliveryDO.getHandle();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Gets pick up address for the order.
     *
     * @return pick up address
     * @exception if an error occurs
     */
    public Address getPickUp()
        throws AirSentException {
        if (pickUpAddress == null) {
            try {
                AddressManager af = HomeManagerImpl.getInstance().getAddressManager();
                pickUpAddress = af.create(deliveryDO.getPickup());
            } catch (Exception e) {
                throw new AirSentException(e);
            } 
        } 
        return pickUpAddress;
    }
    
    /**
     * Gets the drop off address for the order
     *
     * @return drop off address
     * @exception if an error occurs
     */
    public Address getDropOff() 
        throws AirSentException {
        if (dropOffAddress == null) {
            try {
                AddressManager af = HomeManagerImpl.getInstance().getAddressManager();
                dropOffAddress = af.create(deliveryDO.getDropoff());
            } catch (Exception e) {
                throw new AirSentException(e);
            } 
        } 
        return dropOffAddress;
    } 

    /**
     * gets the customer
     * 
     * @return customer
     * @exception if an error occurs
     */
    public Customer getCustomer() 
        throws AirSentException {
        if (customer == null) {
            throw new AirSentException("customer is null");
        }
        return customer;
    } 
    
    /*
     *
     */
    private void setCustomer(Customer c) 
        throws AirSentException, AirSentBusinessException {
        try {
            deliveryDO.setCustomerID(CustomerDO.createExisting(c.getHandle()));
            customer = c;
        } catch (DataObjectException doe) {
            // object can't be found
            throw new AirSentBusinessException(doe);
        } catch (Exception e) {
            throw new AirSentException(e);
        }
    }

    /**
     * Gets the messenger.
     *
     * @return messenger
     * @exception if an error occurs
     */
    public Messenger getMessenger()
        throws AirSentException {
	try {
	    if (messenger == null) {
                if (deliveryDO.getMessengerID() != null) {
                    MessengerManager mf = HomeManagerImpl.getInstance().getMessengerManager();
                    messenger = mf.create(deliveryDO.getMessengerID().getHandle());
                }
	    } 
            return messenger;
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    }

    /**
     * Sets the messenger.
     *
     * @exception if an error occurs
     * @exception if a business rule is violated
     */
    public void setMessenger(Messenger m) 
        throws AirSentException, AirSentBusinessException {
	try {
	    deliveryDO.setMessengerID(MessengerDO.createExisting(m.getHandle()));
            messenger = m;
        } catch (DataObjectException doe) {
            // object can't be found
            throw new AirSentBusinessException(doe);
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * gets the fragile flag
     *
     * @return true if fragile
     * @exception if an error occurs
     */
    public boolean isFragile() 
        throws AirSentException {
	try {
	    return deliveryDO.getFragile();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * sets the fragile flag
     *
     * @param fragile true if fragile
     * @exception if an error occurs
     */
    public void setIsFragile(boolean fragile) 
        throws AirSentException {
	try {
	    this.deliveryDO.setFragile(fragile);
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * gets the urgent flag
     * 
     * @return true if urgent
     * @exception if an error occurs
     */
    public boolean isUrgent() 
        throws AirSentException {
	try {
	    return deliveryDO.getUrgent();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * sets the urgent flag
     * 
     * @param urgent true if urgent
     * @exception if an error occurs
     */
    public void setIsUrgent(boolean urgent) 
        throws AirSentException {
	try {
	    deliveryDO.setUrgent(urgent);
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * gets the size description
     * 
     * @return size
     * @exception if an error occurs
     */
    public String getSize() 
        throws AirSentException {
	try {
	    return deliveryDO.getSize();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * sets the size for the order
     *
     * @param size size description
     * @exception if an error occurs
     */
    public void setSize(String size) 
        throws AirSentException, AirSentBusinessException {
        if (size.length() > Delivery.MAX_DESC) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
	try {
	    deliveryDO.setSize(size);
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * gets the descriton of the order
     *
     * @return descriotn
     * @exception if an error occurs
     */
    public String getDescription() 
        throws AirSentException {
	try {
	    return deliveryDO.getDescription();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * sets the order description
     * 
     * @param description order description
     * @exception if an error occurs
     * @exception if the description is invalid
     */
    public void setDescription(String description) 
	    throws AirSentException, AirSentBusinessException {
        if (description.length() > Delivery.MAX_DESC) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
	try {
	    deliveryDO.setDescription(description);
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * saves the order
     *
     * @exception if an error occurs
     */
    public void save() 
        throws AirSentException {
	try {
            deliveryDO.commit();
	} catch (Exception e) {
            e.printStackTrace();
	    throw new AirSentException(e);
	} 
	update.setUpdateState(System.currentTimeMillis());
    } 

    /**
     * deletes the order
     *
     * @exception if an error occurs
     */
    public void delete() 
        throws AirSentException {
	try {
	    deliveryDO.delete();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
	update.setUpdateState(System.currentTimeMillis());
    } 

    /**
     * returns the pick up time
     *
     * @return pick up time 
     * @exception if an error occurs
     */
    public String getPickedUpTime() 
        throws AirSentException {
	long time = 0;
	try {
	    if ((time = deliveryDO.getPickedUpTime()) == 0) {
		return "-";
	    } else {
		return formatTime(time);
	    } 
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * returns the drop off time
     * 
     * @return drop off time  
     * @exception if an error occurs
     */
    public String getDroppedOffTime() 
        throws AirSentException {
	long time = 0;
	try {
	    if ((time = deliveryDO.getDroppedOffTime()) == 0) {
		return "-";
	    } else {
		return formatTime(time);
	    } 
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Chnages the order to be 'picked up'
     *
     * @exception if an error occurs
     */
    public void pickUp() 
        throws AirSentException {
	try {
	    deliveryDO.setIsPickedUp(true);
	    deliveryDO.setPickedUpTime(System.currentTimeMillis());
	    save();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Changes the order to be in the 'drop off' state
     *
     * @exception if an error occurs
     */
    public void dropOff() 
        throws AirSentException {
	try {
	    deliveryDO.setIsDroppedOff(true);
	    deliveryDO.setDroppedOffTime(System.currentTimeMillis());
	    save();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * returns the pick up statues
     *
     * @return true if order has been pickup
     * @exception if an error occurs
     */
    public boolean isPickedUp() 
        throws AirSentException {
	try {
	    return deliveryDO.getIsPickedUp();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * returns the drop off status
     * 
     * @return true if the order has been dropped off
     * @exception if an error occurs
     */
    public boolean isDroppedOff() 
        throws AirSentException {
	try {
	    return deliveryDO.getIsDroppedOff();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 


    /*
     * Get invoice number.
     */
    public int getInvoice() 
        throws AirSentException {
	try {
	    return deliveryDO.getInvoice();
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /*
     * Set invoice number.
     */
    public void setInvoice(int invoice) 
        throws AirSentException {
	try {
	    deliveryDO.setInvoice(invoice);
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /*
     * Format the time for the day.
     *
     * @param long mSeconds
     * @return String
     */
    private String formatTime(long time) {
	return new Date(time).toGMTString();
    } 
}

