/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: DeliveryManagerImpl.java,v 1.1 2004/05/19 18:15:19
 */

package com.lutris.airsent.business.delivery;


import com.lutris.airsent.spec.AirSentException;

import com.lutris.airsent.business.*;

import com.lutris.airsent.business.delivery.*;
import com.lutris.airsent.spec.delivery.DeliveryManager;

import com.lutris.airsent.spec.delivery.*;
import com.lutris.airsent.data.delivery.*;
import com.lutris.airsent.spec.address.Address;
import com.lutris.airsent.spec.messenger.Messenger;
import com.lutris.airsent.data.person.MessengerDO;
import com.lutris.airsent.spec.customer.Customer;

import com.lutris.airsent.data.person.CustomerDO;
import com.lutris.appserver.server.sql.*;
import com.lutris.dods.builder.generator.query.*;
import java.sql.Date;

/**
 * Implementation of the delivery manger.
 */
public class DeliveryManagerImpl implements DeliveryManager {

    private Update update = null;
    
    /**
     * creates the Manager
     */
    public DeliveryManagerImpl() {
	this.update = new UpdateImpl(System.currentTimeMillis());
    }

    /**
     * Gets the last update time.
     * 
     * @param browserState
     * @param wait
     * @return update time
     * @exception if an error occurs
     */
    public long getUpdate(long browserState, long wait) 
        throws AirSentBusinessException {
	try {
	    return update.needUpdate(browserState, wait);
	} catch (Exception e) {
	    throw new AirSentBusinessException(e);
	} 
    } 

    /**
     * Create a new delivery.
     *
     * @param customer customer associated with the order
     * @return delivey object
     * @exception if an error occurs
     */
    public Delivery create(Customer customer) 
        throws AirSentBusinessException {
	try {
            return new DeliveryImpl(update, customer);
	} catch (Exception e) {
	    throw new AirSentBusinessException(e);
	} 
    } 

    /**
     * Create a delivery based on an order.
     *
     * @return delivey object
     * @exception if an error occurs
     * @exception if business rules are violated
     */
    public Delivery create(Customer customer, OrderForm orderForm) 
        throws AirSentException, AirSentBusinessException {
	try {
            Delivery d;
            if (orderForm.getOrderId() == null) {
                d = new DeliveryImpl(update, customer);
            } else {
                d = new DeliveryImpl(update, orderForm.getOrderId());
            }
            d.setIsFragile(orderForm.getFragile());
            d.setIsUrgent(orderForm.getUrgent());
            d.setSize(orderForm.getSize());
            d.setDescription(orderForm.getDescription());

            Address pua = d.getPickUp();
            pua.setName(orderForm.getPickUpName());
            pua.setStreet1(orderForm.getPickUpAddress1());
            pua.setLocalNumber(orderForm.getPickUpPhone());
            pua.setDirections(orderForm.getPickUpDirections());

            Address doa = d.getDropOff();
            doa.setName(orderForm.getDropOffName());
            doa.setStreet1(orderForm.getDropOffAddress1());
            doa.setLocalNumber(orderForm.getDropOffPhone());
            doa.setDirections(orderForm.getDropOffDirections());
            return d;
	} catch (AirSentBusinessException be) {
	    throw be;
	} catch (Exception e) {
	    throw new AirSentException(e);
	} 
    } 

    /**
     * Find deliver by database id
     *
     * @param handle database id
     * @return delivery
     * @exception if an error occurs
     */
    public Delivery findByHandle(String handle)
            throws AirSentBusinessException {
        try {
            DeliveryQuery query = new DeliveryQuery();
            query.requireUniqueInstance();
            query.setQueryOId(new ObjectId(handle));
            DeliveryDO theDeliveryDO = query.getNextDO();
            return new DeliveryImpl(update, theDeliveryDO);
        } catch (Exception e) {
            throw new AirSentBusinessException(e);
        }
    }

    /**
     * Find deliveries associated with a given customer
     *
     * @param customer the customer
     * @return delivery
     * @exception if an error occurs
     */
    public Delivery[] findByCustomer(Customer customer)
        throws AirSentBusinessException {

        Delivery[] theDeliveryArray = null;
        try {
            DeliveryQuery query = new DeliveryQuery();
            QueryBuilder qb = query.getQueryBuilder();
            qb.addEndClause(" order by oid");
            query.setQueryCustomerID(CustomerDO.createExisting(customer.getHandle()));
            DeliveryDO[] DOarray = query.getDOArray();
            theDeliveryArray = new Delivery[DOarray.length];
            for (int i = 0; i < DOarray.length; i++) {
                theDeliveryArray[i] = new DeliveryImpl(update, DOarray[i]);
            }
        } catch (Exception e) {
            throw new AirSentBusinessException(e);
        }
        return theDeliveryArray;
    }

    /**
     * Find all the deliveries.
     *
     * @return delivery objects
     * @exception if an error occurs
     */
    public Delivery[] findAll() 
        throws AirSentBusinessException {
	Delivery[] theDeliveryArray = null;
	try {
	    DeliveryQuery query = new DeliveryQuery();
            QueryBuilder qb = query.getQueryBuilder();
            qb.addEndClause(" order by oid");
	    DeliveryDO[]  DOarray = query.getDOArray();
	    theDeliveryArray = new Delivery[DOarray.length];
	    for (int i = 0; i < DOarray.length; i++) {
		theDeliveryArray[i] = new DeliveryImpl(update, DOarray[i]);
	    } 
	} catch (Exception e) {
	    throw new AirSentBusinessException(e);
	} 
	return theDeliveryArray;
    } 


    /**
     * Find the specified number the deliveries. 
     *
     * @param subset desired number of deliveries 
     * @return delivery objects
     * @exception if an error occurs
     */
    public Delivery[] findSubset(int subset) 
        throws AirSentBusinessException {
	Delivery[] theDeliveryArray = null;
	try {
	    DeliveryQuery query = new DeliveryQuery();
            QueryBuilder qb = query.getQueryBuilder();
            qb.addEndClause(" order by oid");
	    DeliveryDO[]  DOarray = query.getDOArray();
	    int count = 0;
	    if(DOarray.length <= subset) {
		count  = DOarray.length;
	    } else {
		count = subset;
	    }
	    theDeliveryArray = new Delivery[count];
	    for (int i = 0; i < count; i++) {
		theDeliveryArray[i] = new DeliveryImpl(update, DOarray[i]);
	    } 
	} catch (Exception e) {
	    throw new AirSentBusinessException(e);
	} 
	return theDeliveryArray;
    } 

    /**
     * find all deliveries associated with a given messenger
     *
     * @param messenger messenger
     * @return delivery objects
     * @exception if an error occurs
     */
    public Delivery[] findByMessenger(Messenger messenger) 
        throws AirSentBusinessException {
	Delivery[] theDeliveryArray = null;
	try {
	    DeliveryQuery query = new DeliveryQuery();
	    query.setQueryMessengerID(MessengerDO.createExisting(messenger.getHandle()));
	    //SV query.openParen();
	    //SV query.setQueryIsPickedUp(false);
	    //SV query.or();
	    //SV query.setQueryIsDroppedOff(false);
	    //SV query.closeParen();
	    DeliveryDO[] DOarray = query.getDOArray();
	    int len=0;
	    for (int i = 0; i < DOarray.length; i++) {
		      if ((!DOarray[i].getIsPickedUp()) || (!DOarray[i].getIsDroppedOff())){
		        len=len+1;
		      }
	    }
	    theDeliveryArray = new Delivery[len];
	    int j=0;
	    for (int i = 0; i < DOarray.length; i++) {
		      if ((!DOarray[i].getIsPickedUp()) || (!DOarray[i].getIsDroppedOff())){
		        theDeliveryArray[j] = new DeliveryImpl(update, DOarray[i]);
		        j=j+1;
		      }
	    } 
	} catch (Exception e) {
	  e.printStackTrace();
	    throw new AirSentBusinessException(e);
	} 
	return theDeliveryArray;
    } 

    // -----------------------------------------------------------------
    // not currenty used
    // -----------------------------------------------------------------

    /**
     * find all delivery objects not delivered
     *
     * @return delivery objects
     * @exception if an error occurs
     */
    public Delivery[] findUndelivered() 
        throws AirSentBusinessException {
	Delivery[] theDeliveryArray = null;
	try {
	    DeliveryQuery query = new DeliveryQuery();
	    query.setQueryIsPickedUp(false);
	    query.or();
	    query.setQueryIsDroppedOff(false);
	    DeliveryDO[] DOarray = query.getDOArray();
	    theDeliveryArray = new Delivery[DOarray.length];
	    for (int i = 0; i < DOarray.length; i++) {
		theDeliveryArray[i] = new DeliveryImpl(update, DOarray[i]);
	    } 
	} catch (Exception e) {
	    throw new AirSentBusinessException(e);
	} 
	return theDeliveryArray;
    } 

    /**
     * Find deliveries by invoice
     *
     * @param invoice invoice id
     * @return delivery
     * @exception if an error occurs
     */
    public Delivery findByInvoice(int invoice) 
        throws AirSentBusinessException {
	Delivery theDelivery = null;
	try {
	    DeliveryQuery query = new DeliveryQuery();
	    query.requireUniqueInstance();
	    query.setQueryInvoice(invoice);
	    DeliveryDO theDeliveryDO = query.getNextDO();
	    theDelivery = new DeliveryImpl(update, theDeliveryDO);
	    return theDelivery;
	} catch (Exception e) {
	    throw new AirSentBusinessException(e);
	} 
    } 

    /**
     * Find all the deliveries after a given time.
     *
     * @param data time 
     * @return delivery objects
     * @exception if an error occurs
     */
    public Delivery[] findAllAfter(long date) 
        throws AirSentBusinessException {
	Delivery[] theDeliveryArray = null;
	Date searchDate = new Date(date);
	try {
	    DeliveryQuery query = new DeliveryQuery();
	    QueryBuilder  builder = query.getQueryBuilder();
	    builder.addWhere(DeliveryDO.PickedUpTime, searchDate, QueryBuilder.GREATER_THAN);
	    builder.addWhereOr();
	    builder.addWhere(DeliveryDO.DroppedOffTime, searchDate, QueryBuilder.GREATER_THAN);
	    DeliveryDO[] DOarray = query.getDOArray();
	    theDeliveryArray = new Delivery[DOarray.length];
	    for (int i = 0; i < DOarray.length; i++) {
		theDeliveryArray[i] = new DeliveryImpl(update, DOarray[i]);
	    } 
	} catch (Exception e) {
	    throw new AirSentBusinessException(e);
	} 
	return theDeliveryArray;
    } 
}

