/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: MessengerImpl.java,v 1.1 2004/08/16 09:33:11 slobodan Exp $
 */

package com.lutris.airsent.business.messenger;


import com.lutris.airsent.spec.AirSentException;
import com.lutris.airsent.business.*;


import com.lutris.airsent.spec.messenger.Messenger;
import com.lutris.airsent.spec.customer.Customer;
import com.lutris.airsent.spec.address.Address;
import com.lutris.airsent.business.address.AddressManager;

import com.lutris.airsent.business.address.*;
import com.lutris.airsent.business.address.AddressImpl;

import com.lutris.airsent.data.person.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.dods.builder.generator.query.DataObjectException;

/**
 * Messenger implementation.
 */
public class MessengerImpl implements Messenger, java.io.Serializable {
    MessengerDO messengerDO = null;
    PersonDO personDO = null;
    
    /**
     * Constructs empty Messenger
     *
     * @exception if an error occurs
     */
    MessengerImpl() 
        throws AirSentException {
	try {
	    messengerDO = MessengerDO.createVirgin();
	    personDO = PersonDO.createVirgin();
            AddressManager af = HomeManagerImpl.getInstance().getAddressManager();
            AddressImpl addr = af.create();
            personDO.setAddressID(addr.getDataObject());
	    messengerDO.setPersonID(personDO);
	} catch (Exception ex) {
	    throw new AirSentException("Error creating Messenger",  ex);
	} 
    }

    /**
     * Constructs messenger using data object.
     *
     * @exception if an error occurs
     */
    MessengerImpl(MessengerDO messenger) 
	    throws AirSentException {
	try {
	    this.messengerDO = messenger;
	    this.personDO = messengerDO.getPersonID();
	} catch (Exception ex) {
	    throw new AirSentException("Error creating Messenger", ex);
	} 
    }
    
    /**
     * Constructs messenger using database handle
     *
     * @exception if an error occurs
     */
    MessengerImpl(String id) 
        throws AirSentException {
	try {
	    this.messengerDO = messengerDO.createExisting(id);
	    this.personDO = messengerDO.getPersonID();
	} catch (Exception ex) {
	    throw new AirSentException("Error creating Messenger", ex);
	} 
    }

    /**
     * Get messengers database handle
     *
     * @return database id
     * @exception if an error occurs
     */
    public String getHandle() 
        throws AirSentException {
	try {
	    return messengerDO.getHandle();
	} catch (Exception ex) {
	    throw new AirSentException("Error getting messenger's handle", ex);
	} 
    } 

    /**
     * Get first name.
     *
     * @return first name
     * @exception if an error occurs
     */
    public String getFirstName() 
        throws AirSentException {
	try {
	    return personDO.getFirstName();
	} catch (Exception ex) {
	    throw new AirSentException("Error getting messenger's firstname", ex);
	} 
    } 

    /**
     * Set first name.
     *
     * @param first first name
     * @exception if an error occurs
     * @exception if business rules are violated
     */
    public void setFirstName(String first) 
        throws AirSentException, AirSentBusinessException {
        if (first.length() > Customer.MAX_FIRST_NAME) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
	try {
	    this.personDO.setFirstName(first);
	} catch (Exception ex) {
	    throw new AirSentException("Error setting messenger's name", ex);
	} 
    } 

    /**
     * Get email.
     * @return email address
     * @exception if an error occurs
     */
    public String getEmail() 
        throws AirSentException {
	try {
	    return personDO.getEmail();
	} catch (Exception ex) {
	    throw new AirSentException("Error getting messenger's firstname", ex);
	} 
    } 

    /**
     * Set Email.
     * 
     * @param email
     * @exception if an error occurs
     * @exception if business rules are violated

     */
    public void setEmail(String email) 
        throws AirSentException, AirSentBusinessException {
        if (email.length() > Customer.MAX_EMAIL) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
	try {
	    this.personDO.setEmail(email);
	} catch (Exception ex) {
	    throw new AirSentException("Error setting messenger's name", ex);
	} 
    } 

    /**
     * Get last name.
     *
     * @return last name
     * @exception if an error occurs
     */
    public String getLastName() 
        throws AirSentException {
	try {
	    return personDO.getLastName();
	} catch (Exception ex) {
	    throw new AirSentException("Error getting messenger's lastname", ex);
	} 
    } 

    /**
     * Set last name.
     * 
     * @param last
     * @exception if an error occurs
     * @exception if business rules are violated
     */
    public void setLastName(String last) 
        throws AirSentException, AirSentBusinessException {
        if (last.length() > Customer.MAX_LAST_NAME) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
	try {
	    this.personDO.setLastName(last);
	} catch (Exception ex) {
	    throw new AirSentException("Error setting messenger's last name", ex);
	} 
    } 

    /**
     * Gets the address.
     * @exception if an error occurs
     *
     * @return address
     * @exception if an error occurs
     */
    public Address getAddress() 
        throws AirSentException {
	try {
            AddressManager af = HomeManagerImpl.getInstance().getAddressManager();
            return af.create(personDO.getAddressID());
	} catch (Exception ex) {
	    throw new AirSentBusinessException("Error getting messenger's address", ex);
	} 
    } 


    /**
     * Get badge.
     *
     * @return badge
     * @exception if an error occurs
     */
    public String getBadge() 
        throws AirSentException {
	try {
	    return messengerDO.getBadge();
	} catch (Exception ex) {
	    throw new AirSentBusinessException("Error getting messenger's badge", ex);
	} 
    } 

    /**
     *
     * @exception if an error occurs
     * @exception if business rules are violated
     */
    public void setBadge(String badge) 
        throws AirSentException, AirSentBusinessException {
        if (badge.length() > Messenger.MAX_BADGE) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
	try {
	    this.messengerDO.setBadge(badge);
	} catch (Exception ex) {
	    throw new AirSentException("Error setting messenger's badge", ex);
	} 
    } 

    /**
     * Get password.
     *
     * @return password
     * @exception if an error occurs
     */
    public String getPassword() 
        throws AirSentException {
	try {
	    return messengerDO.getPassword();
	} catch (Exception ex) {
	    throw new AirSentException("Error getting customers's lastname", ex);
	} 
    } 

    /**
     * Set password.
     *
     * @exception if an error occurs
     * @exception if business rules are violated
     */
    public void setPassword(String password) 
        throws AirSentException, AirSentBusinessException {
        if (password.length() > Messenger.MAX_PASSWORD) {
            throw new AirSentBusinessException("Exceeds maximum size");
        }
	try {
	    this.messengerDO.setPassword(password);
	} catch (Exception ex) {
	    throw new AirSentException("Error setting customers's last name", ex);
	} 
    } 

    /**
     * Gets the geocode. NOT currently used.
     *
     * @return code
     * @exception if an error occurs
     */
    public String getGeocode() 
        throws AirSentException {
	try {
	    return messengerDO.getGeocode();
	} catch (Exception ex) {
	    throw new AirSentException("Error getting messenger's geocode", ex);
	} 
    } 

    /**
     * Saves the messenger.
     *
     * @exception if an error occurs
     */
    public void save() 
        throws AirSentException {
	try {
	    this.messengerDO.commit();
	} catch (Exception ex) {
	    throw new AirSentException("Error saving address", ex);
	} 
    } 

    /**
     * Deletes the messenger.
     *
     * @exception if an error occurs
     */
    public void delete() 
        throws AirSentException {
	try {
	    this.messengerDO.delete();
	} catch (Exception ex) {
	    throw new AirSentException("Error deleting address", ex);
	} 
    } 
}

