/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: BasePO.java,v 1.1 2004/08/16 09:33:16 slobodan Exp $
 */

package com.lutris.airsent.presentation;

import com.lutris.airsent.spec.address.*;
import com.lutris.airsent.spec.address.*;

import com.lutris.airsent.*;

import com.lutris.appserver.server.StandardAppUtil;
import org.enhydra.xml.xmlc.XMLObject;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*; 
import com.lutris.appserver.server.Enhydra;
//import com.lutris.xml.xmlc.*;
//import com.lutris.xml.xmlc.html.*;
import com.lutris.logging.*;
import com.lutris.util.KeywordValueException;
import com.lutris.appserver.server.user.User;
import org.w3c.dom.*;
import org.w3c.dom.html.HTMLElement;
import java.lang.reflect.*;
import java.util.*;



/**
 * This is the parent Presentaion object.  All presentation objects
 * should extend this class.
 * 
 * The run method looks for an event parameter and then calls
 * handle<EventName>.  If the "event" Parameter is not defined then
 * the handleDefault() method is called in your child class.
 * 
 * @author
 * @version
 */
public abstract class BasePO implements HttpPresentation {
    private static String EVENT = "event";
    private static String STANDARD_METHOD_PREFIX = "handle";

    /**
     * This is the procedure that is called if there is no "event"
     * HTTP parameter found. It must be overriden by the subclass to
     * do default processing or error checking/handling.
     * 
     * @return String The String representation of the HTML or (other format)
     * of the document to be displayed. This method would need to be changed
     * if you wanted to return binary data as well. It returns a String
     * for simplicity right now.
     */
    public abstract XMLObject handleDefault() 
        throws HttpPresentationException;

    /**
     * This method should be implemented in the subclass so that it returns
     * the authorization level necessary to access the po.
     */
    abstract protected int getRequiredAuthLevel();

    /**
     * Saved input and output context, and session data
     */
    protected HttpPresentationComms myComms = null;
    protected AirSentSessionData    mySessionData = null;

    /**
     * Gets HttpPresentation object
     * 
     * @return The saved comms objects
     * to whichever subclass needs it
     */
    public HttpPresentationComms getComms() {
	return this.myComms;
    } 

    /**
     * Gets the session data
     * 
     * @return session data
     */
    public AirSentSessionData getSessionData() {
	return this.mySessionData;
    } 

    /**
     * This implements the run method in HttpPresentation.
     * 
     * @param HttpPresentationComms
     * @exception Exception
     */
    public void run(HttpPresentationComms comms) throws Exception {
	
	// Reroute based on content.
	rerouteForContent(comms);

	// Initialize new or get the existing session data
	initSessionData(comms);

	// Check if the user can access the given page.
	checkAuthLevel();

	try {
	    
	    // Handle the incoming event request
	    handleEvent(comms);
	} catch (Exception e) {
	    System.err.println("EXCEPTION: " + e);
	    throw new Exception("Exception in run " + e);
	} 
    } 

    /**
     * Method that determines which content to output based on
     * the accept type.
     * Current types:
     * text/html
     * text/xml
     * text/vnd.wap.wml
     * 
     * 
     */
    protected void rerouteForContent(HttpPresentationComms comms) 
        throws AirSentPresentationException {
	DeviceUtils.rerouteForContent(comms);
    } 

    /**
     * Method to get or create the AirSentSessionData object from the user session
     * This object is saved in the AirSentPresentation object
     * 
     * @param HttpPresentationComms
     * @exception Exception
     */
    protected void initSessionData(HttpPresentationComms comms) 
        throws AirSentPresentationException {

	this.myComms = comms;
	try {
	    Object obj = comms.sessionData.get(AirSentSessionData.SESSION_KEY);

	    // If we found the session data, save it in a private data member
	    if (obj != null) {
		this.mySessionData = (AirSentSessionData) obj;
	    } else {
		// If no session data was found, create a new session data instance
		this.mySessionData = new AirSentSessionData();
		comms.sessionData.set(AirSentSessionData.SESSION_KEY, this.mySessionData);
	    } 
	} catch (KeywordValueException ex) {
	    throw new AirSentPresentationException("Trouble initializing user", ex);
	} 
    } 

    /**
     * Checks the session data for a Student, if not there then redirects to the login page
     * return the current authorization level (set during login)
     * 
     * @return an int equal to the current authorization level.
     */
    protected int getCurrentAuthLevel() 
        throws ClientPageRedirectException, AirSentPresentationException {
	int accessLevel = 0;

	try {
	    accessLevel = getSessionData().getUserAuth();
	} catch (Exception ex) {
	    throw new AirSentPresentationException("Trouble getting current authorization level", 
						   ex);
	} 

	return accessLevel;
    } 

    /**
     * Checks the session data to see if the user has the authorization to
     * access the given page.  Authorization levels include:
     * UNAUTH_USER         (0)  --   login not required.
     * CUSTOMER_USER        (1)  --   login as customer required.
     * MESSENGER_USER     (2)  --   login as messenger required.
     * ADMIN_USER  (3)  --   login as administrator required.
     * 
     * SIDE EFFECTS:  redirects to login page if user not authorized to access page.
     */
    protected void checkAuthLevel() 
        throws ClientPageRedirectException, AirSentPresentationException {
	int currentAuth = getCurrentAuthLevel();

	try {
//We need  to allow AirSent_pres to be functional , so if the requested url is /AirSent_pres/ we allow user to acsses to page	 
	
	String uri = myComms.request.getRequestURI();
      	boolean is= uri.startsWith("/AirSent_pres");

if(!is){

   if (currentAuth < getRequiredAuthLevel()) {
		if (currentAuth > AirSentConstants.CUSTOMER_USER) {
		    throw new ClientPageRedirectException(AirSentConstants.ADMIN_LOGIN_PAGE);
		} 

		throw new ClientPageRedirectException("/" + AirSentConstants.HTML_PAGE);
	    } 
	
   }
} catch (Exception ex) {
	    throw new AirSentPresentationException("Trouble checking for user login status", 
						   ex);
	} 
    } 

    /**
     * Method to call the proper method for the incoming event
     * 
     * @param HttpPresentationComms
     * @exception Exception
     */
    public void handleEvent(HttpPresentationComms comms) throws Exception {
	String    event = comms.request.getParameter(EVENT);
	XMLObject returnDoc = null;
	try {
	    if (event == null || event.length() == 0) {
		returnDoc = handleDefault();
	    } else {
		returnDoc = getPage(event);
	    } 
	    comms.response.writeDOM(returnDoc);
	} catch (Exception e) {
	    throw new Exception("Exception writing dom:" +  e);
	}
    } 

    /**
     * this logs out a user from the session.  it first grabs
     * an instance of the sessionData associated with the request
     * and then sets the user to null in the session data.
     * 
     * @param HttpServletRequest request - this is the request that
     * the session data is extracted from.
     * 
     * @exception AirSentPresentationException when an error occurs or if
     * the session data for the request is null.
     */
    public XMLObject handleLogout() throws AirSentPresentationException {
	try {
	    mySessionData = null;
            SessionManager sessionManager = myComms.session.getSessionManager();
            sessionManager.deleteSession(myComms.session);
	    throw new ClientPageRedirectException(AirSentConstants.HTML_PAGE);
	} catch (Exception e) {
	    throw new AirSentPresentationException("Trouble logging out user", 
						   e);
	} 
    } 

    /**
     * If an event parameter is defined then this invokes the method that
     * handles that event.
     * 
     * @param event, the incoming event name
     * @exception Exception
     */
    public XMLObject getPage(String event) throws Exception {
	try {
	    Method    method = this.getClass().getMethod(toMethodName(event), 
                                                         null);
	    XMLObject thePage = (XMLObject) method.invoke(this, null);

	    return thePage;
	} catch (InvocationTargetException ex) {

	    // Rethrow the originating exception if as it should be propagated as is
	    // It could be a page redirect exception, etc.
	    if (ex.getTargetException() instanceof Exception) {
		throw (Exception) ex.getTargetException();
	    } else if (ex.getTargetException() instanceof Error) {
		throw (Error) ex.getTargetException();
	    } else {
		throw ex;
	    } 
	} catch (NoSuchMethodException ex) {

	    // The method to handle the event does not exist.
	    throw new AirSentPresentationException("NO EVENT HANDLER FOUND FOR EVENT: " 
						   + event, ex);
	} catch (IllegalAccessException ex) {

	    // The method to handle the event does not exist.
	    throw new AirSentPresentationException("ILLEGAL ACCESS TO EVENT HANDLER (is it public?): " 
						   + event, ex);
	} 
    } 

    /**
     * Gets the URI prefix. Special because some devices incorrectly 
     * handle this.
     * 
     */
    protected String getURIPrefix() throws AirSentPresentationException {
	try {
	    String scheme = myComms.request.getScheme();
	    String serverName = myComms.request.getServerName();
	    int    serverPort = myComms.request.getServerPort();
	    String applicationPath = myComms.request.getApplicationPath();

	    // don't include default port in the full path
	    if (serverPort == 80) {
		return scheme + "://" + serverName + applicationPath;
	    } else {
		return scheme + "://" + serverName + ":" + serverPort 
                    + applicationPath;
	    } 
	} catch (Exception ex) {
	    throw new AirSentPresentationException("Can't get Server Root Name " 
						   + "Config file.", ex);
	} 
    } 

    /**
     * This sets the first letter of the event parameter value in order
     * to adhere to Java method naming conventions.
     * 
     * @param String event the incoming name of the event
     * @return String the properly capitalized name
     */
    private String toMethodName(String event) {
	StringBuffer methodName = new StringBuffer(STANDARD_METHOD_PREFIX);

	methodName.append(Character.toUpperCase(event.charAt(0)));

	if (event.length() > 1) {
	    methodName.append(event.substring(1));
	} 

	return methodName.toString();
    } 

    /**
     * Returns the application object associated with the
     * current request.
     * 
     * @return the application object.
     */
    public AirSent getApplication() {
	return (AirSent) Enhydra.getApplication();
    } 

    /**
     * Method to write a debugging message to the debug log
     * channel when the DEBUG flag is turned on
     * 
     * @param msg The message to write to the DEBUG log channel
     */
    public static void writeDebugMsg(String msg) {
	Enhydra.getLogChannel().write(Logger.DEBUG, msg);
    } 


    /**
     * Returns true if the given string is null, empty, or contains
     * only white space.
     */
    protected static boolean isNullField(String field) {
	if (field == null) {
	    return true;
	} 

	if (field.trim().equals("")) {
	    return true;
	} 

	return false;
    } 


    /**
     * Returns true if the given string is null, empty, or contains
     * only white space.
     */
    protected static boolean checkField(String field, int size) {
	if (field == null || field.equals("")) {
	    return false;
	} 
        if (field.length() <= size) {
            return true;
        }
	return false;
    } 


    /**
     *
     */
    public static void dumpHeaders(HttpPresentationComms comms) {
        System.out.println("DUMP HEADERS:" );
        try {
            HttpPresentationRequest req = comms.request;
            Enumeration e = req.getHeaderNames();
            while (e.hasMoreElements()) {
                String k = (String) e.nextElement();
                String p = req.getHeader(k);
                System.out.println("  NAME: " + k + "; VALUE: " + p);
            }
        } catch (Exception ex) {
            System.err.println("ERROR: " + ex);
        }
    }

    /**
     *
     */
    public static void dump(HttpPresentationComms comms) {
        System.out.println("DUMP PARAMETERS:" );
        try {
            HttpPresentationRequest req = comms.request;
            Enumeration e = req.getParameterNames();
            while (e.hasMoreElements()) {
                String k = (String) e.nextElement();
                String p = req.getParameter(k);
                System.out.println("  NAME: " + k + "; VALUE: " + p);
            }
        } catch (Exception ex) {
            System.err.println("ERROR: " + ex);
        }
    }
}




