/*
 * Copyright (c) 1999-2001 Lutris Technologies, Inc. All Rights
 * Reserved.
 * 
 * This source code file is distributed by Lutris Technologies, Inc. for
 * use only by licensed users of product(s) that include this source
 * file. Use of this source file or the software that uses it is covered
 * by the terms and conditions of the Lutris Enhydra Development License
 * Agreement included with this product.
 * 
 * This Software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the License for the specific terms
 * governing rights and limitations under the License.
 * 
 * Contributor(s):
 * 
 * $Id: DeliveryManager.java,v 1.1 2004/05/19 18:15:19
 */
package com.lutris.airsent.spec.delivery;

import com.lutris.airsent.spec.AirSentException;


import com.lutris.airsent.spec.customer.Customer;
import com.lutris.airsent.spec.messenger.Messenger;

/**
 * DeliveryFactory Interface declaration
 */
public interface DeliveryManager {

    /**
     * Create a new delivery.
     *
     * @param customer customer associated with the order
     * @return delivey object
     * @exception if an error occurs
     */
    public Delivery create(Customer customer) 
        throws AirSentException;

    /**
     * Create a delivery based on an order.
     *
     * @return delivey object
     * @exception if an error occurs
     * @exception if business rules are violated
     */
    public Delivery create(Customer customer, OrderForm orderForm) 
        throws AirSentException;

    /**
     * Gets the last update time.
     * 
     * @param browserState
     * @param wait
     * @return update time
     * @exception if an error occurs
     */
    public long getUpdate(long browserState, long wait)  
        throws AirSentException;

    /**
     * Find deliver by database id
     *
     * @param handle database id
     * @return delivery
     * @exception if an error occurs
     */
    public Delivery findByHandle(String handle) 
        throws AirSentException;

    /**
     * Find deliveries associated with a given customer
     *
     * @param customer the customer
     * @return delivery
     * @exception if an error occurs
     */
    public Delivery[] findByCustomer(Customer customer) 
        throws AirSentException;

    /**
     * Find deliveries by invoice
     *
     * @param invoice invoice id
     * @return delivery
     * @exception if an error occurs
     */
    public Delivery findByInvoice(int invoice) 
        throws AirSentException;

    /**
     * find all delivery objects not delivered
     *
     * @return delivery objects
     * @exception if an error occurs
     */
    public Delivery[] findUndelivered() 
        throws AirSentException;

    /**
     * Find all the deliveries.
     *
     * @return delivery objects
     * @exception if an error occurs
     */
    public Delivery[] findAll() 
        throws AirSentException;

    /**
     * Find all the deliveries after a given time.
     *
     * @param data time 
     * @return delivery objects
     * @exception if an error occurs
     */
    public Delivery[] findAllAfter(long date) 
        throws AirSentException;

    /**
     * Find the specified number the deliveries. 
     *
     * @param subset desired number of deliveries 
     * @return delivery objects
     * @exception if an error occurs
     */
    public Delivery[] findSubset(int subset) 
        throws AirSentException;

    /**
     * find all deliveries associated with a given messenger
     *
     * @param messenger messenger
     * @return delivery objects
     * @exception if an error occurs
     */
    public Delivery[] findByMessenger(Messenger messenger)
        throws AirSentException;

}

