
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: LoginEventGateway.java,v 1.6 2006/03/31 09:50:32 alex Exp $
 */

package barracudaDiscRack.presentation.personMgmt;

// import disc rack specifics
import barracudaDiscRack.presentation.*;
import barracudaDiscRack.presentation.personMgmt.events.*;
import barracudaDiscRack.business.person.*;
import barracudaDiscRack.business.DiscRackBusinessException;

// import barracuda specifics
import org.barracudamvc.core.event.*;
import org.barracudamvc.core.forms.*;
import org.barracudamvc.core.forms.validators.*;
import org.barracudamvc.core.comp.*;
import org.barracudamvc.core.util.dom.*;
import org.barracudamvc.plankton.data.CollectionsUtil;

import org.apache.log4j.*;

// import dom specifics
import org.w3c.dom.*;
import org.w3c.dom.html.*;

// import java specifics
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.List;
import java.util.Iterator;

/**
 * LoginEventGateway handles the login functionality of the DiscRack app.
 *
 * @author
 * @version
 */
public class LoginEventGateway extends BaseDiscRackEventGateway {

	public LoginEventGateway() {
		//specify who's interested in what
		specifyLocalEventInterests(getLoginFactory, GetLogin.class);
		specifyLocalEventInterests(renderLoginFactory, RenderLogin.class);
		specifyLocalEventInterests(doLoginFactory, DoLogin.class);
	}

	// setup the required listener factories
	private ListenerFactory getLoginFactory = new DefaultListenerFactory() {
		public BaseEventListener getInstance() {
			return new GetLoginHandler();
		}      public String getListenerID() {
			return getID(GetLoginHandler.class);
		}
	};

	private ListenerFactory renderLoginFactory = new DefaultListenerFactory() {
		public BaseEventListener getInstance() {
			return new RenderLoginHandler();
		}     public String getListenerID() {
			return getID(RenderLoginHandler.class);
		}
	};

	private ListenerFactory doLoginFactory = new DefaultListenerFactory() {
		public BaseEventListener getInstance() {
			return new DoLoginHandler();
		}       public String getListenerID() {
			return getID(DoLoginHandler.class);
		}
	};

	// setup a logger for the class
  protected static Logger logger = Logger.getLogger(LoginEventGateway.class.getName());

	//private constants
	private static final String LOGIN_FORM = LoginEventGateway.class.getName()+".LoginForm";		//LoginForm
	private static final String LOGIN_ERR = LoginEventGateway.class.getName()+".LoginErr";		//ValidationException

	//------------------------------------------------------------
	//             Model 2 - Controller Event Handlers
	//------------------------------------------------------------
	/**
	 * GetLoginHandler - this is where we handle any GetLogin event
	 * requests. If they are already logged in, just redirect on.
	 */
	class GetLoginHandler extends DefaultBaseEventListener {
		public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
			// MUST DO FIRST - initialize things from the context
			initUsingContext( context );

			//unpack the necessary entities from the context
			DispatchQueue queue = context.getQueue();

			// if it looks like we are already logged in, try and auto-login
			// otherwise, just render the login screen
			if ( null != getUser() ) {
				if ( logger.isDebugEnabled() ) logger.debug("Adding DoLogin to Queue (Auto-login)");
				queue.addEvent(new DoLogin(context.getEvent()));
			} else {
				if ( logger.isDebugEnabled() ) logger.debug("Adding RenderLogin to Queue");
				queue.addEvent(new RenderLogin(context.getEvent()));
			}
		}
	}

	/**
	 * DoLoginHandler - this is where we handle any AttemptLogin
	 * events. Validate and then redirect accordingly.
	 */
	class DoLoginHandler extends DefaultBaseEventListener {
		public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
			// MUST DO FIRST - initialize things from the context
			initUsingContext( context );

			//unpack the necessary entities from the context
			HttpServletRequest req = context.getRequest();

			//create the login form
			ValidationException ve = null;
			if ( logger.isDebugEnabled() ) logger.debug("Creating login form");
			LoginForm lf = new LoginForm( getComms().session.getHttpSession() );
			try {
				//map/validate the form
				if ( logger.isDebugEnabled() ) logger.debug("Mapping/Validating login form");
				lf.map(req).validate(true);
			} catch ( ValidationException e ) {
				removeUserFromSession();
				if ( logger.isDebugEnabled() ) logger.debug("Validation Exception: "+e);

				if ( logger.isDebugEnabled() ) 
				  CollectionsUtil.printStackTrace(e.getExceptionList(), 0, logger, null);

				ve = e;
			}

			//store a copy of the form and any errors in the queue
			if ( logger.isDebugEnabled() ) logger.debug("Saving form, errors");
			context.putState(LOGIN_FORM, lf);
			context.putState(LOGIN_ERR, ve);

			//redirect to the Main screen
			if ( ve==null ) {
				if ( logger.isDebugEnabled() ) logger.debug("Redirecting to " + BasePO.DISC_EVENT_CATALOG_PAGE);
				throw new ClientSideRedirectException(BasePO.DISC_EVENT_CATALOG_PAGE);
			} else {
				if ( logger.isDebugEnabled() ) logger.debug("InterruptDispatch to GetLogin");
				throw new InterruptDispatchException(new GetLogin());
			}
		}
	}

	//------------------------------------------------------------
	//                Model 2 - View Event Handlers
	//------------------------------------------------------------
	/**
	 * RenderLoginHandler -
	 */
	class RenderLoginHandler extends DefaultBaseEventListener {
		public void handleViewEvent(ViewEventContext context) throws EventException, ServletException, IOException {
			// MUST DO FIRST - initialize things from the context
			initUsingContext( context );

			//unpack the necessary entities from the context
			HttpServletRequest req = context.getRequest();

			//get the XMLC object
			LoginHTML page = (LoginHTML) getComms().xmlcFactory.create(LoginHTML.class);

			//create a template component and render it
			if ( logger.isDebugEnabled() ) logger.debug("Creating template component");
			Node node = page.getDocument().getElementById("LoginForm");
//			BTemplate templateComp = new BTemplate(new LoginModel(context), new DefaultTemplateView(node));

        TemplateView tv = new DefaultTemplateView(node);
        TemplateModel tm = new LoginModel(context);
        BTemplate templateComp = new BTemplate(tm);
        templateComp.setView(tv);

			try {
				templateComp.render(new DefaultViewContext(context));
			} catch ( RenderException re ) {
				logger.warn("Render err:"+re);
			}

			//now actually render the page (the DOMWriter will take care of actually
			//setting the content type)
			if ( logger.isDebugEnabled() ) logger.debug("Rendering document");
			  new DefaultDOMWriter().write(page, context.getResponse());
		}
	}

	//------------------------------------------------------------
	//               Template Models
	//------------------------------------------------------------
	/**
	 * LoginModel
	 */
	class LoginModel extends AbstractTemplateModel {

		LoginForm fm = null;
		ValidationException ve = null;

		//constructor (extract form, exception from context)		
		public LoginModel(EventContext ec) {
			fm = (LoginForm) ec.getState(LOGIN_FORM);
			ve = (ValidationException) ec.getState(LOGIN_ERR);
		}

		//register the model by name
		public String getName() {
			return "Login";
		}

		//provide items by key
		public Object getItem(String key) {
      ViewContext vc = getViewContext();
			if ( key.equals("Username") ) {
				return(fm!=null ? fm.getVal(LoginForm.USER, "").toString() : "");
			} else if ( key.equals("Password") ) {
				return(fm!=null ? fm.getVal(LoginForm.PASSWORD, "").toString() : "");
			} else if ( key.equals("Errors") ) {
				if ( ve==null )	return "";
				List errlist = ve.getExceptionList();
				StringBuffer sb = new StringBuffer(errlist.size()>1 ? "There were several errors:" : "");
				Iterator it = errlist.iterator();
				while ( it.hasNext() ) {
					sb.append(" "+((ValidationException) it.next()).getMessage());
				}
				return sb.toString();
			} else 
			  return super.getItem(key);
		}

	}

	//------------------------------------------------------------
	//               HTML Form Mappings, Validators
	//------------------------------------------------------------

	/**
	 * Login form - define the login form
	 */
	class LoginForm extends DefaultFormMap {
		//login form constants (these values correspond to the HTML param names)
		static final String USER = "login";
		static final String PASSWORD = "password";

		HttpSession session = null;

		public LoginForm(HttpSession isession) {
			session = isession;

			//define the elements
			if ( logger.isDebugEnabled() ) logger.debug("Defining Login form elements");
			this.defineElement(new DefaultFormElement(USER,     FormType.STRING, null, new NotNullValidator("You must enter a Login.")));
			this.defineElement(new DefaultFormElement(PASSWORD,   FormType.STRING, null, new NotNullValidator("You must enter a Password.")));

			//define a form validator
			if ( logger.isDebugEnabled() ) logger.debug("Defining Registration form validator");
			this.defineValidator(new LoginValidator(session));
		}

		/**
		 * we override this method because instead of using the default validation
		 * process (validate elements, then form), we don't want to validate at all
		 * if they are already logged in...we just assume they're valid and return.
		 */
		public FormMap validate(boolean deferExceptions) throws ValidationException {
			//see if they are already logged in. If so, just consider them valid			
			if ( logger.isDebugEnabled() ) logger.debug("Checking to see if already logged in (prevalidated)");
			Person p = getUser();
			if ( null != p ) {
				try {
					String fuser = (String)this.getVal(LoginForm.USER);
					String fpassword = (String)this.getVal(LoginForm.PASSWORD);
					if ( fuser==null && fpassword==null )	return this;	//if user/pwd = null, just use the current settings
					if ( p.getLogin().equals(fuser) &&
							 p.getPassword().equals(fpassword) ) return this;	//if they're != null, make sure they match, otherwise, we'll want to force a complete revalidate
				} catch ( DiscRackBusinessException e ) {
				}
			}

			//if not, go ahead and validate
			return super.validate(deferExceptions);
		}
	}

	/**
	 * Login validator - define a custom form validator
	 */
	class LoginValidator extends DefaultFormValidator {

		HttpSession session = null;

		public LoginValidator(HttpSession isession) {
			session = isession;
		}

		public void validateForm(FormMap imap, boolean deferExceptions) throws ValidationException {
			if ( logger.isDebugEnabled() ) logger.debug("Validating login form");

			//validate the login information
			if ( logger.isDebugEnabled() ) logger.debug("Validate the user/pwd info");
			LoginForm map = (LoginForm) imap;
			String user = map.getVal(LoginForm.USER, "").toString();
			String password = map.getVal(LoginForm.PASSWORD, "").toString();
			String errMsg = "Invalid username or password. Please check your information and try again.";
			try {
				//make sure the login exists and the passwords match
				Person person = PersonFactory.findPerson(user);
				if ( person==null )	throw new ValidationException(map.getElement(LoginForm.USER), errMsg);
				if ( !person.getPassword().equals(password) )	throw new ValidationException(map.getElement(LoginForm.PASSWORD), errMsg);

				//log them in
				setUser(person);
			} catch ( DiscRackBusinessException e ) {
				removeUserFromSession(); //just to be sure we don't get stuck in a loop
				throw new ValidationException(map.getElement(LoginForm.USER), "System error finding user: " + e.getMessage()+". Please try again. If the problem persists, contact your system administrator.", e);
			} catch ( DiscRackPresentationException e ) {
				throw new ValidationException(map.getElement(LoginForm.USER), "System error setting user: " + e.getMessage()+". Please try again. If the problem persists, contact your system administrator.", e);
			}
		}
	}
}

