/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: RegistrationEventGateway.java,v 1.6 2006/03/31 09:50:32 alex Exp $
 */
package barracudaDiscRack.presentation.personMgmt;

import java.io.*;
import java.util.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.xml.xmlc.*;

import org.barracudamvc.core.comp.*;
import org.barracudamvc.core.event.*;
import org.barracudamvc.core.event.helper.*;
import org.barracudamvc.core.forms.*;
import org.barracudamvc.core.forms.validators.*;
import org.barracudamvc.core.util.dom.*;
import org.barracudamvc.plankton.data.CollectionsUtil;

import org.apache.log4j.*;

// import disc rack specifics
import barracudaDiscRack.presentation.*;
import barracudaDiscRack.business.*;
import barracudaDiscRack.business.person.*;
import barracudaDiscRack.presentation.events.*;
import barracudaDiscRack.presentation.personMgmt.events.*;


/**
 * Event handlers (both Controller and View) for the
 * Registration screen
 *
 * <p>We handle the following Control events:
 * <ul>
 *		<li>GetRegister - uses an EventForwardingFactory to
 *			automatically fire a RenderRegister event</li>
 *		<li>DoRegister - create the registration form, map the request
 *			to the form and then validate it. If there weren't any errors,
 *			store the Person record, log the user into the session, and
 *			then fire a GetLogin event (which will cause them to be auto-
 *			logged in). Otherwise, fire a GetRegister event	(saving the
 *			form and any validation errors in the event context so that
 *			the RenderLogin can update the screen correctly.</li>
 * </ul>
 *
 * <p>We handle the following View events:
 * <ul>
 *		<li>RenderRegister - Generate the Registration screen. If the event context
 *			contains a RegistrationForm, we repopulate the screen from the form.
 *			If the context contains login errs, we will show those as well</li>
 * </ul>
 *
 * <p>We define the following Forms:
 * <ul>
 *		<li>RegistrationForm - contains 5 elements: FIRST_NAME, LAST_NAME,
 *			USER, PASSWORD, and REPASSWORD, all of which are Strings and
 *			take NotNullValidators. In addition, registration form uses
 *			RegistrationValidator, which validates the form by making sure
 *			that the passwords match and the person record is not already
 *			on file.</li>
 * </ul>
 *
 * <p>You can <a href="sm_barracudaDiscRack_registration.gif">click here to see a diagram...</a>
 */
public class RegistrationEventGateway extends BaseDiscRackEventGateway {

        //public constants
  protected static Logger logger = Logger.getLogger(RegistrationEventGateway.class.getName());

        //private constants
        private static final String REGISTRATION_FORM = RegistrationEventGateway.class.getName()+".RegistrationForm";			//RegistrationForm
        private static final String REGISTRATION_ERR = RegistrationEventGateway.class.getName()+".RegistrationErr";			//ValidationException

        //this defines the various event handlers
        private ListenerFactory getRegisterFactory =    new EventForwardingFactory(new RenderRegister());
        private ListenerFactory renderRegisterFactory =   new DefaultListenerFactory() {
                public BaseEventListener getInstance() {
                        return new RenderRegisterHandler();
                }    public String getListenerID() {
                        return getID(RenderRegisterHandler.class);
                }
        };
        private ListenerFactory doRegisterFactory =     new DefaultListenerFactory() {
                public BaseEventListener getInstance() {
                        return new DoRegisterHandler();
                }      public String getListenerID() {
                        return getID(DoRegisterHandler.class);
                }
        };

        //private vars


        //-------------------- DefaultEventGateway -------------------
        /**
         * Public constructor
         */
        public RegistrationEventGateway() {
                //specify who's interested in what
                specifyLocalEventInterests(getRegisterFactory, GetRegister.class);
                specifyLocalEventInterests(renderRegisterFactory, RenderRegister.class);
                specifyLocalEventInterests(doRegisterFactory, DoRegister.class);
        }


        //------------------------------------------------------------
        //             Model 2 - Controller Event Handlers
        //------------------------------------------------------------
        /**
         * DoRegisterHandler - this is where we handle any attempt to
         * register.
         */
        class DoRegisterHandler extends DefaultBaseEventListener {
                public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
                        // MUST DO FIRST - initialize things from the context
                        initUsingContext( context );

                        //unpack the necessary entities from the context
                        HttpServletRequest req = context.getRequest();

                        //map, validate the registration form
                        if ( logger.isDebugEnabled() ) logger.debug("Creating registration form");
                        RegistrationForm rf = new RegistrationForm();
                        ValidationException ve = null;
                        try {
                                //map/validate the form
                                if ( logger.isDebugEnabled() ) logger.debug("Mapping/Validating registration form");
                                rf.map(req).validate(true);

                                //actually store the user
                                if ( logger.isDebugEnabled() ) logger.debug("Creating user account");
                                HttpSession session = getComms().session.getHttpSession();
                                try {
                                        Person person = new Person();
                                        person.setLogin(rf.getVal(RegistrationForm.USER, "").toString());
                                        person.setPassword(rf.getVal(RegistrationForm.PASSWORD, "").toString());
                                        person.setFirstname(rf.getVal(RegistrationForm.FIRST_NAME, "").toString());
                                        person.setLastname(rf.getVal(RegistrationForm.LAST_NAME, "").toString());
                                        person.save();
                                        if ( logger.isDebugEnabled() ) logger.debug("Logging user in");
                                        setUser( person );
                                } catch ( DiscRackBusinessException e ) {
                                        //if we get an error here, it's because it was valid when we
                                        //validated, but by the time we stored it it wasn't any longer
                                        if ( logger.isDebugEnabled() ) logger.debug("Biz err:"+e+" Make sure user is Logged out");
                                        removeUserFromSession();
                                        throw new ValidationException(this, "Error creating account: "+e.getMessage()+" Please try again. If the problem persists contact your system administrator.", e);
                                } catch ( DiscRackPresentationException e ) {
                                        if ( logger.isDebugEnabled() ) logger.debug("Biz err:"+e+" Make sure user is Logged out");
                                        removeUserFromSession();
                                        throw new ValidationException(this, "Error creating account: "+e.getMessage()+" Please try again. If the problem persists contact your system administrator.", e);
                                }
                        } catch ( ValidationException e ) {
                                if ( logger.isDebugEnabled() ) logger.debug("Validation Exception: "+e);

                                if ( logger.isDebugEnabled() )
                                  CollectionsUtil.printStackTrace(e.getExceptionList(), 0, logger, null);

                                ve = e;
                        }

                        //store a copy of the form and any errors in the queue
                        if ( logger.isDebugEnabled() ) logger.debug("Saving form, errors");
                        context.putState(REGISTRATION_FORM, rf);
                        context.putState(REGISTRATION_ERR, ve);

                        //redirect appropriately: if it's valid, update the session
                        //and head for the login screen. Otherwise, return to the
                        //Registration screen...
                        if ( ve==null ) {
                                if ( logger.isDebugEnabled() ) logger.debug("Redirecting to GetLogin");
                                throw new ClientSideRedirectException(new GetLogin());
                        } else {
                                if ( logger.isDebugEnabled() ) logger.debug("InterruptDispatch to GetRegister");
                                throw new InterruptDispatchException(new GetRegister());
                        }
                }
        }


        //------------------------------------------------------------
        //                Model 2 - View Event Handlers
        //------------------------------------------------------------
        /**
         * RenderRegisterHandler - this is where we render the registration screen
         */
        class RenderRegisterHandler extends DefaultBaseEventListener {
                public void handleViewEvent(ViewEventContext context) throws EventException, ServletException, IOException {
                        // MUST DO FIRST - initialize things from the context
                        initUsingContext( context );

                        //get the XMLC object
                        RegisterHTML page = (RegisterHTML) getComms().xmlcFactory.create(RegisterHTML.class);

                        //create a template component and render it
                        if ( logger.isDebugEnabled() ) logger.debug("Creating template component");
                        Node node = page.getDocument().getElementById("RegistrationForm");

        TemplateView tv = new DefaultTemplateView(node);
        TemplateModel tm = new RegistrationModel(context);
        BTemplate templateComp = new BTemplate(tm);
        templateComp.setView(tv);

                        try {
                                templateComp.render(new DefaultViewContext(context));
                        } catch ( RenderException re ) {
                                logger.warn("Render err:"+re);
                        }

                        //now actually render it (the DOMWriter will take care of actually
                        //setting the content type)
                        if ( logger.isDebugEnabled() ) logger.debug("Rendering document");
                        new DefaultDOMWriter().write(page, context.getResponse());
                }
        }

        //------------------------------------------------------------
        //               Template Models
        //------------------------------------------------------------
        /**
         * RegistrationModel
         */
        class RegistrationModel extends AbstractTemplateModel {

                RegistrationForm fm = null;
                ValidationException ve = null;

                //constructor (extract form, exception from context)
                public RegistrationModel(EventContext ec) {
                        fm = (RegistrationForm) ec.getState(REGISTRATION_FORM);
                        ve = (ValidationException) ec.getState(REGISTRATION_ERR);
                }

                //register the model by name
                public String getName() {
                        return "Registration";
                }

                //provide items by key
                public Object getItem(String key) {
                        if ( key.equals("FirstName") ) {
                                return(fm!=null ? fm.getVal(RegistrationForm.FIRST_NAME, "") : "");
                        } else if ( key.equals("LastName") ) {
                                return(fm!=null ? fm.getVal(RegistrationForm.LAST_NAME, "") : "");
                        } else if ( key.equals("Username") ) {
                                return(fm!=null ? fm.getVal(RegistrationForm.USER, "") : "");
                        } else if ( key.equals("Password") ) {
                                return(fm!=null ? fm.getVal(RegistrationForm.PASSWORD, "") : "");
                        } else if ( key.equals("Repassword") ) {
                                return(fm!=null ? fm.getVal(RegistrationForm.REPASSWORD, "") : "");
                        } else if ( key.equals("Errors") ) {
                                if ( ve==null )	return "";
                                List errlist = ve.getExceptionList();
                                StringBuffer sb = new StringBuffer(errlist.size()>1 ? "There were several errors:" : "");
                                Iterator it = errlist.iterator();
                                while ( it.hasNext() ) {
                                        sb.append(" "+((ValidationException) it.next()).getMessage());
                                }
                                return sb.toString();
                        } else
                          return super.getItem(key);
                }

        }

        //------------------------------------------------------------
        //               HTML Form Mappings, Validators
        //------------------------------------------------------------
        /**
         * Registration form - define the registration form
         */
        class RegistrationForm extends DefaultFormMap {
                //registration form constants (these values correspond to the HTML params)
                static final String FIRST_NAME = "firstname";
                static final String LAST_NAME = "lastname";
                static final String USER = "login";
                static final String PASSWORD = "password";
                static final String REPASSWORD = "repassword";

                public RegistrationForm() {
                        //define the elements
                        if ( logger.isDebugEnabled() ) logger.debug("Defining Registration form elements");
                        this.defineElement(new DefaultFormElement(FIRST_NAME,   FormType.STRING, null, new NotNullValidator("You must enter a First name.")));
                        this.defineElement(new DefaultFormElement(LAST_NAME,  FormType.STRING, null, new NotNullValidator("You must enter a Last name.")));
                        this.defineElement(new DefaultFormElement(USER,     FormType.STRING, null, new NotNullValidator("You must enter a Login.")));
                        this.defineElement(new DefaultFormElement(PASSWORD,   FormType.STRING, null, new NotNullValidator("You must enter a Password.")));
                        this.defineElement(new DefaultFormElement(REPASSWORD,   FormType.STRING, null, new NotNullValidator("You must Confirm your password.")));

                        //define a form validator
                        if ( logger.isDebugEnabled() ) logger.debug("Defining Registration form validator");
                        this.defineValidator(new RegistrationValidator());
                }
        }

        /**
         * Registration validator - define a custom form validator
         */
        class RegistrationValidator extends DefaultFormValidator {
                public void validateForm(FormMap imap, boolean deferExceptions) throws ValidationException {
                        if ( logger.isDebugEnabled() ) logger.debug("Validating registration form");

                        //make sure the passwords match
                        if ( logger.isDebugEnabled() ) logger.debug("Make sure passwords match");
                        RegistrationForm map = (RegistrationForm) imap;
                        String password = map.getVal(RegistrationForm.PASSWORD, "").toString();
                        String repassword = map.getVal(RegistrationForm.REPASSWORD, "").toString();
                        if ( !password.equals(repassword) )	throw new ValidationException(map.getElement(RegistrationForm.PASSWORD), "Passwords do not match. Please re-enter.");

                        //validate the login information
                        if ( logger.isDebugEnabled() ) logger.debug("Make sure the login isn't taken");
                        String user = map.getVal(RegistrationForm.USER, "").toString();
                        try {
                                if ( PersonFactory.findPerson(user)!=null )	throw new ValidationException(map.getElement(RegistrationForm.USER), "This login already taken. Please select another.");
                        } catch ( DiscRackBusinessException e ) {
                                throw new ValidationException(map.getElement(RegistrationForm.USER), "Error checking login against database. Please try again. If the problem persists, contact your system administrator.", e);
                        }
                }
        }
}

