/*
 * MyTextPane                                                                              !:)==($)
 * <P>
 * @authors: Stefanovic Nenad, Puskas Vladimir, Bojanic Sasa
 * @Date:   July-November 2001.
 */

import javax.swing.*;
import java.util.*;
import java.beans.*;
import java.net.URL;
import javax.swing.text.*;
import javax.swing.event.*;
import java.awt.datatransfer.*;
import javax.swing.text.html.*;
import java.awt.Color;

/**
 * Since JTextPane does not redefines cut, copy, and paste methods inherited from JTextComponent
 * there is a inconsistency in implementation. Although styled text can be drawn and typed in,
 * clipboard operations aren't working as they suppose to. Hence this class.<p>
 *
 * Applets are tipically run in sandboxed environments, so if one wants to use system clipboard
 * it is essential to configure one's machine properly.<br> Locate your JRE folder, and bellow it, 
 * there is <code>lib\security\java.security</code> file. There are lines similar to the next two:<p>
 *    <code>policy.url.1=file:${java.home}/lib/security/java.policy<br>
 *    <code>policy.url.2=file:${user.home}/.java.policy<p>
 * Edit eighter file and insert into appropriate <b>grant</b> section line as follows<p>
 *    <code>permission java.awt.AWTPermission "accessClipboard", "access";<p>
 * <b>Hint</b><br><code>.security</code> files you can edit using your favorite text editor or with
 * <code>${java.home}\bin\policytool.exe</code>
 */
public class MyTextPane extends JTextPane{

   static final private String signature = "!:)==($)";
/**
 * Constructs a new instance.
 *
 * There is not much to do. Let the superclass' constructor does the hard work.
 */
   public MyTextPane(){
      super();
      try {
         jbInit();
      }catch( Exception e ) {
         e.printStackTrace();
      }
      pasteDeletesSelection = true;
   }
/**
 * Transfers the currently selected range to the system clipboard, <b>removing</b>
 * the selection from the text. The current selection is reset. Does nothing for null selections.
 */
   public void cut(){
      copy( true );
   }

   public void copyExt(){
      super.copy();
   }
/**
 * Transfers the currently selected range to the system clipboard, <b>leaving</b> 
 * the selected text intact. The current selection is reset. Does nothing for null selections.
 */
   public void copy(){
      copy( false );
      setSelectionStart( getSelectionEnd());
   }
/**
 * Transfers the contents of the system clipboard into the associated text model. 
 * If there is no selection, the clipboard contents are inserted in front of the current insert 
 * position in the associated view. If the clipboard is empty, does nothing.
 * If there is a selection in the associated view, behavior is described in the following note.
 * <p><b>NOTE:</b><br> Paste operation depends on state which is set by setPasteDeletesSelection
 * method. When true selection is deleted prior to actual pasting, otherwise text is inserted in
 * front of selection. Default is true.
 * @see #setPasteDeletesSelection
 * @see #getPasteDeletesSelection
 */
   public void paste(){
      if( myClipboard == null )
         return;
      if( pasteDeletesSelection ){
         int selectionStart = getSelectionStart();
         int selectionEnd = getSelectionEnd();
         if( selectionStart < selectionEnd ){
            try{
               getStyledDocument().remove( selectionStart, selectionEnd - selectionStart );
            }catch( BadLocationException e ){
               e.printStackTrace();
            }
         }
      }
      try{
         String clip = ( String )( myClipboard.getContents( DataFlavor.stringFlavor ).getTransferData( DataFlavor.stringFlavor ));
// test on length is crucial because substring method will throw an exception for strigs shorter than signature
         if(( clip.length()> signature.length())&&( clip.substring( 0, signature.length()).equals( signature )))
            takeIn( clip );
         else
            super.paste();
      }catch( Exception e ){
         e.printStackTrace();
      }
   }
/**
 * Long and unforgettable name of this method is chosen to explain itself.
 * @see #paste
 * @see #setPasteDeletesSelection
 * @return wether paste method would deleted selection.
 */   
   public boolean getPasteDeletesSelection(){
      return this.pasteDeletesSelection;
   }
/**
 * Long and unforgettable name of this method is chosen to explain itself.
 * @see #paste
 * @see #getPasteDeletesSelection
 * @param value specifying wether paste method would deleted selection
 */   
   public void setPasteDeletesSelection( boolean pasteDeletesSelection ){
      this.pasteDeletesSelection = pasteDeletesSelection;
   }
/* NOTE: Javadoc won't make this appear in generated documentation, 
 * still there are some private stuff, that need to be explained.
 *
 * One cannot know if system security settings will allow to use system clipboard, hence this
 * field. Working methods wouldn't know what clipboard they are accessing, they don't need to
 * know, while myClipboard is working properly.
 */
   private java.awt.datatransfer.Clipboard myClipboard;
/*
 * Because paste can act two ways this field bares what way it should act.
 */
   private boolean pasteDeletesSelection;
/*
 * Initializes the state of this instance.
 * This won't appear in javadoc generated files. Method itself is included,...
 * well I don't know why it is here.
 */
   private void jbInit() throws Exception{
      pickClipboardToUse();
   }
/*
 * CUT and COPY operations are the same except CUT deletes text it copied to clipboard. This
 * private method provides for both.
 */
   private void copy( boolean deleteToo ){
      int selectionStart = getSelectionStart();
      int selectionEnd = getSelectionEnd();
      if( selectionStart >= selectionEnd )
         return;
      StringSelection stringCopied = new StringSelection( returnCopy( selectionStart, selectionEnd ));
      myClipboard.setContents( stringCopied, stringCopied );
      if( deleteToo ){
         try{
            getStyledDocument().remove( selectionStart, selectionEnd - selectionStart );
         }catch( BadLocationException e ){
            e.printStackTrace();
         }
      }
   }
      
/*
 * Here it is. Method pickClipboardToUse tries to connect JTextPane to system clipboard.
 * If connection fails local clipboard is made, so cut, copy, paste functionality could
 * work as expected.
 */
   //private void pickClipboardToUse(){
   public void pickClipboardToUse(){
      if( myClipboard != null )
         return;
      try{
         System.getSecurityManager().checkSystemClipboardAccess();
         myClipboard = getToolkit().getSystemClipboard();
         System.out.println("Using system clipboard.");
      }catch( SecurityException se ){
         myClipboard = new Clipboard("TextEditor");
         System.out.println("Using local clipboard, because system isn't available.");
      }
   }
/*
 * Escape sequences used to store character and paragraph attributes in String are 
 * (all are preceded with backslash sign '\'):
 *
 * family   =   whatever font family is set its name is concatenated after this tag
 * size     =   size of font in decimal representation
 * style    =   (Bold, Italic, or Underlined)
 * align    =   number which return and accept static methods of StyleConstants class
 * color    =   rgb value
 *
 * private method takeIn does dirty work for paste method, parsing in content of myOwnClipboard 
 * and adding text into document together with attributes
 */
   private void takeIn( String myOwnClipboard ){
      StyledDocument theDocument = getStyledDocument();
      SimpleAttributeSet characterAttributes = new SimpleAttributeSet( theDocument.getCharacterElement( getSelectionStart()).getAttributes());
      for( int i = signature.length(); i < myOwnClipboard.length(); ++i ){
         String currentChar = myOwnClipboard.substring( i, 1 + i );
         int insertHere = getSelectionStart();
         if( currentChar.equals("\\")){
            int nextBackSlash = myOwnClipboard.indexOf('\\', 1 + i );
            if(( myOwnClipboard.charAt( i + 1 )=='\\')){
               try{
                  theDocument.insertString( insertHere, currentChar, characterAttributes );
                  i++;
                  continue;
               }catch( BadLocationException e ){
                  e.printStackTrace();
               }
            }
            currentChar = myOwnClipboard.substring( i, myOwnClipboard.indexOf("\\", 1 + i ));
            switch( currentChar.charAt( 2 )){
               case 'l':
                  StyleConstants.setAlignment( characterAttributes,( new Integer( currentChar.substring( 6 ))).intValue());
                  try{
                     theDocument.insertString( insertHere,"\n", characterAttributes );
                     insertHere = getSelectionStart();
                     theDocument.setParagraphAttributes( insertHere, 1, characterAttributes, true );
                  }catch( BadLocationException e ){
                     e.printStackTrace();
                  }
                  break;
               case 't':
                  if( currentChar.substring( 6 ).equals("Bold"))
                     StyleConstants.setBold( characterAttributes, true );
                  else if( currentChar.substring( 6 ).equals("Italic"))
                     StyleConstants.setItalic( characterAttributes, true );
                  else
                     StyleConstants.setUnderline( characterAttributes, true );
                  break;
               case 'a':
                  StyleConstants.setFontFamily( characterAttributes, currentChar.substring( 7 ));
                  StyleConstants.setBold( characterAttributes, false );
                  StyleConstants.setItalic( characterAttributes, false );
                  StyleConstants.setUnderline( characterAttributes, false );
                  break;
               case 'o':
                  StyleConstants.setForeground( characterAttributes, new Color( new Integer( currentChar.substring( 6 )).intValue()));
                  break;
               case 'i':   
                  StyleConstants.setFontSize( characterAttributes,( new Integer( currentChar.substring( 5 ))).intValue());
                  break;
            }
            i += currentChar.length();
         }else{
            try{
               theDocument.insertString( insertHere, currentChar, characterAttributes );
            }catch( BadLocationException e ){
               e.printStackTrace();
            }
         }
      }
   }
   
 /** adding text into document together with attributes
  * @param fotext text for adding
  * @param overwrite indicates whether or not the previous text should be cleared before the new text are added.
 */ 
   public void loadTextf(String fotext,boolean overwrite){
	 if (overwrite){  
	   setText("");
	   }
	   takeIn(fotext);
   }
   
/*
 * private method returnCopy is heart'n'soul of both cut and copy methods. 
 * Running through selection character by character it makes String representation of styled 
 * text which would be stored in clipboard.
 */
   private String returnCopy( int selectionStart, int selectionEnd ) {
      StyledDocument theDocument = getStyledDocument( );
      AttributeSet characterAttributes = theDocument.getCharacterElement( selectionStart ).getAttributes( );
      String result = signature + shotOut( characterAttributes );
      while(selectionStart < selectionEnd) {
         AttributeSet currentAttributes = theDocument.getCharacterElement( selectionStart ).getAttributes( );
         if(! characterAttributes.isEqual( currentAttributes )) {
            result += shotOut( currentAttributes );
            characterAttributes = currentAttributes.copyAttributes( );
         }
         try{
            String theChar = theDocument.getText( selectionStart++, 1 );
            if(theChar.equals( "\n" )) {
               result += "\\align"+ StyleConstants.getAlignment( theDocument.getParagraphElement( selectionStart ).getAttributes())+"\\";
            }else{
               result += theChar;
               if(theChar.equals("\\"))
                  result += theChar;
            }
         }catch( BadLocationException e ) {
            e.printStackTrace();
         }
      }
      return( result );
   }
   
/**
* Makes String representation of styled 
* text which would be stored in file.
*/   
    public String prepareTextForSave(){
	   String res="";
	   res=returnCopy(0,getText().length()-1);
	   return res;
   }
   
 
    
/*
 * private method shotOut returns String representation of current character attributes
 * (font family, size, and style)
 */
   private String shotOut( AttributeSet attributes ){
      String result = "\\family" + StyleConstants.getFontFamily( attributes )+"\\";
      result += "\\color" + StyleConstants.getForeground( attributes ).getRGB()+"\\";
      if( StyleConstants.isBold( attributes )){
         result += "\\styleBold\\";
      }
      if( StyleConstants.isItalic( attributes )){
         result += "\\styleItalic\\";
      }
      if( StyleConstants.isUnderline( attributes )){
         result += "\\styleUnderline\\";
      }
      return( result +"\\size"+ StyleConstants.getFontSize( attributes )+"\\" );
   }
}
