/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.business.cart;

import java.util.Hashtable;
import java.util.Enumeration;
import golfShop.spec.cart.*;
import golfShop.business.item.*;

public class CartImpl implements Cart,java.io.Serializable {

    private static final int maxQuantity = 9999;    // set qty limit

    private CartItemQuery   itemQuery;  // Use this to find items.
    private Hashtable       itemPairs;  // The actual contents.
    private UndoRedoStack   undoRedo;   // Used for undo/redo.

    /**
     * Constructor. Must pass in the query object so that later it 
     * can convert ObjectIds to CartItems.  Start off with an empty cart.
     *
     * @param itemQuery A CartItemQuery object for finding CartItems.
     */
    public CartImpl (CartItemQuery itemQuery) {
        this.itemQuery = itemQuery;
        itemPairs = new Hashtable();

        /*
         * Initialize the undo / redo stack. It will keep copies of
         * the hashtable, and manage the different versions.
         * This call creates it and sets the current version to the
         * initially empty shopping cart. 
         * WARNING: This implementation keeps coppies of the cart, and
         * it's objects. So it's O(N^2). In the future, just
         * store the diffs.
         */
        undoRedo = new UndoRedoStack(itemPairs);
    }

    public void addItem(long ObjectId) {
        addItem(ObjectId, 1);
    }

    public void addItem(long ObjectId, int quantity) {
        if (quantity < 0 || quantity > maxQuantity) {
            // throw exception
        }
        if (containsItem(ObjectId)) {
            // If the item is already in cart, just change the quantity.
            int newQty = quantity + 
                        ((CartItemPairImpl)(itemPairs.get(new Long(ObjectId)))).getQuantity();
            setQuantity(ObjectId, newQty);
        } else if (quantity != 0) {
            // Otherwise, if quantity is non-zero, add the item to the cart.
            CartItem     item     = itemQuery.getItem(ObjectId);
            CartItemPairImpl itemPair = new CartItemPairImpl(item, quantity);
            itemPairs.put(new Long(ObjectId), itemPair);
        }
    }

    public void removeItem(long ObjectId) {
        itemPairs.remove(new Long(ObjectId));
    }

    public Enumeration getContents() {
        return itemPairs.elements();
    }

    public void setQuantity(long ObjectId, int quantity) {
        if (quantity != 0) { 
            CartItemPairImpl itemPair = (CartItemPairImpl) itemPairs.get(new Long(ObjectId));
            if (itemPair != null)
                itemPair.setQuantity(quantity);
        } else {
            removeItem(ObjectId);
        }
    }

    public int getQuantity(long ObjectId) {
        CartItemPairImpl itemPair = (CartItemPairImpl) itemPairs.get(new Long(ObjectId));
        if (itemPair != null) return itemPair.getQuantity();
        return -1;
    }

    public double getTotal() {
        double total = 0.0;
        CartItemPairImpl itemPair;
        Enumeration enumeration = getContents();
        while (enumeration.hasMoreElements()) {
            itemPair = (CartItemPairImpl) enumeration.nextElement();
            int quantity = itemPair.getQuantity();
            double price = itemPair.getItem().getPrice();
            total += quantity * price;
        }
        return total;
    }

    public boolean containsItem(long ObjectId) {
        return (itemPairs.containsKey(new Long(ObjectId)));
    }

    public boolean isEmpty() {
        return (itemPairs.isEmpty());
    }

    public void reset() {
        /*
         * Reset the cart to an empy, fresh state.
         * Just forget about the old contents.
        */
        this.itemPairs = new Hashtable();
        this.undoRedo = new UndoRedoStack(itemPairs);
    }

    
    //------------------------------------------------------------
    //  Support for undo/redo.
    //------------------------------------------------------------ 

    public boolean canUndo () {
        return undoRedo.canUndo();
    }

    public boolean canRedo () {
        return undoRedo.canRedo();
    }

    /*
     * Call addItem and/or setQuantity and/or removeItem as much as you
     * want. When you are done, call this function. It will save a copy of
     * the hash table. 
     * This function should be called after every user interaction with 
     * the cart. For example, adding an item to the cart is one user
     * interaction. Changing the quantity of every item in the cart, and then
     * clicking on the "update" button is also one user interaction, although
     * it results in many calls to setQuantity. After all the calls to
     * setQuantity, call this function. That way all the modifications will
     * be undone and redone in one chunk, which is what the user expects.
     */
    public void doneModifying () { 
        undoRedo.storeNewVersion(itemPairs);
    }

    public void undo () {
        if (canUndo()) 
            itemPairs = undoRedo.undoToPreviousVersion();
    }

    public void redo () {
        if (canRedo()) 
            itemPairs = undoRedo.redoToNewerVersion();
    }

    /**
     * Generate information about the cart for debugging.
     */
    public String toString() {
        StringBuffer buf = new StringBuffer();
        Enumeration enumeration = itemPairs.elements();
        while (enumeration.hasMoreElements()) {
            CartItemPairImpl itemPair = (CartItemPairImpl)enumeration.nextElement();
            buf.append(itemPair.toString());
            buf.append("\n");
        }
        return buf.toString();
    }
}
