/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.business.cart;

import java.util.Stack;
import java.util.Hashtable;
import java.util.Enumeration;
import golfShop.business.cart.CartItemPairImpl;


/*
    Only the cart object needs to use this.
*/
public class UndoRedoStack implements java.io.Serializable {

	private Hashtable 	currentVersion;
	private Stack  		undoStack;
	private Stack  		redoStack;

	/**
	 * Create a new UndoRedoStack. To avoid uninitialized states, you
	 * must pass in the initial version of the hashtable you want to track.
	 * Every time you change this object, call StoreNewVersion().
	 * If you want to undo, call UndoToPreviousVersion(). This returns
	 * the previous version of the hashtable; you should use this in your
	 * program as the new value of the hashtable. If you want to redo,
	 * call RedoToNewerVersion(). This returns a newer version of the
	 * hashtable.
	 *
	 * @author Andrew John
	 * @param initialVersion The starting version of the hashtable to be
	 * tracked.
	 * @version $Revision: 1.1
	 */
	protected UndoRedoStack (Hashtable initialVersion) {
		undoStack = new Stack();
		redoStack = new Stack();
		currentVersion = copyHT(initialVersion);
	}

	/**
	 * After you make a change to your object, call this function to
	 * remember the current version. The previous version is saved for
	 * so that you can do an undo.
	 *
	 * @param newVersion The current version of the object to 
	 * be tracked.
	 */
	protected void storeNewVersion (Hashtable newVersion) {
		// Save the current version for later.
		undoStack.push(currentVersion);
		// Throw out the whole redo stack.
		redoStack = new Stack();
		// This is now the current version.
		currentVersion = copyHT(newVersion);	
	}

	protected boolean canUndo() {
		return !undoStack.empty();
	}

	protected Hashtable undoToPreviousVersion () {
		if (undoStack.empty()) 
			// Error condition.
			return copyHT(currentVersion);
		else {
			// Save the current version in case they redo.
			redoStack.push(currentVersion);
			// Go back to the previous version.
			currentVersion = (Hashtable) undoStack.pop();
			return copyHT(currentVersion);
		}
	}

	protected boolean canRedo() {
		return !redoStack.empty();
	}

	protected Hashtable redoToNewerVersion () {
		if (redoStack.empty())
			return copyHT(currentVersion);
		else {
			undoStack.push(currentVersion);
			currentVersion = (Hashtable) redoStack.pop();
			return copyHT(currentVersion);
		}
	}

	protected Object getCurrentVersion () {
		return copyHT(currentVersion);
	}

	/*
	    Make a copy of the hashtable. If you call clone() on a hashtable
	    it gives you a new hashtable, but it contains pointers to the
	    same key objects and value objects. If you change the
	    value objects, they will be changed in both coppies of the 
	    hashtable. This is not what we want for undo functionality.
	    So this function creates a new hashtable and populates it
	    with clones of the value objects. The key objects are not cloned.
		
		All the versions stored in this object must be insulated from 
		being modified by the rest of the program. So make coppies of the
		input, and only give out coppies.
	*/
	private Hashtable copyHT(Hashtable ht) {
		Hashtable newHT = new Hashtable();
		Enumeration e = ht.keys(); 
		while (e.hasMoreElements()) {
			Object key = e.nextElement();
			CartItemPairImpl value = (CartItemPairImpl) ht.get(key);
			CartItemPairImpl valueCopy = value.copy();
			newHT.put(key, valueCopy);
		}
		return newHT;
	}

	// For debugging only.
	private void printHT(Hashtable ht) {
		Enumeration e = ht.keys(); 
		System.out.println("Hashtable contents:");
		while (e.hasMoreElements()) {
			Object key = e.nextElement();
			CartItemPairImpl value = (CartItemPairImpl) ht.get(key);
			System.out.println("ObjectId " + value.getItem().getObjectId() + 
											" x " + value.getQuantity());
		}
		System.out.println("\n");
	}
}

		

