/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.business.user;

import java.io.*;
import java.net.*;
import java.security.*;
import java.util.*;
import com.lutris.appserver.server.user.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
import golfShop.data.user.UserDOImpl;

import golfShop.business.item.ItemQuery;
import golfShop.business.cart.CartImpl;
import golfShop.spec.user.*;
import golfShop.spec.LoginException;
/**
 * The User Manager Business Object.  This object handles requests
 * for user specific data and login verification.
 *
 * @author  Shawn McMurdo
 * @version $Revision: 1.1 $
 */
public class GolfShopUserManagerImpl implements GolfShopUserManager {
    /**
     * Username and password verification was successful.
     */
    public final int AUTH_OK = 0;

    /**
     * Username and password verification failed because of
     * authentication failure.
     * AUTH_FAILED is thrown by a login request if the username
     * does not exist in the User database, the password was
     * incorrect, or an otherwise unknown error occured during the login
     * request.  The reason for the third case is that it is not wise from a
     * security standpoint to give unknown users potentially valuable
     * information about possible errors occuring in the login facility.
     */
    public final int AUTH_FAILED = 1;

    /**
     * UNKNOWN_ERROR is indicated by methods other than the user login method
     * to report that an unknown error has prevented a request from
     * being processed.  The UserManager.login() method reports
     * AUTH_FAILED for unknown errors for security reasons.
     */
    public  final int UNKNOWN_ERROR = 2;

    /**
     * IO_ERROR is thrown by methods of the UserManager if a
     * communication failure prevented access to required information
     * from the User database or elsewhere.
     */
    public final int IO_ERROR = 3;
    
    /**
     * PERMISSION_DENIED indicates that a UserManager method has
     * failed because the requesting user has insufficient privilege
     * to perform the requested operation.
     */
    public final int PERMISSION_DENIED = 4;
    
    /**
     * MULTIPLE_LOGIN indicates that a user has attempted to login
     * more than the maximum number of allowable sessions.
     */
    public final int MULTIPLE_LOGIN = 5;
    
    /**
     * ACCOUNT_DISABLED indicates that the account for the
     * requested user has been disabled by the administrator.
     */
    public final int ACCOUNT_DISABLED = 6;
    
    /**
     * PASSWORD_TYPO indicates that the two copies of the new
     * password were not identical, implying that a user may have
     * accidentally typed a new password incorrectly.
     */
    public  final int PASSWORD_TYPO = 7;

    /**
     * PASSWORD_WEAK indicates that the UserManager refused
     * to accept a user's password change request because the new password
     * does not comply with policy relating to password strength.  For
     * example, the password may not have been long enough or contained
     * enough nonalphabetic characters.
     */
    public  final int PASSWORD_WEAK = 8;

    /**
     * UNKNOWN_HOST indicates that a login attempt failed because the IP
     * address of the requester's host was not furnished with the login
     * request.
     */
    public final int UNKNOWN_HOST = 9;

    /**
     * USERNAME_ALREADY_EXISTS indicates that a new account could not be
     * created because an account with that same username already exists.
     */
    public  final int USERNAME_ALREADY_EXISTS = 10;

    /**
     * The GolfShop application instance that this manager is
     * a part of.
     */


    /**
     * Process an authentication request.  Verify
     * that the user's account and password are valid.
     *
     * @param username The username for the account.
     * @param password The password entered by the user.
     *
     * @returns An int representing success (AUTH_OK), or a failure
     * reason (AUTH_FAILED, AUTH_IO_ERROR).
     */
    public int authenticate(String username, String password) {
        // Get user from database.
        UserDO theUser = UserDOImpl.lookupUser(username);
        if (theUser == null) {
            // No such user.
            return AUTH_FAILED;
        }
        // Check password.
        return theUser.authenticate(password);
    }
    
    /**
     * Process a login request issued by a user.  Verify
     * that the user's account and password are valid.
     * A username may be logged in multiple times. If the login is
     * sucessfull, the session object's user pointer is set to the
     * user data object represeting the logged in username.
     *
     * @param username The user identification entered by the user.
     * @param password The password entered by the user.
     * @param session The session object for the broweser attempting the
     * login.
     * @exception LoginException Thrown if authentication fails.
     */
    public void login(String username, String password, Session session) 
                        throws LoginException, SessionException {
        Session standardSession = (Session)session;
        SessionManager standardSessionManager =
            (SessionManager)session.getSessionManager();
        SessionData sd = standardSession.getSessionData();
        // Verify the login.
        // Get user from database.
        UserDO theUser = UserDOImpl.lookupUser(username);
        if (theUser == null) 
            // No such user.
            refuse(AUTH_FAILED);
        // Check password.
        if (theUser.authenticate(password) != AUTH_OK)
            refuse(AUTH_FAILED);

        /*
         * Even if the user is already logged in, allow the login.
         * If they are already logged in, they have already connected to
         * the Lutris Business Server, and so they already have a session
         * cookie and an assocated session object exists in the session
         * manager.
         * Either way, give them a new (empty) cart.
         *
         * This is an important policy decision. We could re-use the
         * existing session key, so the two users share one cart. Or we
         * could have the new login to terminate the old session by 
         * elimintating the previous session key. Or we could deny the
         * new login. It depends on how you want your application to behave.
         */
        if (standardSession.getUser() != null) {
            standardSession.clearUser();
            try {
                sd.remove("cart"); // Throw away old cart.
            } catch (KeywordValueException e) {
                throw new FatalExceptionError(e);
            }
        }

        /*
         * Save the user in the session. This signifies that the
         * session is valid, and the user is logged in. When the user fisrt
         * connects to this application, she will be given a new session,
         * but the user is not logged in so the user pointer is null.
         * That is, new sessions have an internal user pointer. It is
         * initally null. Until we set it, the user for a given session
         * is not considered logged in.
         */ 
        standardSession.setUser((UserDOImpl)theUser);

        /*
         * Save the cart in the session data so it is available for each
         * request they make.
         */
        try {
            sd.set("cart", new CartImpl(ItemQuery.global));
        } catch (KeywordValueException e) {
            throw new LoginException(UNKNOWN_ERROR);
        }
    }
    
    /**
     * 
     * @param session The session to log out.
     */
    public void logout(Session session) throws SessionException {
        if (session != null) {
        	/*DTDT 260604*/
	        try{
	        	SessionData sessionData=session.getSessionData();
	        	String [] sessinDataKeys=sessionData.leafKeys();
	        	for (int i=0;i<sessinDataKeys.length;i++){
	        		sessionData.remove(sessinDataKeys[i]);
	        	}
	        }catch (Exception e){
	        	throw new SessionException(e.getMessage());
	        }
	        
            //session.getSessionManager().deleteSession(session.getSessionKey());
        }
    }
    
    /**
     * Internal convenience method to throw a login refusal error.
     * 
     * @param why The error code indicating why the request failed.
     */
    private static final void refuse(int why) throws LoginException {
        throw new LoginException(why);
    }



    /**
     * This function creates a new account, then calls login() above to
     * create the session etc...
     *
     * @param username The user identification entered by the user.
     * @param password The password entered by the user.
     * @returns A cookie representing the user's new session.
     * @exception LoginException Thrown if authentication fails.
     */
    public void createAccount(String username,
                              String password, 
                              String address1,
                              String address2,
                              String city,
                              String state, 
                              String zip,
                              String creditCard,
                              String email, 
                              Session session) throws LoginException, SessionException {

        // Verify the username is available.
        // Get user from database.
        UserDO theUser = UserDOImpl.lookupUser(username);
        if (theUser != null) {
            // That user name is already taken.
            refuse(USERNAME_ALREADY_EXISTS);
        }
        // Create the new user account/data-object.
        UserDO newUser = UserDOImpl.createUser(username,
                        password, address1, address2, city, 
                        state, zip, creditCard, email);
        if (newUser == null) 
            refuse(UNKNOWN_ERROR);
        // Now that the account exists, log in like normal.
        login(username, password, session);
    }


    /*
     * This is just defined so that we meet the UserManager interface
     * specs. In a larger application this would be usefull, but in this
     * demo it is not used.
    */
    public User getUser(String username) {
        return (User) UserDOImpl.lookupUser(username);
    }
}

