/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.data.user;

import java.io.Serializable;
import com.lutris.util.ConfigException;
import com.lutris.appserver.server.*;
import golfShop.data.user.*;
import golfShop.spec.user.UserDO;
import com.lutris.appserver.server.user.*;
import golfShop.spec.LoginException;
/**
 *  The application specific user data object. This is the base object that
 * does most of the work. All the policy decisions happen here. 
 *  The abstract functions are for accessing the store of all users. 
 * There will be at least 3 sub-classes that implement these functions, 
 * one that just keeps the list in memory, one that reads and writes to a 
 * text file, and one that acesses an ldap database. These abstract functions
 * are private, so that all 3 flavors appear the same to the outside world.
 *  This is a data object, not a business object. This object hides the
 * storage medium (memory, file, ldap etc..). The class methods get users
 * from the storage medium, and create new users. The object methods give
 * you access to the user data, and update changes to the storage medium.
 * Passwords are protected, but other than that no policy is enforeced.
 * For example, cheching password length, making sure the correct password
 * was enntered etc.. is the responsiblity of the user buisness object.
 * 
 * @author Andrew John
 * @version $Revision: 1.1 $
 */
public class UserDOImpl implements UserDO,User, Serializable {

public static final int AUTH_OK = 0;
public static final int AUTH_FAILED = 1;
     
    protected boolean isDirty = false;
    protected String username = null;
    private String password = null;
    protected String address1 = null;
    protected String address2 = null;
    protected String city = null;
    protected String state = null;
    protected String zip = null;
    protected String creditCard = null;
    protected String email= null;

    private static UserStore storage = null;

    /**
     *    Create the correct user store. 
     */
    public static void InitializeStorageOption(String opt,String fn) throws ApplicationException{
  
  
        try {
            Class c = null;
            if (opt.equalsIgnoreCase("memory")){
              c = Class.forName("golfShop.data.user.MemoryUserStore");
              storage = (UserStore) c.newInstance();
              storage.initializeUserStore();
           }
            else if (opt.equalsIgnoreCase("file"))
           {
            c = Class.forName("golfShop.data.user.FileUserStore");
            storage = (UserStore) c.newInstance();
            storage.initializeUserStore(fn);
           }
            else if (opt.equalsIgnoreCase("ldap"))
           {
                c = Class.forName("golfShop.data.user.LdapUserStore");
           }
            else 
                throw new ApplicationException("Invalid user store option " +
                    opt);
           
        
        } catch (ClassNotFoundException cnfe) {
            throw new ApplicationException("Unable to create user store" + 
                " of type: " + opt + "\n" + cnfe);
        } catch (IllegalAccessException iae) {
            throw new ApplicationException("Unable to create user store" + 
                " of type: " + opt + "\n" + iae);
        } catch (InstantiationException ie) {
            throw new ApplicationException("Unable to create user store" + 
                " of type: " + opt + "\n" + ie);
        }
         
    }

    /**
     * Constructor. This is protected so that the outside world cannot
     * create users. To obtain an already existing user, call the class
     * method lookupUser(). To create a new user, call the class method
     * createUser().
     * 
     * @param username A unique String representing the user's identity.
     * @param password A String containing the user's password.
     * @param userid An int containing the user's unique id number.
     */ 
    protected UserDOImpl(String username, String password, String address1,
                        String address2, String city, String state,
                        String zip, String creditCard, String email) {
        this.username = username;
        this.password = password;
        this.address1 = address1;
        this.address2 = address2;
        this.city = city;
        this.state = state;
        this.zip = zip;
        this.creditCard = creditCard;
        this.email = email;
        this.isDirty = false;
    }
   

    public static UserDO lookupUser(String username) {
        return storage.lookupUserFromUserStore(username);
    }
        

    /**
     *   This is the public interface for creating user data objects.
     */
    public static UserDO createUser(String username, String password, 
                        String address1, String address2, String city, 
                        String state, String zip, String creditCard, 
                        String email) {
        // Make sure they don't already exist.
        if (storage.usernameInUserStore(username))
            return null;

        // Create them.
        UserDOImpl newGuy =  new UserDOImpl (username, password, address1, address2, 
                                     city, state, zip, creditCard, email);
        // Add the new user to the collection of all users.
        storage.addUserToUserStore(newGuy);

        // Return the new user.
        return newGuy;
    }


    /**
     *   This is needed to fulfill the requirements of being a User object.
     *   It is not used in this demo.
     *   Changes a user's password.
     *   The old password must be supplied and it must match the
     *   password stored in the user database. 
     *   This only changes the copy in memory, call commitChanges() to
     *   write the new password out to the storage medium.
     */
    public void changePassword(String old, String new1, String new2,
                            User initiatingUser) throws LoginException {
        if ((new1 != new2) || (authenticate(old) != AUTH_OK)) 
            throw new LoginException(AUTH_FAILED);
        password = new1;
        isDirty = true;
    }

    /**
     */
    public String getName() {
        return username;
    }

   
    /**
     * Process an authentication request.  Verify
     * that the user's account and password are valid.
     *
     * @param password The password entered by the user.
     * @return True if the password matches, false otherwise.
     */
    public int authenticate(String password) {
        if (this.password.equals(password)) 
                  return AUTH_OK;
        else
                  return AUTH_FAILED;
    }

    protected String getPassword() {
        return this.password;
    }
 

    /**
     * Write any changes to the storage medium. If no changes have been
     * made, no action will be taken.
     */
    public void commitChanges() { 
        if (!isDirty)
            return;
        storage.updateUserInUserStore(this);
        isDirty = false;
    }


    public String getAddress1() {
        return address1;
    }

    public void setAddress1(String address1) {
        if (address1.compareTo(this.address1) != 0) {
            this.address1 = address1;
            isDirty = true;
        }
    }

    public String getAddress2() {
        return address2;
    }

    public void setAddress2(String address2) {
        if (address2.compareTo(this.address2) != 0) {
            this.address2 = address2;
            isDirty = true;
        }
    }

    public String getCity() {
        return city;
    }

    public void setCity(String city) {
        if (city.compareTo(this.city) != 0) {
            this.city = city;
            isDirty = true;
        }
    }

    public String getState() {
        return state;
    }

    public void setState(String state) {
        if (state.compareTo(this.state) != 0) {
            this.state = state;
            isDirty = true;
        }
    }

    public String getZip() {
        return zip;
    }

    public void setZip(String zip) {
        if (zip.compareTo(this.zip) != 0) {
            this.zip = zip;
            isDirty = true;
        }
    }

    public String getCreditCard() {
        return creditCard;
    }

    public void setCreditCard(String creditCard) {
        if (creditCard.compareTo(this.creditCard) != 0) {
            this.creditCard = creditCard;
            isDirty = true;
        }
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        if (email.compareTo(this.email) != 0) {
            this.email = email;
            isDirty = true;
        }
    }

    /**
     * Compare this object to another user object.  They are
     * equal if the user name is the same.
     */
    public boolean equals(Object obj) {
        return (obj instanceof UserDO)
            && (((UserDO)obj).getName().equals(getName()));
    }

    /**
     * Convert to string for debugging.
     */
    public String toString() {
        return "isDirty = \"" + isDirty + "\", " 
            + "username = \"" + username + "\", " 
            + "password = \"" + password + "\", " 
            + "address1 = \"" + address1 + "\", " 
            + "address2 = \"" + address2 + "\", " 
            + "city = \"" + city + "\", " 
            + "state = \"" + state + "\", " 
            + "zip = \"" + zip + "\", " 
            + "creditCard = \"" + creditCard + "\", " 
            + "email = \"" + email + "\", ";
    }
}
 
