/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.presentation.xmlc.cart;

import org.enhydra.xml.xmlc.*;
import org.enhydra.xml.xmlc.html.*;
import com.lutris.appserver.server.httpPresentation.*;
import java.io.*;
import java.net.URLEncoder;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import golfShop.presentation.xmlc.utilities.*;
import java.math.BigDecimal;
import java.util.Enumeration;
import golfShop.spec.cart.*;
/**
 * Generate a HTML table to display the cart contents.  It is based on the
 * prototype table in the Contents.html or Confirm.html page, which
 * implement ContentsTable.
 */
public class ContentsTableFormatter {
    /**
     * Prefix for dynamic quantify field; will have object id added.
     */
    public static final String QTY_PREFIX = "qty_";

    /**
     * Prefix for delete field; will have object id added.
     */
    public static final String DEL_PREFIX = "del_";

    /**
     * Id for item num in prototype row.
     */
    private static final String ITEM_NUM_ID = "itemNum";

    /**
     * Id for item quantity in prototype row.
     */
    private static final String ITEM_QUANTITY_ID = "itemQuantity";

    /**
     * Id for item desc in prototype row.
     */
    private static final String ITEM_DESC_ID = "itemDesc";

    /**
     * Object being operated on.
     */
    private ContentsTable htmlObj;

    /**
     * Prototype row for the table.
     */
    private HTMLTableRowElement protoRow;
    
    /**
     * Is table dynamic?
     */
    private boolean dynamic;

    /**
     * Cart to add to HTML object.
     */
    private Cart cart;

    /**
     * Generate a table row for an item based on the prototype.
     *
     * @param rowNum Row number in table.
     * @param itemPair The item information to display.
     */
    private HTMLTableRowElement makeItemRow(int rowNum, CartItemPair itemPair) {
	// Get necessary info.
	int quantity = itemPair.getQuantity();
	CartItem item = itemPair.getItem();
	BigDecimal price = new BigDecimal(item.getPrice());
	BigDecimal total = new BigDecimal(quantity * item.getPrice());

	// Format price and total to have 2 digits past the decimal.
	price = price.setScale(2, BigDecimal.ROUND_HALF_UP);
	total = total.setScale(2, BigDecimal.ROUND_HALF_UP);

        // Setup new row.
        HTMLTableRowElement newRow = (HTMLTableRowElement)protoRow.cloneNode(true);

        // Item number
        Element itemNumElem = XMLCUtil.getRequiredElementById(ITEM_NUM_ID, newRow);
        XMLCUtil.getFirstText(itemNumElem).setData(Integer.toString(rowNum));

        // Quantity
        String quantityStr = Integer.toString(quantity);
        if (dynamic) {
            // Update input node.
            HTMLInputElement quantityElem = (HTMLInputElement)XMLCUtil.getRequiredElementById(ITEM_QUANTITY_ID, newRow);
            quantityElem.setName(QTY_PREFIX + item.getObjectId());
            quantityElem.setValue(quantityStr);
        } else {
            // The node will only be an input node if it came from the 
            // cart HTML.  Figure it out and either replace input node or
            // change the next node.
            Element quantityElem = XMLCUtil.getRequiredElementById(ITEM_QUANTITY_ID, newRow);
            if (quantityElem instanceof HTMLInputElement) {
                Text text = htmlObj.createTextNode(quantityStr);
                quantityElem.getParentNode().replaceChild(text, quantityElem);
            } else {
                XMLCUtil.getFirstText(quantityElem).setData(quantityStr);
            }
        }

        // Description
        Element itemDescElem = XMLCUtil.getRequiredElementById(ITEM_DESC_ID, newRow);
        XMLCUtil.getFirstText(itemDescElem).setData(item.getName());

        // Price
        Element itemPriceElem = XMLCUtil.getRequiredElementById("itemPrice", newRow);
        XMLCUtil.getFirstText(itemPriceElem).setData(price.toString());
        
        // Total
        Element itemTotalElem = XMLCUtil.getRequiredElementById("itemTotal", newRow);
        XMLCUtil.getFirstText(itemTotalElem).setData(total.toString());

        // Delete checkbox, only in a dynamic table.
        if (dynamic) {
            HTMLInputElement itemDeleteElem = (HTMLInputElement)XMLCUtil.getRequiredElementById("itemDelete", newRow);
            itemDeleteElem.setName(DEL_PREFIX + item.getObjectId());
        }
        return newRow;
    }

    /**
     * Fill in table rows from cart and total cost.
     */
    private void fillInRows() {
        
try{
        Enumeration items = cart.getContents(); 
        HTMLTableRowElement costRow = htmlObj.getElementCostRow();
        Node parent = costRow.getParentNode();
       
        int rowNum = 1;
	while (items.hasMoreElements()) {
            HTMLTableRowElement newRow =
                makeItemRow(rowNum++, (CartItemPair)items.nextElement());
            parent.insertBefore(newRow, costRow);
        }

	BigDecimal total = new BigDecimal(cart.getTotal());
        total = total.setScale(2, BigDecimal.ROUND_HALF_UP);
        htmlObj.setTextCost(total.toString());
        //same thing
        } catch(NullPointerException ex) {
     }    
    }

    /**
     * Recursively search for a row and deleted it if it doesn't have an id
     * or if its the prototype row.
     */
    private void deleteDemoRows(Node node) {
        if (node instanceof HTMLTableRowElement) {
            String id = ((HTMLTableRowElement)node).getId();
            if ((id == null) || (id.length() == 0) || (node == protoRow)) {
                node.getParentNode().removeChild(node);
            }
        } else {
            // Not a row, search children.
            Node child = node.getFirstChild(); 
            while (child != null) {
                Node next = child.getNextSibling();
                deleteDemoRows(child);
                child = next;
                
            }
        }
    }

    /**
     * Constructor.  External instances are never created.
     *
     * @param tableHtmlObj Object with HTML table for items.
     * @param tableCart Cart used to generate the table.
     */
    private ContentsTableFormatter(ContentsTable tableHtmlObj, Cart tableCart,
                                   boolean isDynamic) {
        htmlObj = tableHtmlObj;
        cart = tableCart;
        dynamic = isDynamic;
        protoRow = htmlObj.getElementProtoRow();
        deleteDemoRows(htmlObj.getElementItemTable());
    }

    /**
     * File in the table object from a cart.
     */
    public static void fillInTable(ContentsTable tableHtmlObj, Cart tableCart, 
                                   boolean isDynamic) {
        ContentsTableFormatter table =
            new ContentsTableFormatter(tableHtmlObj, tableCart, isDynamic);
        table.fillInRows();
    }
}
