/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.presentation.xmlc.login;

import java.io.*;
import java.net.*;
import com.lutris.util.*;
import com.lutris.http.*;
import golfShop.GolfShop;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.user.*;
import golfShop.spec.user.*;
import golfShop.spec.LoginException;
/**
 * Presentation Object that processes the new account requests and redirects
 * to the appropriate page. It redirects to either the new account page
 * or the main page (if sucessfull).
 *
 * @author Andrew John
 * @version $Revision: 1.1 $
 */
public class AccountProcessor implements HttpPresentation {
    /**
     * State object in session.
     */
    private LoginState loginState;

    private static final String nousername = 
	"You must enter a username for the new account.";
    private static final String nopassword = 
	"You must enter the new password twice.";
    private static final String badpassword = 
	"Error entering password! Enter the new password twice.";
    private static final String badlogin = 
	"Incorrect username or password.";
    private static final String unknownhost = 
	"Login failed: Unknown host name.";
    private static final String noremotehost = 
	"Login failed: No remote host header.";
    private static final String loginfailed = 
	"Account creation failed: Unknown reason!";
    private static final String userexists = 
	"Account creation failed: Username is already in use.";
    private static final String badusername = 
	"That is not a valid user name.";
    private static final String toomanylogins = 
	"Account creation failed: Exceeded maximum number of sessions.";
    private static final String permdenied = 
	"Login failed: Permission denied.";
    private static final String disabled = 
	"Login failed: Account is disabled.";


    /**
     * If a string is null, make it into an empty string.
     */
    private String fixStr(String str) {
        if (str == null) {
            str = "";
        }
        return str;
    }


    /**
     * Entry point for presentation.
     */
    public void run(HttpPresentationComms comms)
		    throws IOException, PageRedirectException, Exception {
        loginState = LoginState.get(comms.session);

	GolfShop application = (GolfShop) comms.application;
	String fail_url = comms.request.getAppFileURIPath("login/NewAccount.po");
	String success_url = comms.request.getAppFileURIPath("main/Main.po");
	InetAddress[] inet = new InetAddress[1];

	// Get the username and passwords from the html form. 
	// These are mandatory fields. Redirect if nothing was entered,
	// or no data was sent, or the data is bad.
        String username = comms.request.getParameter("username");
	if (username == null) {
	    myRedirect(fail_url, true, nousername, null, comms);
        }
	if (username.length() == 0) {
	    myRedirect(fail_url, true, badusername, null, comms);
        }
        String password = comms.request.getParameter("password");
        String password2 = comms.request.getParameter("password2");
	if ((password == null) || (password2 == null)) {
	    myRedirect(fail_url, true, nopassword, username, comms);
        }
	if (!password.equals(password2)) {
	    myRedirect(fail_url, true, badpassword, username, comms);
        }
	if (password.length() == 0) {
	    myRedirect(fail_url, true, badpassword, username, comms);
        }

	// Now get the optional fields. Most users will leave these
	// blank for now, and fill them out at checkout time.
	// getParameter() willr return null if there was no html form
	// data passed back, so be paranoid.
	String address1 = fixStr(comms.request.getParameter("address1"));
	if (address1 == null) {
            address1 = "";
        }
	String address2 = fixStr(comms.request.getParameter("address2"));
	if (address2 == null) {
            address2 = "";
        }
	String city = fixStr(comms.request.getParameter("city"));
	String state = fixStr(comms.request.getParameter("state"));
	String zip = fixStr(comms.request.getParameter("zip"));
	String creditCard = fixStr(comms.request.getParameter("creditCard"));
	String email = fixStr(comms.request.getParameter("email"));
    
	//
	// For this application, future requests are only accepted from
	// the same ip address they logged in from.
	//
	String remoteHost = comms.request.getRemoteHost();
	// Be paranoid
	if (remoteHost == null) {
	    myRedirect(fail_url, true, noremotehost, username, comms);
        }
	inet[0] = null;
	try {
	    inet[0] = InetAddress.getByName(remoteHost);
	} catch (UnknownHostException uh) {
            // Will deal with below.
        }
	if (inet[0] == null) {
	    myRedirect(fail_url, true, unknownhost, username, comms);
        }

	try {
	    //
	    // Actually create the account (and log in). If it works, a new
	    // user will be created, then the user manager will create a new 
	    // session and add it to the session manager.
	    // It returns the session key, which is used to refer to the
	    // session. We need to send the key back to the browser in a
	    // cookie, so it will be passed back to this application with
	    // each future request.
	    //
	    GolfShopUserManager userManager = application.getUserManager();
	    userManager.createAccount(username, password, address1, 
			              address2, city, state, zip, creditCard, 
                                      email, comms.session);
	    // We have successfully logged in!
	    myRedirect(success_url, false, null, username, comms);
/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run GolfShop_pres ) so
 * we cannot create a new user
 * We need  to allow GolfShop_pres to be functional ,  response
 * will be default HTML page with message
 */ 
 
        } catch(NullPointerException ex) {
        
		myRedirect(fail_url, true, "You cannot register user while runing GolfShop_pres", username, comms);
	}catch (LoginException le) {
	    switch (le.reason) {
	    case GolfShopUserManager.UNKNOWN_ERROR:
		myRedirect(fail_url, true, loginfailed, username, comms);
		break;
	    case GolfShopUserManager.IO_ERROR:
		myRedirect(fail_url, true, badlogin, username, comms);
		break;
	    case GolfShopUserManager.AUTH_FAILED:
		myRedirect(fail_url, true, badlogin, username, comms);
		break;
	    case GolfShopUserManager.MULTIPLE_LOGIN:
		myRedirect(fail_url, true, toomanylogins, username, comms);
		break;
	    case GolfShopUserManager.PERMISSION_DENIED:
		myRedirect(fail_url, true, permdenied, username, comms);
		break;
	    case GolfShopUserManager.ACCOUNT_DISABLED:
		myRedirect(fail_url, true, disabled, username, comms);
		break;
	    case GolfShopUserManager.USERNAME_ALREADY_EXISTS:
		myRedirect(fail_url, true, userexists, username, comms);
		break;
	    default:
		myRedirect(fail_url, true, loginfailed, username, comms);
		break;
	    }
	}
    }

    /*
     * Redirect to a new page. If deny is true, then the denied message and
     * username is set in the state object.
     */
    private void myRedirect(String url, boolean deny, String msg, 
		String username, HttpPresentationComms comms) 
		throws HttpPresentationException {

	ClientPageRedirectException e = new ClientPageRedirectException(url);
	if (deny) {
            loginState.lastError = msg;
	    if ((username != null) && (username.length() > 0)) {
		loginState.userName = username;
            }
	}
        throw e;
    }
}
