/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package golfShop.presentation.xmlc.main;

import org.enhydra.xml.xmlc.*;
import org.enhydra.xml.xmlc.html.*;
import com.lutris.appserver.server.httpPresentation.*;
import java.io.*;
import java.net.URLEncoder;
import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import golfShop.presentation.xmlc.utilities.*;
import golfShop.spec.inventory.*;

/**
 * This presentation object dynamically creates the hierarchical shelf
 * (directory of categories and items in the store).  With categories either
 * open or closed.  This is a fairly ugly thing to do in HTML.
 *
 * Parameters:
 * <UL>
 * <LI> button - If a open/close button was pressed, this is the category
 *      id associated with that button.
 * <LI> open - A comma-seperated list of the catagory ids of the categories
 *      that are open.
 * </UL>
 */
public class Shelf implements HttpPresentation {
    /**
     * URL to reference ourselve.
     */
    private static final String SHELF_URL = "Shelf.po";

    /**
     * URL to reference the item presentation object.
     */
    private static final String ITEM_URL = "Item.po";

    /**
     * Image to use for open categories.
     */
    private static final String OPEN_CAT_IMAGE = "../media/true.gif";
    private static final String CLOSED_CAT_IMAGE = "../media/false.gif";

    /**
     * The catagory id of the button that was pressed or NULL if
     * one was not pressed.
     */
    private Long buttonCatId;

    /**
     * Table of category ids that are open.  Each entry contains a Long
     * object.
     */
    private Hashtable openCatIds = new Hashtable();

    /**
     * CGI argument containing all open category ids.  If null,
     * then none are open.
     */
    private String openCatCgiArg;
    
    /**
     * Shelf HTML object.
     */
    private ShelfHTML shelfHtml;
    
    /**
     * Pointers to category prototype entries.
     */
    HTMLParagraphElement sampleCatBlock;
    HTMLAnchorElement sampleCatAnchor;
    HTMLImageElement sampleCatImg;
    Text sampleCatText;

    /**
     * Pointers to item prototype entries.
     */
    HTMLParagraphElement sampleItemBlock;
    HTMLAnchorElement sampleItemAnchor;
    Text sampleItemText;
    
    /**
     * Remove prototype table and save pointers to the prototype
     * entries.  For easy of use, prototypes are edited and then
     * cloned.
     */
    private void prepareHtml(HttpPresentationComms comms) {
       
        shelfHtml = (ShelfHTML)comms.xmlcFactory.create(ShelfHTML.class);
       
        // Save pointers to prototypes, remove ids as they shouldn't be duplicated.
        sampleCatBlock = shelfHtml.getElementSampleCatBlock();
        sampleCatAnchor = shelfHtml.getElementSampleCatAnchor();
        sampleCatImg = shelfHtml.getElementSampleCatImg();
        sampleCatText = XMLCUtil.getFirstText(shelfHtml.getElementSampleCatText());

        sampleItemBlock = shelfHtml.getElementSampleItemBlock();
        sampleItemAnchor = shelfHtml.getElementSampleItemAnchor();
        sampleItemText = XMLCUtil.getFirstText(shelfHtml.getElementSampleItemText());

        // Remove children of prototype list.
        Node listNode = shelfHtml.getElementList();
        Node child = listNode.getFirstChild();
        while (child != null) {
            Node nextChild = child.getNextSibling();
            listNode.removeChild(child);
            child = nextChild;
        }
        
    }

    /**
     * Create a new HTML entry for a category.
     */
    private void makeCategoryHtml(Inventory category, Node parent,
                                  boolean isOpen, int level) {
        long id = category.getObjectId();

        String url = SHELF_URL + "?button=" + id;
        if (openCatCgiArg != null) {
            url += "&" + openCatCgiArg;
        }
        sampleCatAnchor.setHref(url);
        if (isOpen) {
            sampleCatImg.setSrc(OPEN_CAT_IMAGE);
        } else {
            sampleCatImg.setSrc(CLOSED_CAT_IMAGE);
        }
        sampleCatText.setData(category.getDesc());
        sampleCatBlock.setClassName("level" + level);
        parent.appendChild(sampleCatBlock.cloneNode(true));
    }

    /**
     * Create a new HTML entry for an item.
     */
    private void makeItemHtml(Inventory item, Node parent, int level) {
        String url = ITEM_URL + "?ref=../contents/" + item.getRef()
            + "&add=" + item.getObjectId();
        sampleItemAnchor.setHref(url);
        sampleItemAnchor.setTarget("item");
        sampleItemText.setData(item.getDesc());
        sampleItemBlock.setClassName("level" + level);
        parent.appendChild(sampleItemBlock.cloneNode(true));
    }

    /**
     * Recursively process a category or item entry, adding it to the DOM.
     * Sets node class using level for CSS.
     */
    private void processInventoryEntry(Inventory entry, Node parent, int level) {
        if (entry.isCategory()) {
            boolean isOpen = openCatIds.contains(new Long(entry.getObjectId()));
            makeCategoryHtml(entry, parent, isOpen, level);
            if (isOpen) {
                Vector entries = entry.getItems(entry.getObjectId());
                for (int idx = 0; idx < entries.size(); idx++) {
                    processInventoryEntry((Inventory)entries.elementAt(idx), parent, level+1);
                }
            }
        } else {
            makeItemHtml(entry, parent, level);
        }
    }

    /**
     * Generate the DOM entries for the current state of the inventory.
     */
    private void processInventory() {
    try{  
    Inventory inventory = InventoryFactory.getInventory("golfShop.business.inventory.InventoryImpl");
    processInventoryEntry(inventory.getInventoryById(0),
                              shelfHtml.getElementList(), 1); 
/*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer  when we run GolfShop_pres ) so
 * we cannot create a list of category 
 * We need  to allow GolfShop_pres to be functional 
 */ 
 
        } catch(NullPointerException ex) {
          
        }
    }

    /**
     * Parse CGI arguments into fields.
     */
    private void parseArgs(HttpPresentationRequest request)
        throws HttpPresentationException {

        // Parse button id.
        String button = request.getParameter("button");
        if (button != null) {
            try {
                buttonCatId = new Long(button);
            } catch (NumberFormatException except) {
                // Ignore, probably a bogus URL.
            }
        }
        
        String open = request.getParameter("open");
        if (open != null) {
            StringTokenizer tokens = new StringTokenizer(open, ",");
            while (tokens.hasMoreTokens()) {
                String token = tokens.nextToken();
                try {
                    Long id = new Long(token);
                    openCatIds.put(id, id);
                } catch (NumberFormatException except) {
                    // Ignore, probably a bogus URL.
                }
            }
        }
    }

    /**
     * Modify the open category list based on the currenmt button.
     * Also generate CGI argument for the open categories.
     */
    private void adjustOpenCategories() {
        if (buttonCatId != null) {
            // Adjust the open entry for the current button.
            if (openCatIds.containsKey(buttonCatId)) {
                openCatIds.remove(buttonCatId);
            } else {
                openCatIds.put(buttonCatId, buttonCatId);
            }
        }

        // Generate CGI arguments.
        StringBuffer buf = new StringBuffer();
        Enumeration openIds = openCatIds.keys();

        while (openIds.hasMoreElements()) {
            if (buf.length() > 0) {
                buf.append(",");
            }
            buf.append(((Long)openIds.nextElement()).toString());
        }
        if (buf.length() > 0) {
            openCatCgiArg = "open=" + buf.toString();
        }
    }


    /**
     * Entry.
     */
    public void run(HttpPresentationComms comms)
        throws HttpPresentationException {

        parseArgs(comms.request);
        prepareHtml(comms);
        adjustOpenCategories();
        processInventory();
        comms.response.writeDOM(shelfHtml);
    }
}
