package chat.business;

import java.util.Vector;
import java.util.Enumeration;


/**
 *  This class manages the contents of a discussion. Messages may be added
 *  at any time. There is an option limit on the size of the message queue,
 *  if so the oldest messages are deleted as new ones arrive. <P>
 * 
 *  Optionally the method startHarvester() may be called, once, to start
 *  a background thread that enforces an age limit on messages. If this
 *  method is not called, then there will be no age limit. <P>
 *
 *  This class implements what I call the "on hold push" algorithm.
 *  The server (this class) maintains some state (a list of messages).
 *  Clients (ContentsPresentation.po and the web browser) request snapshots of this state. 
 *  If the state has changed
 *  since the last time the client asked for a snapshot, then a new
 *  snapshot is sent to the client right away. If the state has not changed
 *  since the last time the client asked for a snapshot, then the request
 *  for a snapshot blocks until the state changes (either by a new message
 *  being added, or messages being deleted). As soon as the client has
 *  processed the snapshot it gets, it immediatly requests a new snapshot. <P>
 *
 *  This has the following advantages:
 *
 *    As soon as new information is available, it is sent immediatly to
 *    all interested parties.
 *
 *    Since a whole snapshot is sent every time, the problem of having
 *    to do "diffs" between the old and new states is avoided. As is the
 *    problem of how to recover when this mechanism breaks down.
 *
 *    Only standard HTTP is used, so the method is compatible with firewalls,
 *    proxies, all browsers, and even, in the case of an applet, 
 *    paranoid security policies. (HTTP requests are commonly
 *    blocked on a read in real life. But usually it is a filesystem read,
 *    and the delay is much shorter.) For an example of this, try running
 *    this application in the Enhydra Multiserver, and then add a WAI
 *    connection. Your requests are being tunneled via CORBA from the
 *    Netscape server to the Multiserver, yet the application runs fine.
 *
 *    Because the interested parties all have already established a socket
 *    connection, and are poised and ready to read data, they recieve the
 *    new state the instant it changes, pretty much as quickly as is possible.
 *
 *    It is simple, robust and effective. 
 * 
 *  This has the following disadvantages:
 *
 *    If you have N users, you will have N open socket connections. This
 *    method does not scale well to very large values of N. If N is too large,
 *    some users will experience network errors, and their automatic 
 *    reloading will stop. See your operating system for details on the
 *    maximum number of open sockets, and the optional NumThreads setting
 *    for connections in the Enhydra Multiserver config file.
 *
 *    The whole state is sent every time. This uses more bandwidth than
 *    methods that only send deltas to the state.
 *
 *    If the clients stop asking for snapshots, there is no way (from the
 *    server) to do anything about it.
 *  
 *  In order to tell if a client has a current copy of the state of things,
 *  timestamps are used. Every time the state changes, a unique identifier
 *  is assigned to the new current state (using time). The important thing
 *  is that the identifiers uniquely identify their state. Time is just
 *  one way of generating unique identifiers. Every time the client asks
 *  for a snapshot, it also sends the identifier for the state it currently
 *  has (there must be a special identifier used for the first request, when
 *  the client has no data. In this program it sends 0). If the id sent with
 *  the request matches, then the request blocks until the state changes.
 *
 *  In real life most connections to the server use TCP/IP, which imposes
 *  a timeout limit on how long requests can block on a read. 
 *  More importantly, web browsers will give up after a short period of time.
 *  Therefore, when a request is sent in, in addition to the current id a
 *  time limit is sent. This tells the server the maximum number of seconds
 *  the client is willing to wait (in this program 60 seconds is used).
 *  If this time limit expires, and the state has not yet changed, the
 *  server must return an answer to the client. In this program a snapshot
 *  is returned, even though the browser already has the same data 
 *  (if the client were an applet, a special "no change" response could be
 *  sent). If a client wants to force an update, or wants to do a 
 *  non-blocking read for some other reason, then it simply sets the timeout
 *  to zero.
 *
 *
 *  Static methods are used because there is only one chat room. This
 *  program's main goal is to show off a simple yet "real" Enhydra application.
 */
public class Discussion implements Runnable {

    // The current list of Message objects, in the order they were recieved.
    static private Vector contents = new Vector();

    // The total number of messages added to this discussion.
    static private long totalReceived = 0;
 
    // The unique identifier for the current state.
    static private long lastStateChangeTime = System.currentTimeMillis(); 

    // The current number of clients blocked on a read.
    static private int numWaiting = 0;

    // Only create at most one instance. This is only used by the thread.
    static private Discussion singleton;

    // The thread that deletes old messages. This will only be started if
    // startHarvester() is called (which is optional).
    static private Thread harvester;
 
    // Default settings for the thread. Will be overwritten by startHarvester().
    static private int lifetimeSec = 300;
    static private int intervalSec = 10;

    // Maximum allowed number of messages. 0 means no limit.
    public static int maxQueueSize = 200;
 

    /**
     *  Do not allow instances. Use the static methods.
     *  Only one instance is ever created, and that is just for the thread.
     */
    private Discussion() {
    }


    /**
     *  Add a message to the discussion. Any waiting clients will be
     *  instantly notified.
     */
    public static void addMessage(String name, String text) {
        MessageImpl msg = new MessageImpl(name, text);
        synchronized (contents) {
            // Add it to the list.
            totalReceived++;
            contents.addElement(msg);
            // If there are too many, delete the oldest ones.
            while ((maxQueueSize > 0) && (contents.size() > maxQueueSize))
                contents.removeElementAt(0);
            // Pick a new unique state identifier.
            // Wake up all the clients who are blocked on read.
            updateCurrentState();
        }
    }



    /**
     *  Create a new unique identifier for the current state. 
     *  Then wake up all the clients who are blocked on read.
     *  This uses the current time, but it could be based on the content, or
     *  a counter. The important thing is that the new identifier is one
     *  that has never been used before.
     */
    static private void updateCurrentState() {
        long now = System.currentTimeMillis();
        synchronized (contents) {
            // Be sure that the new time will be at least 1 msec greater 
            // than the old time.
            if (now <= lastStateChangeTime)
                // Extremely rare.
                lastStateChangeTime++;
            else
                // Most of the time.
                lastStateChangeTime = now;
            // Wake up any pending readers.
            contents.notifyAll();
        }
    }



    /**
     *  Throw out all the current messages. 
     *  All the waiting clients will be immediatly notified.
     */
    static public void clear() {
        synchronized (contents) {
            contents.removeAllElements();
            updateCurrentState();
        }
    }


    /**
     *  How many messages have been recieved in total?
     */
    static public long getTotalReceived() {
        return totalReceived;
    }


    /**
     *  How many messages are there right now in the list?
     */
    static public long getCurrentSize() {
        return contents.size();
    }


    /**
     *  Get a snapshot of the current state. Might block.
     *
     *  @param currentState
     *    The state identifier that was returned last time this was called.
     *    This is the id of the state that the client currently has. It is
     *    asking for a snapshot of the state after things change and
     *    this id is not longer current. If this has already happened, the
     *    call will immediatly return. If it has not happned, the call will
     *    block (not return) until things change.
     *  @param wait
     *    The maxmimum number of seconds this call is allowed to block for.
     *    Send 0 for an instant response. If the call blocks and then
     *    runs out of time, a snapshot is returned.
     *  @return
     *    A Snapshot object. This is just a way to return two things at
     *    once: a Vector of Message objects and a state identifier.
     *    The client should use the state identifier for the next call to
     *    this method. The client should call this method again as soon as
     *    it is done displaying the results.
     */
    public static SnapshotImpl getContents(long currentState, long wait) {
        waitForNewMessage(currentState, wait);
        return new SnapshotImpl((Vector) contents.clone(), lastStateChangeTime);
    }



    /**
     *  Helper function. Only does the waiting. Waits till browserState
     *  is not the current state, or for wait seconds, whichever comes
     *  first. This function will return immediatly if no waiting is
     *  needed. While waiting the thread is sleeping, not kept running
     *  in a loop (which would waste processor time).
     */
    private static void waitForNewMessage(long browserState, long wait) {
        if ((browserState != lastStateChangeTime) || (wait <= 0))
            // Already new data to report.
            return;
        long now = System.currentTimeMillis();
        long giveUpTime = now + (wait * 1000);
        // We loop here because the wait might return prematurely.
        while (browserState == lastStateChangeTime) {
            long leftToGo = giveUpTime - now;
            if (leftToGo <= 0)
                // Either the wait time was 0 or we ran out of time.
                break;
            synchronized (contents) {
                // Keep track of how many threads are waiting. This is done
                // by dead reckoning, so care must be taken to make sure the
                // counter doesn't get off.
                numWaiting ++;
                try {
                    contents.wait(leftToGo);
                } catch (InterruptedException e) {
                } finally {
                    numWaiting--;
                }
            }
            now = System.currentTimeMillis();
        }
    }


    /**
     *  How may clients are blocked on a read? This is, effectivly, the
     *  number of people participating in the discussion.
     */
    public static int getNumWaiting() {
        return numWaiting;
    }



    /**
     *  Internal use only. This is provided only for the thread that deletes
     *  messages that are too old.
     */
    public void run() {
        while (true) {
            /*
             *  Sleep for a while.
             */
            long now = System.currentTimeMillis();
            long wakeUp = now + (intervalSec * 1000);
            while (now < wakeUp) {
                long leftToGo = wakeUp - now;
                if (leftToGo <= 0) 
                    break;
                try {
                    Thread.sleep(leftToGo);
                } catch (InterruptedException e) {
                } 
                now = System.currentTimeMillis();
            }  
            /*
             *  Now make a pass through the message list and delete any
             *  that are too old. Keep track of whether or not we actually
             *  delete any.
             */
            boolean dirty = false;
            synchronized (contents) {
                long expires = System.currentTimeMillis() - 
                               (1000 * lifetimeSec);
                Enumeration e = contents.elements();
                while (e.hasMoreElements()) {
                    MessageImpl msg = (MessageImpl) e.nextElement();
                    if (msg == null)
                        continue;
                    if (msg.getWhen() < expires) { 
                        // We found an old one! Remove it.
                        contents.removeElement(msg);
                        dirty = true;
                    }
                }
            }
            /*
             *  If we just changed the contents of the list, then we
             *  need to get a new state identifier and wake up all the
             *  clients.
             */
            if (dirty) {
                synchronized (contents) {
                    updateCurrentState();
                }
            }
            // Go back to sleep...
        }
    }


    /**
     *  Call this function (only once) if you want to start the
     *  harverter thread. It runs in the background and periodically
     *  deletes messages that are too old. Most of the time the thread
     *  is sleeping. If this method is not called, then no age limit on
     *  messages will be enforced. <P>
     *
     *  The longest a message can live is lifetimeSec + intervalSec seconds
     *  (in the extreme case). <P>
     *
     *  The default value for the interval is a little shorter than the
     *  default value for the browser's timeout. This is so that if the
     *  browser is left idle (and messages start being deleted), the
     *  refresh cycle will sync up with this threads interval, and
     *  fewer updates will happen (delete, delete, delete... instead of
     *  timout, delete, timeout, delete, timeout, delete...).  It's not
     *  a big deal, but hey...
     *
     *  @param lifetimeSec
     *    How long should messages be kept for (seconds).
     *  @param 
     */ 
    static public void startHarvester(int lifetimeSec, int intervalSec) {
        // Be sure we don't start two threads.
        if ((harvester == null) || !harvester.isAlive()) {
            Discussion.lifetimeSec = lifetimeSec;
            Discussion.intervalSec = intervalSec;
            if (singleton == null)
                singleton = new Discussion();
            harvester = new Thread(singleton);
            harvester.start();
        }
    }
static public void stopHarvester(){
   if ((harvester != null) || harvester.isAlive()) {
             harvester.stop();
     }
    
}

}

     
