/*
 * Andy John
 * 2/99
 * Chat room
 * andy@enhydra.org
 * www.enhydra.org
 * 
 * Chat - A simple chat room Enhydra application.
 * 
 *  The purpose of this program is to demonstrate ways to use the Enhydra
 *  application framework. Jolt is used for two of the dynamic HTML
 *  pages. One of the pages is generated by a presentation object written
 *  by hand.
 * 
 *  This application uses what an algorithm I call "on hold push", because
 *  it gives you the effect of "push technology", but only uses standard
 *  HTTP. You instantly see when other people send in a message. But it
 *  will work through proxies and firewalls. It is even applicable to
 *  applets, which have additional security restrictions (the stand alone
 *  debugger uses this algorithm to show the requests made to the 
 *  servlet that is being debugged). See the Discussion object for a
 *  description of on hold push.
 *
 *  www.enhydra.org 
 */

package chat;

import com.lutris.appserver.server.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;

import chat.spec.*;

/**
 * The application object.
 * Reads all the settings from the config file.
 */
public class ChatApplication extends StandardApplication {

    private String roomName = "Chat Room";
    private String bgColor = "";
    private String clearCommand = "";

    /**
     *  Read all the chat room settings at startup time. Some of the
     *  settings are saved, and the other classes refer to this class when
     *  they need the values. Some of the settings are only used once to
     *  initialize parts of the program.
     */
    public void startup(Config appConfig) throws ApplicationException {
        super.startup(appConfig); // Be sure to call this!
        /*
         *  Get the title of this chat room from the config file.
         */
        try {
            roomName = appConfig.getString("RoomName", "Chat Room");
        } catch (ConfigException e) {
            throw new ApplicationException("Bogus RoomName in config file.", e);
        }
        /*
         *  How long should messages be kept? Get from config file.
         */
        int messageLifetimeSecs;
        try {
            messageLifetimeSecs = appConfig.getInt("MessageLifetimeSecs", 300);
        } catch (ConfigException e) {
            throw new ApplicationException(
                                "Bogus MessageLifetimeSecs in config file.", e);
        }
        /*
         *  How often to delete messages that are too old? 
         */
        int harvestIntervalSecs;
        try {
            harvestIntervalSecs = appConfig.getInt("HarvestIntervalSecs", 55);
        } catch (ConfigException e) {
            throw new ApplicationException("Bogus HarvestIntervalSecs in config file.", e);
        }
        /*
         *  Get the max number of messages allowed (if any) from the config
         *  file. You don't know how many hits you will get. If you allow
         *  data structures to grow without limit, your application may
         *  use up all the memory and kill the virtual machine.
         */
        int maxQueueSize;
        try {
            maxQueueSize = appConfig.getInt("MaxQueueSize", 200);
        } catch (ConfigException e) {
            throw new ApplicationException(
                                       "Bogus MaxQueueSize in config file.", e);
        }
        /*
         *   Get the background color string (if any) from the config file.
         */
        try {
            bgColor = appConfig.getString("BackgroundColor", "#99CCFF");
        } catch (ConfigException e) {
            throw new ApplicationException(
                                    "Bogus BackgroundColor in config file.", e);
        }
        /*
         *  Get the clear command (if any) from the config file.
         */
        try {
            clearCommand = appConfig.getString("ResetCommand", "");
        } catch (ConfigException e) {
            throw new ApplicationException(
                                    "Bogus ResetCommand in config file.", e);
        }
        /*
         *  Configure the DiscussionImpl object.
         *  Maybe start the message deleting thread.
         */
        
        DiscussionManager discussionManager = DiscussionManagerFactory.getDiscussionManager("chat.business.DiscussionManagerImpl");
      
 /*
 * Catch Null pointer exception ( we canot make a instances of classes from business layer when we run chat_pres )
 * We need  to allow chat_pres to be functional 
 */     
  try{
        discussionManager.setMaxQueueSize(maxQueueSize);
      
        if (harvestIntervalSecs < 1)
            harvestIntervalSecs = 1;
        if (messageLifetimeSecs > 0) 
        
           discussionManager.startHarvester(messageLifetimeSecs, harvestIntervalSecs);
 
    }catch(NullPointerException e){}
 
    }

 
    public boolean requestPreprocessor(HttpPresentationComms comms)
                   throws Exception {
        return super.requestPreprocessor(comms);
    }


public void shutdown(){
   DiscussionManager discussionManager = DiscussionManagerFactory.getDiscussionManager("chat.business.DiscussionManagerImpl");
    try{
        discussionManager.stopHarvester();
    }catch(NullPointerException e){}
   }
    /**
     *  The title of this chat room.
     */
    public String getRoomName() {
        return roomName;
    }


    /**
     *  What color should the pages be? This is used by both the entry frame
     *  and the contents frame.
     */
    public String getBgColor() {
        return bgColor;
    }

    /**
     *  What's the clear command? This will be "" if there is no command.
     *  If this string is sent in (in a message), then delete all the 
     *  messages currently in the queue.
     */
    public String getClearCommand() {
        return clearCommand;
    }


    /**
     *  This is only called if this application is being run in the
     *  Enhydra Multiserver. 
     *  Displays usage stats that an administrator would care about.
     *  The number of listeners is the number of requets blocked
     *  on a read, waiting for messages to appear or be deleted (often
     *  web browsers do "keep alive" behind the scenes, even if the users
     *  hit "stop", this might not go to zero immediatly).
     * 
     * @return HTML that is displayed in the status page of the Multiserver.
     */
    public String toHtml() {
        String response;
        
      
        
         DiscussionManager discussionManager = DiscussionManagerFactory.getDiscussionManager("chat.business.DiscussionManagerImpl");
         int n = discussionManager.getNumWaiting();
        
        if (n == 1)
            response = "There is 1 person listening.";
        else
            response = "There are " + n + " people listening.";
      
       long l = discussionManager.getTotalReceived();
       
        if (l == 1)
            response += "<br>A total of 1 message has been received so far.";
        else
            response += "<br>A total of " + l + " messages have been received.";
   
          l = discussionManager.getCurrentSize();
        if (l == 1)
            response += "<br>There is currently 1 message.";
        else
            response += "<br>There are currently " + l + " messages.";
        return response;
    }

}

