/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DiscImpl.java,v 1.1 2004/08/16 09:39:19 slobodan Exp $
 */

package discRack.business.disc;

import discRack.spec.*;
import discRack.data.person.PersonDO;
import discRack.data.disc.DiscDO;

import discRack.business.DiscRackBusinessException;

import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.appserver.server.sql.ObjectIdException;
import com.lutris.dods.builder.generator.query.DataObjectException;
import org.enhydra.dods.exceptions.AssertionDataObjectException;

/**
 * Represents a disc. 
 */
public class DiscImpl implements Disc{
    /**
     * The DO of the disc.
     */
    protected DiscDO myDO = null;

    /**
     * The public constructor.
     */
    public DiscImpl() throws DiscRackBusinessException {
        try {
            this.myDO = DiscDO.createVirgin();  
        } catch(DatabaseManagerException ex) {
            throw new DiscRackBusinessException("Error creating empty Disc", ex);
        } catch(ObjectIdException ex) {
            throw new DiscRackBusinessException("Error creating empty Disc", ex);
        }
    }

    /** The protected constructor
     *
     * @param theDisc The data object of the disc.
     */
    protected DiscImpl(DiscDO theDisc) 
        throws DiscRackBusinessException  {
        this.myDO = theDisc;
    }

    /**
     * Gets the object id for the disc
     *
     * @return the object id.
     * @exception DiscRackBusinessException if an error occurs
     *   retrieving data (usually due to an underlying data layer
     *   error).
     */
    public String getHandle()
        throws DiscRackBusinessException {
        try {
            return this.myDO.getHandle();
        } catch(DatabaseManagerException ex) {
            throw new DiscRackBusinessException("Error getting disc's handle", ex);
        }
    }

    /**
     * Gets the title for the disc
     *
     * @return the title.
     * @exception DiscRackBusinessException if an error occurs
     *   retrieving data (usually due to an underlying data layer
     *   error).
     */
    public String getTitle() 
        throws DiscRackBusinessException {
        try {
            return myDO.getTitle();   
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error getting disc's title", ex);
        }
    }
    
    /**
     * Gets the artist for the disc
     *
     * @return the artist.
     * @exception DiscRackBusinessException if an error occurs
     *   retrieving data (usually due to an underlying data layer
     *   error).
     */
    public String getArtist()
        throws DiscRackBusinessException {
        try {
            return myDO.getArtist();   
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error getting disc's artist", ex);
        }
    }
    
    /**
     * Gets the genre for the disc
     *
     * @return the genre.
     * @exception DiscRackBusinessException if an error occurs
     *   retrieving data (usually due to an underlying data layer
     *   error).
     */
    public String getGenre()
        throws DiscRackBusinessException {
        try {
            return myDO.getGenre();   
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error getting disc's genre", ex);
        }
    }
    
    /**
     * Gets the preference for the disc
     *
     * @return true if like the disc, false if not
     * @exception DiscRackBusinessException if an error occurs
     *   retrieving data (usually due to an underlying data layer
     *   error).
     */
    public boolean isLiked()
        throws DiscRackBusinessException {
        try {
            return myDO.getIsLiked();   
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error getting disc's likedness", ex);
        }
    }
    
    /**
     * Sets the title for the disc.
     *
     * @param  title
     * @exception DiscRackBusinessException if an error occurs
     *   setting the data (usually due to an underlying data layer
     *   error).
     */
    public void setTitle(String title) 
        throws DiscRackBusinessException {
        try {
            this.myDO .setTitle(title);
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error setting disc's title", ex);
        }
    }
    
    /**
     * Sets the artist for the disc.
     *
     * @param  artist
     * @exception DiscRackBusinessException if an error occurs
     *   setting the data (usually due to an underlying data layer
     *   error).
     */
    public void setArtist(String artist) 
        throws DiscRackBusinessException {
        try {
            this.myDO.setArtist(artist);
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error setting disc's artist", ex);
        }
    }
    
    /**
     * Sets the genre for the disc.
     *
     * @param  genre
     * @exception DiscRackBusinessException if an error occurs
     *   setting the data (usually due to an underlying data layer
     *   error).
     */
    public void setGenre(String genre) 
        throws DiscRackBusinessException {
        try {
            this.myDO.setGenre(genre);
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error setting disc's genre", ex);
        }
    }
    
    /**
     * Sets the owner for the disc.
     *
     * @param owner
     * @exception DiscRackBusinessException if an error occurs
     *   setting the data (usually due to an underlying data layer
     *   error).
     */
    public void setOwner(Person owner) 
        throws DiscRackException
    {
        try {
            this.myDO.setOwner(PersonDO.createExisting(owner.getHandle()));
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error setting disc's owner", ex);
        }
    }
    
    /**
     * Sets the preference for the disc.
     *
     * @param isLiked
     * @exception DiscRackBusinessException if an error occurs
     *   setting the data (usually due to an underlying data layer
     *   error).
     */
    public void setLiked(boolean isLiked) 
        throws DiscRackBusinessException {
        try {
            this.myDO.setIsLiked(isLiked);
        } catch(DataObjectException ex) {
            throw new DiscRackBusinessException("Error setting disc's likedness", ex);
        }
    }
    

    /**
     * Commits all changes to the database.
     *
     * @exception DiscRackBusinessException if an error occurs
     *   retrieving data (usually due to an underlying data layer
     *   error).
     */
    public void save() 
        throws DiscRackBusinessException, AssertionDataObjectException {
        try {
            this.myDO.commit();
        } 
        catch (AssertionDataObjectException ex) {
            throw new AssertionDataObjectException("Read-only table: DML operations not allowed", ex);
        } 
        catch(Exception ex) {
            throw new DiscRackBusinessException("Error saving disc", ex);
        }
    }
    
    /**
     * Deletes the disc from the database.
     *
     * @exception DiscRackBusinessException if an error occurs
     *   deleting data (usually due to an underlying data layer
     *   error).
     */
    public void delete() 
        throws DiscRackBusinessException, AssertionDataObjectException {
        try {
            this.myDO.delete();
        }
        catch (AssertionDataObjectException ex) {
            throw new AssertionDataObjectException("Read-only table: DML operations not allowed", ex);
        } 
        catch(Exception ex) {
            throw new DiscRackBusinessException("Error deleting disc", ex);
        }
    }
}
