/* -----------------------------------------------------------------------------
 * Copyright (c) 2001, Low Kin Onn
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither name of the Scioworks Pte. Ltd. nor the names of its contributors
 * may beused to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * -----------------------------------------------------------------------------
 */

package scioworks.imap.presentation.base;

import java.util.Vector;

import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.util.KeywordValueException;

import org.w3c.dom.*;
import org.w3c.dom.html.*;



import scioworks.imap.presentation.ImapWebSessionData;
import scioworks.imap.presentation.*;
import scioworks.imap.presentation.KMSPresentationException;

public abstract class BasePO
    implements HttpPresentation, BaseParameters, BaseEvents, BaseMessages, BaseURLs {

  protected static final int m_firstEntryIndex = 1;
  protected static final int m_pageLength = 10;

  protected HttpPresentationComms m_comms       = null;
  protected ImapWebSessionData    m_ImapWebSessionData = null;

  abstract public String handleDefault()
    throws HttpPresentationException, ServerPageRedirectException;

  /**
   * Default required login, override otherwise
   */
  protected boolean isLoginRequired() {
    return true;
  }

  /**
    * @return The saved comms objects
    * to whichever subclass needs it
    */
  public HttpPresentationComms getComms() {
    return m_comms;
  }

  public ImapWebSessionData getImapWebSessionData() {
    return m_ImapWebSessionData;
  }

  public void setSessionData(ImapWebSessionData sessionData) {
    this.m_ImapWebSessionData = sessionData;
  }

  protected void initSessionData(HttpPresentationComms comms)
    throws KMSPresentationException {

    this.m_comms = comms;
    try {

      // try to get the imapWeb session data (if any)
      Object obj = getComms().sessionData.get(ImapWebSessionData.SESSION_KEY);

      // If we found the session data, save it in a private data member
      if(null != obj) {
        m_ImapWebSessionData = (ImapWebSessionData) obj;

      } else {

        // If no session data was found, create a new session data instance
        this.m_ImapWebSessionData = new ImapWebSessionData(null);
        getComms().sessionData.set(ImapWebSessionData.SESSION_KEY, this.m_ImapWebSessionData);
      }
    } catch (KeywordValueException ex) {
      System.out.println("Problem getting session data from session: " +
      ex.getMessage());
    }
  }

  /**
    * Checks the session data for a User, if not there then redirects to the login page
    */
  protected void checkForUserLogin()
    throws KMSPresentationException {

    try {
      if (!m_ImapWebSessionData.isValidSession()) {

         String requestedPO = getComms().request.getRequestURI();
         getComms().response.writeHTML(
             this.showErrorPage("Session Expired",
                                "You session has expired. Please re-login",
                                "Re-login", URL_login, "_parent"));
      } else {
        //System.out.println("USER ALREADY LOGGED IN WITH A SESSION");
      }
    } catch (Exception ex) {
      throw new KMSPresentationException("Trouble checking for user login status", ex);
    }
  }

  public void run(HttpPresentationComms comms)
      throws KMSPresentationException {

    // Initialize new or get the existing session data
    initSessionData(comms);

    // Check if the user needs to be logged in for this request.
    if(isLoginRequired()) {
      checkForUserLogin();
    }

    try {
      // Handle the incoming event request
      handleEvent(comms);
    } catch (HttpPresentationException e) {
      Logger.writeError("BasePO::run", e);
      throw new KMSPresentationException("An error has occured. "+
        "Please contact your administrator. Error message: "+e.getMessage());
    }

  }

  public void handleEvent(HttpPresentationComms comms)
      throws HttpPresentationException {

    String returnHTML = null;

    try {
      if (this.getStringParameter(PARAM_imageData).equals("y")) {
        String returnMimeType = handleDefault();
        comms.response.setContentType(returnMimeType);
        return;

      } else {
        returnHTML = handleDefault();
      }
    } catch (Exception e) {
      returnHTML = this.showErrorPage("Internal Error",
                  "An error has occured. Please contact your administrator. "+
                  "Error message: "+e.getMessage(),
                  "", "");
      Logger.writeError("BasePO::handleEvent", e);
    }

    getComms().response.writeHTML(returnHTML);
  }

  protected long getLongParameter(String parm)
      throws HttpPresentationException, NumberFormatException {
    long res = 0;
    try {
      res = Long.parseLong(m_comms.request.getParameter(parm));
    } catch (NumberFormatException e) {
      res = -1;
    }
    return res;
  }

  protected int getIntParameter(String parm)
      throws HttpPresentationException, NumberFormatException {
    int res = 0;
    try {
      res = Integer.parseInt(m_comms.request.getParameter(parm));
    } catch (NumberFormatException e) {
      res = -1;
    }
    return res;
  }

  protected String getStringParameter(String parm)
      throws HttpPresentationException {
    String value = m_comms.request.getParameter(parm);
    if ((value == null) || (value.length() == 0)) {
      value = "";
    }
    return value;
  }

  protected String[] getStringParameterArray(String parm)
      throws HttpPresentationException {
    return m_comms.request.getParameterValues(parm);
  }

  protected void setStringParameter(String name, String value)
    throws HttpPresentationException {

    m_comms.response.setHeader(name, value);
  }

  protected long[] getLongParameterArray(String parm)
      throws HttpPresentationException {
    String[] strArr = m_comms.request.getParameterValues(parm);
    long[] longArr = new long[strArr.length];
    for (int i=0; i<strArr.length; i++) {
      longArr[i] = Long.parseLong(strArr[i]);
    }
    return longArr;
  }

  protected String showErrorPage(String errorHeader, String errorText,
      String errorLinkText, String errorLink, String target) {
    errorPageHTML page = (errorPageHTML)m_comms.xmlcFactory.create(errorPageHTML.class);
    page.setTextErrorHeader(errorHeader);
    page.setTextErrorText(errorText);
    page.setTextErrorLinkText(errorLinkText);

    HTMLAnchorElement fileLink;
    fileLink = page.getElementErrorLink();
    if(errorLinkText.equals("") || errorLink.equals("")) {
      this.removeElement(page, fileLink);
    } else {
      fileLink.setHref(errorLink);
      if(!target.equals("")) {
        fileLink.setTarget(target);
      }
    }
    return page.toDocument();
  }

  protected String showErrorPage(String errorHeader, String errorText,
      String errorLinkText, String errorLink) {
    return showErrorPage(errorHeader, errorText, errorLinkText, errorLink, "");
  }

  protected String showCompletionPage(String header, String text,
      String linkText, String link) {
    messagePageHTML page = (messagePageHTML)m_comms.xmlcFactory.create(messagePageHTML.class);
    page.setTextMessageHeader(header);
    page.setTextMessageText(text);
    page.setTextMessageLinkText(linkText);

    HTMLAnchorElement fileLink = page.getElementMessageLink();
    if(linkText.equals("") || link.equals("")) {
      this.removeElement(page, fileLink);
    } else {
      fileLink.setHref(link);
      /*
      if(!target.equals("")) {
        fileLink.setTarget(target);
      }
      */
    }
    return page.toDocument();
  }

  protected void removeElement(org.enhydra.xml.xmlc.html.HTMLObjectImpl page,
                               Element currElement) {
    if (currElement != null) {
      Node parentNode = currElement.getParentNode();
      parentNode.removeChild(currElement);
    } else {
        Logger.writeDebug("Error: unfound element "+currElement.getLocalName());
    }
  }

  protected void removeElement(org.enhydra.xml.xmlc.html.HTMLObjectImpl page,
                               java.util.HashSet elementSet) {
    Element currElement;
    java.util.Iterator it = elementSet.iterator();
    Node parentNode;
    while (it.hasNext()) {
      currElement = (Element) it.next();
      if (currElement != null) {
        parentNode = currElement.getParentNode();
        parentNode.removeChild(currElement);
      } else {
        Logger.writeDebug("Error: unfound element "+currElement.getLocalName());
      }
    }
  }

  protected void processSelectList(org.enhydra.xml.xmlc.html.HTMLObjectImpl page,
      HTMLSelectElement lookUpElement, Vector displayList, Vector valueList,
      String selectedItem) {

    // remove the hard coded drop down elements
    for (int i=lookUpElement.getLength()-1; i>=0; i--) {
      lookUpElement.remove(i);
    }

    for(int i=0; i<valueList.size(); i++) {
      // Create and append options, set to select if matches value
      HTMLOptionElement o1 = (HTMLOptionElement) page.createElement("OPTION");
      o1.setValue((String)valueList.elementAt(i));
      o1.appendChild(page.createTextNode((String)displayList.elementAt(i)));
      if (selectedItem.equals((String)valueList.elementAt(i))) {
        o1.setAttribute("selected","selected");
      }
      lookUpElement.appendChild(o1);
    }
  }
}
