/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: BasePO.java,v 1.1 2005/12/28 12:01:09 slobodan Exp $
 */

package jtaDiscRack.presentation;

import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.Enhydra;
import com.lutris.logging.*;
import com.lutris.util.KeywordValueException;

import org.enhydra.xml.xmlc.XMLObject;

import java.lang.reflect.*;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.transaction.UserTransaction;

import jtaDiscRack.spec.*;
import jtaDiscRack.*;

/**
 * This is the parent Presentaion object. All presentation objects should extend
 * this class.
 * 
 * The run method looks for an event parameter and then calls handle<EventName>.
 * If the "event" Parameter is not defined then the handleDefault() method is
 * called in your child class.
 * 
 * 
 */
public abstract class BasePO implements HttpPresentation {

	protected static final String USER_KEY = "DiscRackPerson";

	protected static String LOGIN_PAGE = "personMgmt/Login.po";

	protected static String DISC_CATALOG_PAGE = "discMgmt/DiscCatalog.po";

	private static String EVENT = "event";

	private static String STANDARD_METHOD_PREFIX = "handle";

	/**
	 * This is the procedure that is called if there is no "event" HTTP
	 * parameter found. It must be overriden by the subclass to do default
	 * processing or error checking/handling.
	 * 
	 * @return String The String representation of the HTML or (other format) of
	 *         the document to be displayed. This method would need to be
	 *         changed if you wanted to return binary data as well. It returns a
	 *         String for simplicity right now.
	 */
	public abstract XMLObject handleDefault() throws HttpPresentationException;

	/**
	 * This method should be implemented in the subclass so that it returns true
	 * if this particular request requires the user to be logged in, otherwise
	 * false.
	 */
	protected abstract boolean loggedInUserRequired();

	/**
	 * Saved input and output context, and session data
	 */
	protected HttpPresentationComms myComms = null;

	/**
	 * Session Data object reference
	 */
	protected JtaDiscRackSessionData mySessionData = null;

	/**
	 * Transaction object reference
	 */
	protected UserTransaction transaction = null;

	/**
	 * Gets HttpPresentation object
	 * 
	 * @return The saved comms objects to whichever subclass needs it
	 */
	public HttpPresentationComms getComms() {
		return this.myComms;
	}

	/**
	 * Gets the session data
	 * 
	 * @return session data
	 */
	public JtaDiscRackSessionData getSessionData() {
		return this.mySessionData;
	}

	/**
	 * Sets the user into the session
	 * 
	 * @param thePerson
	 *            the person to be set in the session
	 * @exception DiscRackPresentationException
	 */
	public void setUser(Person thePerson)
			throws JtaDiscRackPresentationException {
		this.getSessionData().setUser(thePerson);
	}

	/**
	 * Gets the user from the session
	 * 
	 * @return the person object in the session
	 */
	public String getUserHandle() {
		return this.getSessionData().getUserHandle();
	}

	/**
	 * Method to remove the current user from the session
	 */
	public void removeUserFromSession() {
		this.getSessionData().removeUser();
	}

	/**
	 * This implements the run method in HttpPresentation.
	 * 
	 * @param comms
	 *            HttpPresentationComms
	 * @exception Exception
	 */
	public void run(HttpPresentationComms comms) throws Exception {
		boolean doCommit = true;
		try {
			// Initialize new or get the existing session data
			initSessionData(comms);

			initTransaction();

			// Check if the user needs to be logged in for this request.
			if (this.loggedInUserRequired()) {
				checkForUserLogin();
			}
			// Handle the incoming event request
			handleEvent(comms);
		} catch (Exception ex) {
			doCommit = false;
			throw new Exception(ex);
		} finally {
			if (doCommit) {
				commitTransaction();
			} else {
				rollbackTransaction();
			}
		}
	}

	/**
	 * Method to get or create the AgSessionData object from the user session
	 * This object is saved in the EbrokerPresentation object
	 * 
	 * @param comms
	 *            HttpPresentationComms
	 * @exception Exception
	 */
	protected void initSessionData(HttpPresentationComms comms)
			throws JtaDiscRackPresentationException {
		this.myComms = comms;

		try {
			Object obj = comms.sessionData
					.get(JtaDiscRackSessionData.SESSION_KEY);
			// If we found the session data, save it in a private data member
			if (null != obj) {
				this.mySessionData = (JtaDiscRackSessionData) obj;
			} else {
				// If no session data was found, create a new session data
				// instance
				this.mySessionData = new JtaDiscRackSessionData();
				comms.sessionData.set(JtaDiscRackSessionData.SESSION_KEY,
						this.mySessionData);
			}
		} catch (KeywordValueException ex) {
			writeDebugMsg("Problem getting session data from session: "
					+ ex.getMessage());
		}
	}

	/**
	 * Method initializes DB transation instance
	 */
	protected void initTransaction() throws Exception {
		Context ctx = new InitialContext();
		transaction = (UserTransaction) ctx
				.lookup(UserTransactionJNDI.jNDIName);
		if (transaction != null) {
			transaction.begin();
		}
	}

	/**
	 * Method commits DB transation instance
	 */
	protected void commitTransaction() throws Exception {
		if (transaction!=null){
			transaction.commit();
		}

	}

	private void rollbackTransaction() throws Exception {
		if (transaction!=null){
			transaction.rollback();
		}
	}

	/**
	 * Checks the session data for a User, if not there then redirects to the
	 * login page
	 */
	protected void checkForUserLogin() throws ClientPageRedirectException,
			JtaDiscRackPresentationException {

		try {
			String userHandle = getUserHandle();

			if (null == userHandle || "".equals(userHandle)) {
				writeDebugMsg("USER NOT FOUND IN SESSION");
				// send to LoginPage if a logged in user is required.
				writeDebugMsg("REDIRECTING TO LOGIN PAGE");
				throw new ClientPageRedirectException(getComms().request
						.getApplicationPath()
						+ LOGIN_PAGE);
			}
		} catch (Exception ex) {
			throw new JtaDiscRackPresentationException(
					"Trouble checking for user login status", ex);
		}
	}

	/**
	 * Method to call the proper method for the incoming event
	 * 
	 * @param comms
	 *            HttpPresentationComms
	 * @exception Exception
	 */
	public void handleEvent(HttpPresentationComms comms) throws Exception {
		String event = comms.request.getParameter(EVENT);

		XMLObject returnHTML = null;

		if (event == null || event.length() == 0) {
			returnHTML = handleDefault();
		} else {
			returnHTML = getPageContentForEvent(event);
		}
		comms.response.writeDOM(returnHTML);
	}

	/**
	 * If an event parameter is defined then this invokes the method that
	 * handles that event.
	 * 
	 * @param event
	 *            the incoming event name
	 * @exception Exception
	 */
	public XMLObject getPageContentForEvent(String event) throws Exception {
		try {
			Method method = this.getClass()
					.getMethod(toMethodName(event), null);
			XMLObject thePage = (XMLObject) method.invoke(this, null);
			return thePage;

		} catch (InvocationTargetException ex) {
			// Rethrow the originating exception if as it should be propagated
			// as is
			// It could be a page redirect exception, etc.
			if (ex.getTargetException() instanceof Exception) {
				throw (Exception) ex.getTargetException();
			} else if (ex.getTargetException() instanceof Error) {
				throw (Error) ex.getTargetException();
			} else {
				throw ex;
			}
		} catch (NoSuchMethodException ex) {
			// The method to handle the event does not exist.
			throw new JtaDiscRackPresentationException(
					"NO EVENT HANDLER FOUND FOR EVENT: " + event, ex);
		} catch (IllegalAccessException ex) {
			// The method to handle the event does not exist.
			throw new JtaDiscRackPresentationException(
					"ILLEGAL ACCESS TO EVENT HANDLER (is it public?): " + event,
					ex);
		}
	}

	/**
	 * This sets the first letter of the event parameter value in order to
	 * adhere to Java method naming conventions.
	 * 
	 * @param event
	 *            the incoming name of the event
	 * @return String the properly capitalized name
	 */
	private String toMethodName(String event) {
		StringBuffer methodName = new StringBuffer(STANDARD_METHOD_PREFIX);
		methodName.append(Character.toUpperCase(event.charAt(0)));

		if (event.length() > 1) {
			methodName.append(event.substring(1));
		}

		return methodName.toString();
	}

	/**
	 * Returns the application object associated with the current request.
	 * 
	 * @return the application object.
	 */
	public JtaDiscRack getApplication() {
		return (JtaDiscRack) Enhydra.getApplication();
	}

	/**
	 * Method to write a debugging message to the debug log channel when the
	 * DEBUG flag is turned on
	 * 
	 * @param msg
	 *            The message to write to the DEBUG log channel
	 */
	public static void writeDebugMsg(String msg) {
		Enhydra.getLogChannel().write(Logger.DEBUG, msg);
	}
}
