/**
 * Title:        OpenUSS - Open Source University Support System
 *               My Piggy Bank Example
 * Description:  Enhydra Presentation Object
 * Copyright:    Copyright (c) 2003 by B. Lofi Dewanto, T. Menzel
 * Company:      University of Muenster, HTWK Leipzig
 * @author  B. Lofi Dewanto, T. Menzel
 * @version 1.1
 */
package net.sourceforge.ejosa.piggybank.presentation.enhydra;

import com.lutris.appserver.server.Enhydra;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.user.User;

import com.lutris.logging.*;

import com.lutris.util.KeywordValueException;

//import com.lutris.xml.xmlc.*;
//import com.lutris.xml.xmlc.html.*;

import java.lang.Throwable;
import java.lang.reflect.*;

import java.text.*;

import java.util.*;

import org.openuss.presentation.enhydra.framework.*;

import org.openuss.utility.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;


/**
 * The base presentation object for foundation components.
 * This should be used to show a html file.
 *
 * @author  B. Lofi Dewanto, T. Menzel
 * @version 1.1
 */
abstract public class FoundationPO extends BasePO implements HttpPresentation,
                                                             BaseLayout {
    protected static String MAIN_PAGE = "/Welcome.po";

    /**
     * Saved the session data.
     */
    protected FoundationSessionData mySessionData = null;

    /**
     * @return The saved comms objects
     * to whichever subclass needs it
     */
    public FoundationSessionData getSessionData() {
        return this.mySessionData;
    }

    /**
     * Method to remove the all session data.
     */
    public void removeAllSessionData() {
        this.getSessionData().getAndClearUserMessage();
    }

    /**
     * Method to log the locale in to a session.
     */
    public void setLocale(Locale locale) throws FoundationPOException {
        this.getSessionData().setLocale(locale);
    }

    /**
     * Get the locale.
     */
    public Locale getLocale() {
        return this.getSessionData().getLocale();
    }

    /**
     * Superclass method override. For public means everybody can
     * access this, otherwise he or she needs to be activated first.
     */
    public boolean isForPublicAccess() throws BasePOException {
        // As a standard value, every pages are for the public
        // access -> true
        // You have to override this, if you want to change
        // this behaviour
        return true;
    }

    /**
     * This implements the run method in HttpPresentation.
     */
    public void run(HttpPresentationComms comms) throws Exception {
        // Initialize new or get the existing session data
        initSessionData(comms);


        // Check the security
        checkSecurity();


        // Handle the incoming event request
        handleEvent(comms);
    }

    /**
     * Method to get or create the AgSessionData object from the user session
     * This object is saved in the EbrokerPresentation object
     */
    protected void initSessionData(HttpPresentationComms comms)
                            throws FoundationPOException {
        this.myComms = comms;

        try {
            Object obj = comms.sessionData.get(
                                 FoundationSessionData.SESSION_KEY);

            // If we found the session data, save it in a private data member
            if (null != obj) {
                this.mySessionData = (FoundationSessionData) obj;
            } else {
                // If no session data was found, create a new session data instance
                this.mySessionData = new FoundationSessionData();
                comms.sessionData.set(FoundationSessionData.SESSION_KEY, 
                                      this.mySessionData);
            }
        } catch (KeywordValueException ex) {
            writeDebugMsg("Problem getting session data from session: " + 
                          ex.getMessage());
        }
    }

    /**
     * Checks the session data for a User, if not there then redirects to the login
     * page.
     */
    protected void checkForUserLogin() throws ClientPageRedirectException, 
                                              FoundationPOException {
        // Do nothing
    }

    /**
     * Checks the session data for a AccessList, if not there then redirects
     * to the public login page, where the user can apply for the access.
     */
    protected void checkForAccessList() throws ClientPageRedirectException, 
                                               FoundationPOException {
        // Do nothing
    }

    /**
     * Check the security.
     */
    protected void checkSecurity() throws Exception {
        // Check if the user needs to be logged in for this request
        if (this.loggedInUserRequired()) {
            checkForUserLogin();
        }
    }

    /**
     * This method has to be implemented. This write down the right-side
     * information on the right side of the page. "Treebar".
     */
    public void showTreebar(Object page) throws BasePOException {
        // Do nothing
    }

    /**
     * This method has to be implemented. This updates the date on
     * the top right.
     */
    public void showUpdatedDate(Object page) throws BasePOException {
        // Do nothing
    }

    /**
     * This method has to be implemented. This shows all the page
     * layout. All layout methods should be called within
     * this method.
     */
    public String showLayout(Object page) throws BasePOException {
        // Return nothing
        return null;
    }
}