/**
 * Title:        OpenUSS - Open Source University Support System
 * Description:  Utility for OpenUSS
 * Copyright:    Copyright (c) B. Lofi Dewanto
 * Company:      University of Muenster
 * @author  B. Lofi Dewanto
 * @version 1.0
 */
package org.openuss.utility;

import java.io.PrintWriter;

/**
 * Root exception for all exceptions available. 
 * This code was taken from the article at the Java World
 * Java World Tip 91 by Terren Suydam (c)
 * "Use Nested Exception in a Multitiered Environment"
 *
 * @author  B. Lofi Dewanto
 * @version 1.0
 */
import java.io.StringWriter;


public class NestingException extends Exception {
    // Nested exception
    private Throwable nestedException;

    // String representation of stack trace - not transient!
    private String stackTraceString;

    /**
 * java.lang.Exception constructors.
 */
    public NestingException() {
    }

    /**
 * java.lang.Exception constructors.
 */
    public NestingException(String msg) {
        super(msg);
    }

    /**
 * Additional c'tors - nest the exceptions, storing the stack trace.
 */
    public NestingException(Throwable nestedException) {
        this.nestedException = nestedException;
        stackTraceString = generateStackTraceString(nestedException);
    }

    /**
 * Additional c'tors - nest the exceptions, storing the stack trace.
 */
    public NestingException(String msg, Throwable nestedException) {
        this(msg);
        this.nestedException = nestedException;
        stackTraceString = generateStackTraceString(nestedException);
    }

    /**
 * Convert a stack trace to a String so it can be serialized.
 */
    static public String generateStackTraceString(Throwable t) {
        StringWriter s = new StringWriter();
        t.printStackTrace(new PrintWriter(s));

        return s.toString();
    }

    /**
 * Methods.
 */
    public Throwable getNestedException() {
        return nestedException;
    }

    /**
 * Descend through linked-list of nesting exceptions, & output trace.
 * Note that this displays the 'deepest' trace first.
 */
    public String getStackTraceString() {
        // If there's no nested exception, there's no stackTrace
        if (nestedException == null) {
            return null;
        }

        StringBuffer traceBuffer = new StringBuffer();

        if (nestedException instanceof NestingException) {
            traceBuffer.append(
                    ((NestingException) nestedException).getStackTraceString());
            traceBuffer.append("-------- nested by:\n");
        }

        traceBuffer.append(stackTraceString);

        return traceBuffer.toString();
    }

    /**
 * Overrides Exception.getMessage().
 */
    public String getMessage() {
        // superMsg will contain whatever String was passed into the
        // constructor, and null otherwise.
        String superMsg = super.getMessage();

        // if there's no nested exception, do like we would always do
        if (getNestedException() == null) {
            return superMsg;
        }

        StringBuffer theMsg = new StringBuffer();

        // get the nested exception's message
        String nestedMsg = getNestedException().getMessage();

        if (superMsg != null) {
            theMsg.append(superMsg).append(": ").append(nestedMsg);
        } else {
            theMsg.append(nestedMsg);
        }

        return theMsg.toString();
    }

    /**
 * Overrides Exception.toString().
 */
    public String toString() {
        StringBuffer theMsg = new StringBuffer(super.toString());

        if (getNestedException() != null) {
            theMsg.append("; \n\t---> nested ").append(getNestedException());
        }

        return theMsg.toString();
    }
}