/**
 * Title:        OpenUSS - Open Source University Support System
 * Description:  Utility Class for uploading the FileObject
 * Copyright:    Copyright (c) B. Lofi Dewanto
 * Company:      University of Muenster
 * @author  B. Lofi Dewanto
 * @version 1.0
 */
package org.openuss.utility;

import com.lutris.appserver.server.httpPresentation.*;

import com.lutris.mime.*;

//import com.lutris.xml.xmlc.*;

import java.io.*;

import java.rmi.*;

import java.text.*;

import java.util.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;


/**
 * Helper for uploading a file.
 * This class gets the data for uploading the file with its
 * date, filename, title and the file object itself.
 *
 * @author  B. Lofi Dewanto
 * @version 1.0
 */
public class UploadFileHelper {
    // The contents of the update form
    protected static String FILE_DATE = "Date";
    protected static String FILE_NAME = "Filename";
    protected static String FILE_TITLE = "Name";
    protected static String FILE_FILE = "File";

    // Data
    protected String title;
    protected String filename;
    protected String date;
    protected FileObjectWrapper fileObject;

    /**
     * Constructor.
     */
    public UploadFileHelper() throws Exception {
        // Nothing to do  
    }

    /**
     * Constructor.
     */
    public UploadFileHelper(HttpPresentationComms comms)
                     throws Exception {
        this();


        // Calculate...
        calculateData(comms);
    }

    /**
     * Get the title.
     */
    public String getTitle() {
        return title;
    }

    /**
     * Set the title.
     */
    public void setTitle(String title) {
        this.title = title;
    }

    /**
     * Get the date.
     */
    public String getDate() {
        return date;
    }

    /**
     * Set the date.
     */
    public void setDate(String date) {
        this.date = date;
    }

    /**
     * Get the filename.
     */
    public String getFilename() {
        return filename;
    }

    /**
     * Set the filename.
     */
    public void setFilename(String filename) {
        this.filename = filename;
    }

    /**
     * Get the size.
     */
    public long getSize() {
        if (fileObject != null) {
            // Calculate the size of the FileObject
            byte[] data = fileObject.getData();
            long fileSize = data.length;

            // Put this in kilobyte, don't forget to
            // add one on it
            long result = (fileSize / 1024) + 1;

            return result;
        } else {
            // No file
            return 0;
        }
    }

    /**
     * Get the file.
     */
    public FileObjectWrapper getFileObject() {
        return fileObject;
    }

    /**
     * Set the file.
     */
    public void setFileObject(FileObjectWrapper fileObject) {
        this.fileObject = fileObject;
    }

    /**
     * Calculate the posted data.
     */
    protected void calculateData(HttpPresentationComms comms)
                          throws Exception {
        // Get the content type first
        String contentType = comms.request.getContentType();

        if (contentType == null) {
            contentType = "";
        }

        // Create the content header
        ContentHeader contentHdr = new ContentHeader("Content-Type: " + 
                                                     contentType);

        // Get the input stream
        HttpPresentationInputStream input = comms.request.getInputStream();

        // Create the multiple mime
        MultipartMimeInput mime = new MultipartMimeInput(input, contentHdr);
        MultipartMimeInputStream in;

        // Go through the multiple content...
        while ((in = mime.nextPart()) != null) {
            MimeHeader hdr = in.getHeader("Content-Disposition");

            // Check the header
            if (hdr != null) {
                String contentName = null;

                ContentHeader chdr = new ContentHeader(hdr);
                contentName = chdr.getParameter("name");

                // --- Get the file ---
                if (contentName.equals(FILE_FILE)) {
                    /* !!!For testing purpose: Directly writing the file to the
                       file system!!!
                    int n;
                    byte[] buffer = new byte[1024];
                    String outFileName = "c:/temp/" + contentName +
                      System.currentTimeMillis();
                                                                                           
                    FileOutputStream out = new FileOutputStream(outFileName);
                                                                                           
                    while ((n = in.read(buffer, 0, buffer.length)) > 0) {
                      out.write(buffer, 0, n);
                    }
                                                                                           
                    out.close();
                     */

                    // Read the stream
                    int n;
                    byte[] buffer = new byte[1024];

                    ByteArrayOutputStream outByte = new ByteArrayOutputStream();

                    while ((n = in.read(buffer, 0, buffer.length)) > 0) {
                        outByte.write(buffer, 0, n);
                    }


                    // Save into the FileObject
                    outByte.flush();

                    byte[] data = outByte.toByteArray();

                    // Save into the FileObject, if the size bigger than zero
                    if (outByte.size() > 0) {
                        fileObject = new FileObjectWrapper(data);
                    }

                    outByte.close();
                }

                // --- Get the title ---
                if (contentName.equals(FILE_TITLE)) {
                    int n;
                    byte[] buffer = new byte[1024];

                    StringBuffer sb = new StringBuffer();

                    while ((n = in.read(buffer, 0, buffer.length)) > 0) {
                        for (int i = 0; i < n; i++) {
                            sb.append((char) (((int) (buffer[i]) + 0x100) & 
                                          0xff));
                        }
                    }


                    // Save in the private variable
                    title = new String(sb);
                }

                // --- Get the date ---
                if (contentName.equals(FILE_DATE)) {
                    int n;
                    byte[] buffer = new byte[1024];

                    StringBuffer sb = new StringBuffer();

                    while ((n = in.read(buffer, 0, buffer.length)) > 0) {
                        for (int i = 0; i < n; i++) {
                            sb.append((char) (((int) (buffer[i]) + 0x100) & 
                                          0xff));
                        }
                    }


                    // Save in the private variable
                    date = new String(sb);
                }

                // --- Get the filename ---
                if (contentName.equals(FILE_NAME)) {
                    int n;
                    byte[] buffer = new byte[1024];

                    StringBuffer sb = new StringBuffer();

                    while ((n = in.read(buffer, 0, buffer.length)) > 0) {
                        for (int i = 0; i < n; i++) {
                            sb.append((char) (((int) (buffer[i]) + 0x100) & 
                                          0xff));
                        }
                    }


                    // Save in the private variable
                    filename = new String(sb);
                }
            }


            // Close the input but not the whole upload
            in.close();
        }
    }
}