package projectmanagement.business.employee;

import projectmanagement.business.ProjectManagementBusinessException;
import projectmanagement.data.employee.*;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.appserver.server.sql.ObjectId;
import com.lutris.appserver.server.sql.ObjectIdException;
import com.lutris.dods.builder.generator.query.*;
import com.lutris.appserver.server.Enhydra;
import com.lutris.logging.*;

import projectmanagement.spec.employee.*;

/**
 * Used to find the instance of Employee.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class EmployeeManagerImpl implements EmployeeManager {

   /**
    * The getAllEmployees method performs a database query to
    * return all <CODE>Employee</CODE> objects representing the
    * row in the <CODE>Employees</CODE> table.
    * @return all the employees, or null if there are no any.
    * @exception ProjectManagementBusinessException
    *    if there is a problem retrieving employee information.
    */
    public  Employee[] getAllEmployees()
         throws ProjectManagementBusinessException {
         
      try {
         EmployeeQuery query = new EmployeeQuery();

         EmployeeDO[] foundEmployees = query.getDOArray();
         if(foundEmployees.length != 0) {
            EmployeeImpl[] cs=new EmployeeImpl[foundEmployees.length];
            for (int i=0; i<foundEmployees.length; i++) {
               cs[i]=new EmployeeImpl(foundEmployees[i]);
            }
            return cs;
         } else {
            return null;
         }
      } catch(NonUniqueQueryException ex) {
         Enhydra.getLogChannel().write(Logger.DEBUG,
            "Non-unique employee found in database: "+ex.getMessage());
         throw new ProjectManagementBusinessException("Non unique employee found");
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Database error retrieving employees: ", ex);
       }/*catch(QueryException ex) {
         throw new ProjectManagementBusinessException("Query exception retrieving employees: ", ex);
      }*/
      
   }

   /**
    * The findEmployeeByID method performs a database query to
    * return a <CODE>Employee</CODE> object
    * representing the row in the <CODE>employee</CODE> table
    * that matches the object id.
    *
    * @param id, the object id of the employee table.
    * @return
    *    the employee. null if there isn't a employee associated
    *    the id
    * @exception ProjectManagementBusinessException
    *    if there is a problem retrieving employee information.
    */
   public  Employee findEmployeeByID(String id)
         throws ProjectManagementBusinessException {
    
      Employee theEmployee = null;

      try {
         EmployeeQuery query = new EmployeeQuery();
         //set query
         query.setQueryOId(new ObjectId(id));
         // Throw an exception if more than one user by this name is found
         query.requireUniqueInstance();
         EmployeeDO theEmployeeDO = query.getNextDO();
         theEmployee = new EmployeeImpl(theEmployeeDO);
         return theEmployee;
      } catch(Exception ex) {
         throw new ProjectManagementBusinessException("Exception in findEmployeeByID()", ex);
      }
   }

   /**
    * The findEmployee method performs a database query to
    * return a <CODE>Employee</CODE> object
    * representing the row in the <CODE>employees</CODE> table
    * that matches login name with the username.
    *
    * @param username, the login name of the employee.
    * @return
    *    the employee. null if there isn't a employee associated
    *    the username
    * @exception ProjectManagementBusinessException
    *    if there is a problem retrieving employee information.
    */
    public  Employee findEmployee(String username)
         throws ProjectManagementBusinessException {
      try {
       
         EmployeeQuery query = new EmployeeQuery();
       
         // set query.
         query.setQueryLogin(username);
         // Throw an exception if more than one user by this name is found
         query.requireUniqueInstance();

         EmployeeDO[] foundEmployee = query.getDOArray();
         if(foundEmployee.length != 0) {
            return new EmployeeImpl(foundEmployee[0]);
         } else {
            return null;
         }
       } catch(NonUniqueQueryException ex) {
         Enhydra.getLogChannel().write(Logger.DEBUG,
            "Non-unique user found in database: " +
            ex.getMessage());
         throw new ProjectManagementBusinessException("More than one user found with username: " +
            username);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Database error retrieving user: ", ex);
      } catch(QueryException ex) {
         throw new ProjectManagementBusinessException("Query exception retrieving user: ", ex);
      }
   }

public Employee	getEmployee()
         throws ProjectManagementBusinessException
         { 
          return new EmployeeImpl();
         }
}
