package projectmanagement.business.project;

import projectmanagement.business.customer.*;
import projectmanagement.business.ProjectManagementBusinessException;
import projectmanagement.data.project.*;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.appserver.server.sql.ObjectIdException;
import com.lutris.dods.builder.generator.query.DataObjectException;

import projectmanagement.spec.project.*;
import projectmanagement.spec.customer.*;
/**
 * Represents a project.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class ProjectImpl implements Project {
   /**
    * The DO of the Project.
    */
   protected ProjectDO myDO = null;

   /**
    * The public constructor.
    */
   public ProjectImpl() throws ProjectManagementBusinessException {
      try {
         this.myDO = ProjectDO.createVirgin();
      } catch(DatabaseManagerException ex) {
         throw new ProjectManagementBusinessException("Error creating empty project", ex);
      } catch(ObjectIdException ex) {
         throw new ProjectManagementBusinessException("Error creating object ID for project", ex);
      }
   }

   /** The public constructor
    *
    * @param theProject. The data object of the project.
    */
   public ProjectImpl(ProjectDO theProject) {
      this.myDO = theProject;
   }

   /**
    * Gets the DO object for the project
    *
    * @return the DO object.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data.
    */
   public ProjectDO getDO()
         throws ProjectManagementBusinessException {
      if (this.myDO!=null) {
         return this.myDO;
      } else {
         throw new ProjectManagementBusinessException("Error getting DO object for project");
      }
   }

   /**
    * Gets the object id for the project
    *
    * @return the object id.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getHandle()
      throws ProjectManagementBusinessException {
      try {
         return this.myDO.getHandle();
      } catch(DatabaseManagerException ex) {
         throw new ProjectManagementBusinessException("Error getting handle for project", ex);
      }
   }

   /**
    * Sets the name for the project.
    *
    * @param the name.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setName(String name)
      throws ProjectManagementBusinessException {
      try {
         myDO.setName(name);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting project name", ex);
      }
   }

   /**
    * Gets the name for the project
    *
    * @return the name.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getName ()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getName();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting project name", ex);
      }
   }

   /**
    * Sets the money per hour for the project.
    *
    * @param the money per hour.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setMoneyPerHour(double moneyPerHour)
      throws ProjectManagementBusinessException {
      try {
         myDO.setMoneyPerHour(moneyPerHour);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting money per hour", ex);
      }
   }

   /**
    * Gets the money per hour for the project
    *
    * @return the money per hour.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public double getMoneyPerHour ()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getMoneyPerHour();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting money per hour", ex);
      }
   }

   /**
    * Sets the customer for the project.
    *
    * @param the customer.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setCustomer(Customer customer)
      throws ProjectManagementBusinessException {
      try {
         myDO.setCustomer(((CustomerImpl)customer).getDO());
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting customer", ex);
      }
   }

   /**
    * Gets the customer.
    *
    * @return the customer.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public Customer getCustomer()
      throws ProjectManagementBusinessException {
      try {
         return new CustomerImpl(myDO.getCustomer());
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting customer", ex);
      }
   }

   /**
    * Commits all changes to the database.
    *
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public void save() throws ProjectManagementBusinessException {
      try {
         this.myDO.save();
      } catch(Exception ex) {
         throw new ProjectManagementBusinessException("Error saving project", ex);
      }
   }

   /**
    * Deletes the project from database.
    *
    * @exception ProjectManagementBusinessException if an error occurs
    *   deleting data (usually due to an underlying data layer
    *   error).
    */
   public void delete() throws ProjectManagementBusinessException {
      try {
         this.myDO.delete();
      } catch(Exception ex) {
         throw new ProjectManagementBusinessException("Error deleting project", ex);
      }
   }

}
