package projectmanagement.business.project;

import projectmanagement.business.ProjectManagementBusinessException;
import projectmanagement.data.project.*;

import projectmanagement.business.customer.*;

import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.appserver.server.sql.ObjectId;
import com.lutris.appserver.server.sql.ObjectIdException;
import com.lutris.dods.builder.generator.query.*;
import com.lutris.appserver.server.Enhydra;
import com.lutris.logging.*;

import projectmanagement.spec.customer.*;
import projectmanagement.spec.project.*;
/**
 * Used to find the instance of Project.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class ProjectManagerImpl implements ProjectManager {

   /**
    * The getAllProjects method performs a database query to
    * return all <CODE>Project</CODE> objects representing the
    * row in the <CODE>Projects</CODE> table.
    * @return all the projects, or null if there are no any.
    * @exception ProjectManagementBusinessException
    *    if there is a problem retrieving project information.
    */
    public  Project[] getAllProjects()
         throws ProjectManagementBusinessException {
      try {
         ProjectQuery query = new ProjectQuery();

         ProjectDO[] foundProjects = query.getDOArray();
         if(foundProjects.length != 0) {
            ProjectImpl[] cs=new ProjectImpl[foundProjects.length];
            for (int i=0; i<foundProjects.length; i++) {
               cs[i]=new ProjectImpl(foundProjects[i]);
            }
            return cs;
         } else {
            return null;
         }
      } catch(NonUniqueQueryException ex) {
         Enhydra.getLogChannel().write(Logger.DEBUG,
            "Non-unique project found in database: "+ex.getMessage());
         throw new ProjectManagementBusinessException("Non unique project found");
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Database error retrieving projects: ", ex);
      } /*catch(QueryException ex) {
         throw new ProjectManagementBusinessException("Query exception retrieving projects: ", ex);
      }*/
   }

   /**
    * The findProjectByID method performs a database query to
    * return a <CODE>Project</CODE> object
    * representing the row in the <CODE>project</CODE> table
    * that matches the object id.
    *
    * @param id, the object id of the project table.
    * @return
    *    the project. null if there isn't a project associated
    *    the id
    * @exception ProjectManagementBusinessException
    *    if there is a problem retrieving project information.
    */
   public Project findProjectByID(String id)
         throws ProjectManagementBusinessException {
      ProjectImpl theProject = null;

      try {
         ProjectQuery query = new ProjectQuery();
         //set query
         query.setQueryOId(new ObjectId(id));
         // Throw an exception if more than one user by this name is found
         query.requireUniqueInstance();
         ProjectDO theProjectDO = query.getNextDO();
         theProject = new ProjectImpl(theProjectDO);
         return theProject;
      } catch(Exception ex) {
         throw new ProjectManagementBusinessException("Exception in findProjectByID()", ex);
      }
   }

    /**
     * The getAllProjectsForCustomer method performs a database query to
     * return a <CODE>Project</CODE> objects representing the row in the 
     * <CODE>project</CODE> table that matches the customer given by 
     * customerID.
     *
     * @param customerID the object id of Customer for which we need projects.
     * @return
     *    array of projects for given customer. null if there isn't a project 
     *    associated the customerID
     * @exception ProjectManagementBusinessException
     *    if there is a problem retrieving project information.
     */
    public Project[] getAllProjectsForCustomer(String customerID) 
    throws ProjectManagementBusinessException {

        if (customerID == null)
            return getAllProjects();
        try {
            ProjectQuery query = new ProjectQuery();
            
            CustomerManagerImpl customerManager = new CustomerManagerImpl();
            CustomerImpl customer = (CustomerImpl)customerManager.findCustomerByID(customerID);
            
            query.setQueryCustomer(customer.getDO(),QueryBuilder.EQUAL);
            
            ProjectDO[] foundProjects = query.getDOArray();
            if(foundProjects.length != 0) {
                ProjectImpl[] projs=new ProjectImpl[foundProjects.length];
                for (int i=0; i<foundProjects.length; i++) {
                    projs[i]=new ProjectImpl(foundProjects[i]);
                }
                return projs;
            } else {
                return null;
            }
        } catch(NonUniqueQueryException ex) {
            Enhydra.getLogChannel().write(Logger.DEBUG,
            "Non-unique worksheet found in database: "+ex.getMessage());
            throw new ProjectManagementBusinessException("Non unique worksheet found");
        } catch(DataObjectException ex) {
            throw new ProjectManagementBusinessException("Database error retrieving projects: ", ex);
        } catch(QueryException ex) {
            throw new ProjectManagementBusinessException("Query exception retrieving projects: ", ex);
        }
    }

    /**
     * The getAllProjectsForCustomer method performs a database query to
     * return a <CODE>Project</CODE> objects representing the row in the 
     * <CODE>project</CODE> table that matches the customer 
     *
     * @param customer the customer for which we need projects.
     * @return
     *    array of projects for given customer. null if there isn't a project 
     *    associated the customer
     * @exception ProjectManagementBusinessException
     *    if there is a problem retrieving project information.
     */
    public Project[] getAllProjectsForCustomer(Customer customer) 
    throws ProjectManagementBusinessException {

        if (customer == null)
            return getAllProjects();
        try {
            ProjectQuery query = new ProjectQuery();
            query.setQueryCustomer(((CustomerImpl)customer).getDO(),QueryBuilder.EQUAL);
            
            ProjectDO[] foundProjects = query.getDOArray();
            if(foundProjects.length != 0) {
                ProjectImpl[] projs=new ProjectImpl[foundProjects.length];
                for (int i=0; i<foundProjects.length; i++) {
                    projs[i]=new ProjectImpl(foundProjects[i]);
                }
                return projs;
            } else {
                return null;
            }
        } catch(NonUniqueQueryException ex) {
            Enhydra.getLogChannel().write(Logger.DEBUG,
            "Non-unique worksheet found in database: "+ex.getMessage());
            throw new ProjectManagementBusinessException("Non unique worksheet found");
        } catch(DataObjectException ex) {
            throw new ProjectManagementBusinessException("Database error retrieving projects: ", ex);
        } catch(QueryException ex) {
            throw new ProjectManagementBusinessException("Query exception retrieving projects: ", ex);
        }
    }
public Project getProject()
      throws ProjectManagementBusinessException{
     return new ProjectImpl(); 
    }

}
