package projectmanagement.business.timewage;

import projectmanagement.business.project.*;
import projectmanagement.business.employee.*;
import projectmanagement.business.ProjectManagementBusinessException;
import projectmanagement.data.timewage.*;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.appserver.server.sql.ObjectIdException;
import com.lutris.dods.builder.generator.query.DataObjectException;

import projectmanagement.spec.timewage.*;
import projectmanagement.spec.employee.*;
import projectmanagement.spec.project.*;

import java.sql.Date;

/**
 * Represents a pay rate.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class PayRateImpl implements PayRate {
   /**
    * The DO of the PayRate.
    */
   protected PayRateDO myDO = null;

   /**
    * The public constructor.
    */
   public PayRateImpl() throws ProjectManagementBusinessException {
      try {
         this.myDO = PayRateDO.createVirgin();
      } catch(DatabaseManagerException ex) {
         throw new ProjectManagementBusinessException("Error creating empty pay rate", ex);
      } catch(ObjectIdException ex) {
         throw new ProjectManagementBusinessException("Error creating object ID for pay rate", ex);
      }
   }

   /** The public constructor
    *
    * @param thePayRate. The data object of the pay rate.
    */
   public PayRateImpl(PayRateDO thePayRate) {
      this.myDO = thePayRate;
   }

   /**
    * Gets the DO object for the pay rate
    *
    * @return the DO object.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data.
    */
   public PayRateDO getDO()
         throws ProjectManagementBusinessException {
      if (this.myDO!=null) {
         return this.myDO;
      } else {
         throw new ProjectManagementBusinessException("Error getting DO object for pay rate");
      }
   }

   /**
    * Gets the object id for the pay rate
    *
    * @return the object id.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getHandle()
      throws ProjectManagementBusinessException {
      try {
         return this.myDO.getHandle();
      } catch(DatabaseManagerException ex) {
         throw new ProjectManagementBusinessException("Error getting handle for pay rate", ex);
      }
   }

   /**
    * Sets the rate for the pay rate.
    *
    * @param the rate.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setRate(double rate)
      throws ProjectManagementBusinessException {
      try {
         myDO.setRate(rate);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting pay rate rate", ex);
      }
   }

   /**
    * Gets the rate for the pay rate
    *
    * @return the rate.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public double getRate ()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getRate();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting pay rate rate", ex);
      }
   }

   /**
    * Sets the date the payrate is valid from.
    *
    * @param the date the payrate is valid from.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setValidFrom(Date validFrom)
      throws ProjectManagementBusinessException {
      try {
         myDO.setValidFrom(validFrom);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting valid from", ex);
      }
   }

   /**
    * Gets the date the payrate is valid from.
    *
    * @return the date the payrate is valid from.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public Date getValidFrom ()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getValidFrom();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting valid from", ex);
      }
   }

   /**
    * Sets the employee for the pay rate.
    *
    * @param the employee.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setEmployee(Employee employee)
      throws ProjectManagementBusinessException {
      try {
         myDO.setEmployee(((EmployeeImpl)employee).getDO());
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting employee", ex);
      }
   }

   /**
    * Gets the employee.
    *
    * @return the employee.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public Employee getEmployee()
      throws ProjectManagementBusinessException {
      try {
         return new EmployeeImpl(myDO.getEmployee());
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting employee", ex);
      }
   }

   /**
    * Sets the project for the pay rate.
    *
    * @param the project.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setProject(Project project)
      throws ProjectManagementBusinessException {
      try {
         myDO.setProject(((ProjectImpl)project).getDO());
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting project", ex);
      }
   }

   /**
    * Gets the project.
    *
    * @return the project.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public Project getProject()
      throws ProjectManagementBusinessException {
      try {
         return new ProjectImpl(myDO.getProject());
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting project", ex);
      }
   }

   /**
    * Commits all changes to the database.
    *
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public void save() throws ProjectManagementBusinessException {
      try {
         this.myDO.save();
      } catch(Exception ex) {
         throw new ProjectManagementBusinessException("Error saving pay rate", ex);
      }
   }

   /**
    * Deletes the pay rate from database.
    *
    * @exception ProjectManagementBusinessException if an error occurs
    *   deleting data (usually due to an underlying data layer
    *   error).
    */
   public void delete() throws ProjectManagementBusinessException {
      try {
         this.myDO.delete();
      } catch(Exception ex) {
         throw new ProjectManagementBusinessException("Error deleting pay rate", ex);
      }
   }

}
