package projectmanagement.business.timewage;

import projectmanagement.business.ProjectManagementBusinessException;
import projectmanagement.data.timewage.*;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.appserver.server.sql.ObjectIdException;
import com.lutris.dods.builder.generator.query.DataObjectException;

import java.sql.Date;
import java.sql.Time;

import projectmanagement.spec.timewage.*;
/**
 * Represents a work sheet.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class WorkSheetImpl implements WorkSheet {
   /**
    * The DO of the WorkSheet.
    */
   protected WorkSheetDO myDO = null;

   /**
    * The public constructor.
    */
   public WorkSheetImpl() throws ProjectManagementBusinessException {
      try {
         this.myDO = WorkSheetDO.createVirgin();
      } catch(DatabaseManagerException ex) {
         throw new ProjectManagementBusinessException("Error creating empty work sheet", ex);
      } catch(ObjectIdException ex) {
         throw new ProjectManagementBusinessException("Error creating object ID for work sheet", ex);
      }
   }

   /** The public constructor
    *
    * @param theWorkSheet. The data object of the work sheet.
    */
   public WorkSheetImpl(WorkSheetDO theWorkSheet) {
      this.myDO = theWorkSheet;
   }

   /**
    * Gets the DO object for the work sheet
    *
    * @return the DO object.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data.
    */
   public WorkSheetDO getDO()
         throws ProjectManagementBusinessException {
      if (this.myDO!=null) {
         return this.myDO;
      } else {
         throw new ProjectManagementBusinessException("Error getting DO object for work sheet");
      }
   }

   /**
    * Gets the object id for the work sheet
    *
    * @return the object id.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getHandle()
      throws ProjectManagementBusinessException {
      try {
         return this.myDO.getHandle();
      } catch(DatabaseManagerException ex) {
         throw new ProjectManagementBusinessException("Error getting handle for work sheet", ex);
      }
   }

   /**
    * Sets the PayRate for the work sheet.
    *
    * @param the pay rate.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setPayRate(PayRate thePayRate)
      throws ProjectManagementBusinessException {
      try {
         myDO.setPayRate(((PayRateImpl)thePayRate).getDO());
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting work sheet rate", ex);
      }
   }

   /**
    * Gets the pay rate for the work sheet
    *
    * @return the pay rate.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public PayRate getPayRate ()
      throws ProjectManagementBusinessException {
      try {
         return new PayRateImpl(myDO.getPayRate());
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting work sheet pay rate", ex);
      }
   }

   /**
    * Sets the working date for work sheet.
    *
    * @param the working date for the worksheet.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setWorkingDate(Date workingDate)
      throws ProjectManagementBusinessException {
      try {
         myDO.setWorkingDate(workingDate);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting working date", ex);
      }
   }

   /**
    * Gets the working date for work sheet.
    *
    * @return the working date for work sheet.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public Date getWorkingDate ()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getWorkingDate();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting the working date for work sheet", ex);
      }
   }

   /**
    * Sets the time started for the work sheet.
    *
    * @param the time started.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setTimeStarted(Time timeStarted)
      throws ProjectManagementBusinessException {
      try {
         myDO.setTimeStarted(timeStarted);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting time started", ex);
      }
   }

   /**
    * Gets the time started.
    *
    * @return the time started.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public Time getTimeStarted()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getTimeStarted();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting time started", ex);
      }
   }

   /**
    * Sets the time finished for the work sheet.
    *
    * @param the time finished.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setTimeFinished(Time timeFinished)
      throws ProjectManagementBusinessException {
      try {
         myDO.setTimeFinished(timeFinished);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting time finished", ex);
      }
   }

   /**
    * Gets the time finished.
    *
    * @return the time finished.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public Time getTimeFinished()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getTimeFinished();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting time finished", ex);
      }
   }

   /**
    * Sets the personal comment for the work sheet.
    *
    * @param the personal comment.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setPersonalComment(String personalComment)
      throws ProjectManagementBusinessException {
      try {
         myDO.setPersonalComment(personalComment);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting work sheet personal comment", ex);
      }
   }

   /**
    * Gets the personal comment for the work sheet
    *
    * @return the personal comment.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getPersonalComment ()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getPersonalComment();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting work sheet personal comment", ex);
      }
   }

   /**
    * Sets the comment for client for the work sheet.
    *
    * @param the comment for client.
    * @exception ProjectManagementBusinessException if an error occurs
    *   setting the data (usually due to an underlying data layer
    *   error).
    */
   public void setCommentForClient(String commentForClient)
      throws ProjectManagementBusinessException {
      try {
         myDO.setCommentForClient(commentForClient);
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error setting work sheet comment for client", ex);
      }
   }

   /**
    * Gets the comment for client for the work sheet
    *
    * @return the comment for client.
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public String getCommentForClient ()
      throws ProjectManagementBusinessException {
      try {
         return myDO.getCommentForClient();
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Error getting work sheet comment for client", ex);
      }
   }

   /**
    * Commits all changes to the database.
    *
    * @exception ProjectManagementBusinessException if an error occurs
    *   retrieving data (usually due to an underlying data layer
    *   error).
    */
   public void save() throws ProjectManagementBusinessException {
      try {
         this.myDO.save();
      } catch(Exception ex) {
         throw new ProjectManagementBusinessException("Error saving work sheet", ex);
      }
   }

   /**
    * Deletes the work sheet from database.
    *
    * @exception ProjectManagementBusinessException if an error occurs
    *   deleting data (usually due to an underlying data layer
    *   error).
    */
   public void delete() throws ProjectManagementBusinessException {
      try {
         this.myDO.delete();
      } catch(Exception ex) {
         throw new ProjectManagementBusinessException("Error deleting work sheet", ex);
      }
   }

}
