package projectmanagement.business.timewage;

import projectmanagement.business.ProjectManagementBusinessException;
import projectmanagement.business.employee.*;
import projectmanagement.business.project.*;
import projectmanagement.data.timewage.*;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.appserver.server.sql.ObjectId;
import com.lutris.appserver.server.sql.ObjectIdException;
import com.lutris.dods.builder.generator.query.*;
import com.lutris.appserver.server.Enhydra;
import com.lutris.logging.*;

import projectmanagement.spec.employee.*;
import projectmanagement.spec.project.*;
import projectmanagement.spec.timewage.*;
/**
 * Used to find the instance of WorkSheet.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public class WorkSheetManagerImpl implements WorkSheetManager{

   /**
    * The getAllWorkSheets method performs a database query to
    * return all <CODE>WorkSheet</CODE> objects representing the
    * row in the <CODE>WorkSheets</CODE> table.
    * @return all the worksheets, or null if there are no any.
    * @exception ProjectManagementBusinessException
    *    if there is a problem retrieving workshit information.
    */
    public  WorkSheet[] getAllWorkSheets()
         throws ProjectManagementBusinessException {
      try {
         WorkSheetQuery query = new WorkSheetQuery();

         WorkSheetDO[] foundWorkSheets = query.getDOArray();
         if(foundWorkSheets.length != 0) {
            WorkSheetImpl[] cs=new WorkSheetImpl[foundWorkSheets.length];
            for (int i=0; i<foundWorkSheets.length; i++) {
               cs[i]=new WorkSheetImpl(foundWorkSheets[i]);
            }
            return cs;
         } else {
            return null;
         }
      } catch(NonUniqueQueryException ex) {
         Enhydra.getLogChannel().write(Logger.DEBUG,
            "Non-unique workshit found in database: "+ex.getMessage());
         throw new ProjectManagementBusinessException("Non unique worksheet found");
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Database error retrieving worksheets: ", ex);
      } /*catch(QueryException ex) {
         throw new ProjectManagementBusinessException("Query exception retrieving worksheets: ", ex);
      }*/
   }

   /**
    * The getAllWorkSheetsForEmployee method performs a database query to
    * return all <CODE>WorkSheet</CODE> objects representing the
    * row in the <CODE>WorkSheets</CODE> table that belongs to the
    * given employee.
    * @param employee          the employee
    * @return all the worksheets for given employee
    * @exception ProjectManagementBusinessException
    *    if there is a problem retrieving worksheet information.
    */
    public  WorkSheet[] getAllWorkSheetsForEmployee(Employee employee)
         throws ProjectManagementBusinessException {
      try {
         WorkSheetQuery query = new WorkSheetQuery();
         QueryBuilder mainQuery=query.getQueryBuilder();
         QueryBuilder subQuery=new QueryBuilder();
         subQuery.select(PayRateDO.PrimaryKey);
         subQuery.addWhere(PayRateDO.Employee,((EmployeeImpl)employee).getDO(),QueryBuilder.EQUAL);
         mainQuery.addWhereIn(WorkSheetDO.PayRate,subQuery);

         WorkSheetDO[] foundWorkSheets = query.getDOArray();
         if(foundWorkSheets.length != 0) {
            WorkSheetImpl[] cs=new WorkSheetImpl[foundWorkSheets.length];
            for (int i=0; i<foundWorkSheets.length; i++) {
               cs[i]=new WorkSheetImpl(foundWorkSheets[i]);
            }
            return cs;
         } else {
            return null;
         }
      } catch(NonUniqueQueryException ex) {
         Enhydra.getLogChannel().write(Logger.DEBUG,
            "Non-unique worksheet found in database: "+ex.getMessage());
         throw new ProjectManagementBusinessException("Non unique worksheet found");
      } catch(DataObjectException ex) {
         throw new ProjectManagementBusinessException("Database error retrieving worksheets: ", ex);
      } catch(QueryException ex) {
         throw new ProjectManagementBusinessException("Query exception retrieving worksheets: ", ex);
      }
   }

   /**
    * The findWorkSheetByID method performs a database query to
    * return a <CODE>WorkSheet</CODE> object
    * representing the row in the <CODE>worksheet</CODE> table
    * that matches the object id.
    *
    * @param id, the object id of the worksheet table.
    * @return
    *    the worksheet. null if there isn't a worksheet associated
    *    the id
    * @exception ProjectManagementBusinessException
    *    if there is a problem retrieving worksheet information.
    */
   public  WorkSheet findWorkSheetByID(String id)
         throws ProjectManagementBusinessException {
      WorkSheetImpl theWorkSheet = null;

      try {
         WorkSheetQuery query = new WorkSheetQuery();
         //set query
         query.setQueryOId(new ObjectId(id));
         // Throw an exception if more than one user by this name is found
         query.requireUniqueInstance();
         WorkSheetDO theWorkSheetDO = query.getNextDO();
         theWorkSheet = new WorkSheetImpl(theWorkSheetDO);
         return theWorkSheet;
      } catch(Exception ex) {
         throw new ProjectManagementBusinessException("Exception in findWorkSheetByID()", ex);
      }
   }

    /**
     * The getAllWorkSheetsForEmployeeProjectPair method performs 
     * a database query to return a <CODE>WorkSheet</CODE> objects
     * representing the row in the <CODE>worksheet</CODE> table
     * that matches the employee given by employeeID and project 
     * given by projectID.
     *
     * @param employeeID, the object id of Employee for which we need worksheets.
     * @param projectID, the object id of Project for which we need worksheets.
     * @return
     *    array of worksheets. null if there isn't a worksheet associated
     *    the employeeID and projectID
     * @exception ProjectManagementBusinessException
     *    if there is a problem retrieving worksheet information.
     */
    public  WorkSheet[] getAllWorkSheetsForEmployeeProjectPair(String employeeID,String projectID)
    throws ProjectManagementBusinessException {
        try {
            WorkSheetQuery query = new WorkSheetQuery();
            if (employeeID != null || projectID != null) {
                QueryBuilder mainQuery=query.getQueryBuilder();
                QueryBuilder subQuery=new QueryBuilder();
                subQuery.select(PayRateDO.PrimaryKey);
                if (employeeID != null) {
               EmployeeManagerImpl employeeManager=new EmployeeManagerImpl();
               EmployeeImpl employee=(EmployeeImpl)employeeManager.findEmployeeByID(employeeID);       
                  
                    subQuery.addWhere(PayRateDO.Employee,employee.getDO(),QueryBuilder.EQUAL);
                }
                if (projectID != null) {
                ProjectManagerImpl projectManager = new ProjectManagerImpl();
                Project project = projectManager.findProjectByID(projectID);  
                  
                    subQuery.addWhere(PayRateDO.Project,((ProjectImpl)project).getDO(),QueryBuilder.EQUAL);
                }
                mainQuery.addWhereIn(WorkSheetDO.PayRate,subQuery);
            }
            WorkSheetDO[] foundWorkSheets = query.getDOArray();
            if(foundWorkSheets.length != 0) {
                WorkSheetImpl[] ws=new WorkSheetImpl[foundWorkSheets.length];
                for (int i=0; i<foundWorkSheets.length; i++) {
                    ws[i]=new WorkSheetImpl(foundWorkSheets[i]);
                }
                return ws;
            } else {
                return null;
            }
                
        } catch(NonUniqueQueryException ex) {
            Enhydra.getLogChannel().write(Logger.DEBUG,
            "Non-unique worksheet found in database: "+ex.getMessage());
            throw new ProjectManagementBusinessException("Non unique worksheet found");
        } catch(DataObjectException ex) {
            throw new ProjectManagementBusinessException("Database error retrieving worksheets: ", ex);
        } catch(QueryException ex) {
            throw new ProjectManagementBusinessException("Query exception retrieving worksheets: ", ex);
        }
    }

    /**
     * The getAllWorkSheetsForEmployeeProjectPair method performs 
     * a database query to return a <CODE>WorkSheet</CODE> objects
     * representing the row in the <CODE>worksheet</CODE> table
     * that matches the employee and project.
     *
     * @param employee employee for which we need worksheets.
     * @param project project for which we need worksheets.
     * @return
     *    array of worksheets. null if there isn't a worksheet associated
     *    the employee and project
     * @exception ProjectManagementBusinessException
     *    if there is a problem retrieving worksheet information.
     */
    public  WorkSheet[] getAllWorkSheetsForEmployeeProjectPair(Employee employee, Project project)
    throws ProjectManagementBusinessException {
        try {
            WorkSheetQuery query = new WorkSheetQuery();
            if (employee != null || project != null) {
                QueryBuilder mainQuery=query.getQueryBuilder();
                QueryBuilder subQuery=new QueryBuilder();
                subQuery.select(PayRateDO.PrimaryKey);
                if (employee != null) 
                    subQuery.addWhere(PayRateDO.Employee,((EmployeeImpl)employee).getDO(),QueryBuilder.EQUAL);
                if (project != null) 
                    subQuery.addWhere(PayRateDO.Project,((ProjectImpl)project).getDO(),QueryBuilder.EQUAL);
                mainQuery.addWhereIn(WorkSheetDO.PayRate,subQuery);
            }
            WorkSheetDO[] foundWorkSheets = query.getDOArray();
            if(foundWorkSheets.length != 0) {
                WorkSheetImpl[] ws=new WorkSheetImpl[foundWorkSheets.length];
                for (int i=0; i<foundWorkSheets.length; i++) {
                    ws[i]=new WorkSheetImpl(foundWorkSheets[i]);
                }
                return ws;
            } else {
                return null;
            }
                
        } catch(NonUniqueQueryException ex) {
            Enhydra.getLogChannel().write(Logger.DEBUG,
            "Non-unique worksheet found in database: "+ex.getMessage());
            throw new ProjectManagementBusinessException("Non unique worksheet found");
        } catch(DataObjectException ex) {
            throw new ProjectManagementBusinessException("Database error retrieving worksheets: ", ex);
        } catch(QueryException ex) {
            throw new ProjectManagementBusinessException("Query exception retrieving worksheets: ", ex);
        }
    }

    /**
     * The getAllWorkSheetsForEmployeeProjectPair method performs 
     * a database query to return a <CODE>WorkSheet</CODE> objects
     * representing the row in the <CODE>worksheet</CODE> table
     * that matches the employee given by employeeID, project given
     * by projectID and between date1 and date2 (including date1).
     *
     * @param employeeID, the object id of Employee for which we need worksheets.
     * @param projectID, the object id of Project for which we need worksheets.
     * @param date1 begining date (including date1)
     * @param date2 ending date
     * @return
     *    array of worksheets. null if there isn't a worksheet associated
     *    the employeeID, projectID and between date1 and date2
     * @exception ProjectManagementBusinessException
     *    if there is a problem retrieving worksheet information.
     */
    public WorkSheet[] getAllWorkSheetsForEmployeeProjectPairFromDate1ToDate2(
                    String employeeID, String projectID, java.sql.Date date1, java.sql.Date date2)
    throws ProjectManagementBusinessException {
        try {
            WorkSheetQuery query = new WorkSheetQuery();
            QueryBuilder mainQuery=query.getQueryBuilder();
            if (employeeID != null || projectID != null) {
                QueryBuilder subQuery=new QueryBuilder();
                subQuery.select(PayRateDO.PrimaryKey);
                if (employeeID != null) {
           EmployeeManagerImpl employeeManager=new EmployeeManagerImpl();
           Employee employee= employeeManager.findEmployeeByID(employeeID);   
                    subQuery.addWhere(PayRateDO.Employee,((EmployeeImpl)employee).getDO(),QueryBuilder.EQUAL);
                }
                if (projectID != null) {
             ProjectManagerImpl projectManager = new ProjectManagerImpl();
             Project project = projectManager.findProjectByID(projectID);
                    subQuery.addWhere(PayRateDO.Project,((ProjectImpl)project).getDO(),QueryBuilder.EQUAL);
                }
                mainQuery.addWhereIn(WorkSheetDO.PayRate,subQuery);
            }
            mainQuery.addWhere(WorkSheetDO.WorkingDate,date1,QueryBuilder.GREATER_THAN_OR_EQUAL);
            mainQuery.addWhere(WorkSheetDO.WorkingDate,date2,QueryBuilder.LESS_THAN_OR_EQUAL);
            WorkSheetDO[] foundWorkSheets = query.getDOArray();
            if(foundWorkSheets.length != 0) {
                WorkSheetImpl[] ws=new WorkSheetImpl[foundWorkSheets.length];
                for (int i=0; i<foundWorkSheets.length; i++) {
                    ws[i]=new WorkSheetImpl(foundWorkSheets[i]);
                }
                return ws;
            } else {
                return null;
            }
                
        } catch(NonUniqueQueryException ex) {
            Enhydra.getLogChannel().write(Logger.DEBUG,
            "Non-unique worksheet found in database: "+ex.getMessage());
            throw new ProjectManagementBusinessException("Non unique worksheet found");
        } catch(DataObjectException ex) {
            throw new ProjectManagementBusinessException("Database error retrieving worksheets: ", ex);
        } catch(QueryException ex) {
            throw new ProjectManagementBusinessException("Query exception retrieving worksheets: ", ex);
        }
    }

    /**
     * The getAllWorkSheetsForEmployeeProjectPair method performs 
     * a database query to return a <CODE>WorkSheet</CODE> objects
     * representing the row in the <CODE>worksheet</CODE> table
     * that matches the employee, project and between date1 and date2
     * (including date1).
     *
     * @param employee Employee for which we need worksheets.
     * @param project Project for which we need worksheets.
     * @param date1 begining date (including date1)
     * @param date2 ending date
     * @return
     *    array of worksheets. null if there isn't a worksheet associated
     *    the employeeID, projectID and between date1 and date2
     * @exception ProjectManagementBusinessException
     *    if there is a problem retrieving worksheet information.
     */
    public WorkSheet[] getAllWorkSheetsForEmployeeProjectPairFromDate1ToDate2(
                    Employee employee, Project project, java.sql.Date date1, java.sql.Date date2)
    throws ProjectManagementBusinessException {
        try {
            WorkSheetQuery query = new WorkSheetQuery();
            QueryBuilder mainQuery=query.getQueryBuilder();
            if (employee != null || project != null) {
                QueryBuilder subQuery=new QueryBuilder();
                subQuery.select(PayRateDO.PrimaryKey);
                if (employee != null) 
                    subQuery.addWhere(PayRateDO.Employee,((EmployeeImpl)employee).getDO(),QueryBuilder.EQUAL);
                if (project != null) 
                    subQuery.addWhere(PayRateDO.Project,((ProjectImpl)project).getDO(),QueryBuilder.EQUAL);
                mainQuery.addWhereIn(WorkSheetDO.PayRate,subQuery);
            }
            mainQuery.addWhere(WorkSheetDO.WorkingDate,date1,QueryBuilder.GREATER_THAN_OR_EQUAL);
            mainQuery.addWhere(WorkSheetDO.WorkingDate,date2,QueryBuilder.LESS_THAN_OR_EQUAL);
            WorkSheetDO[] foundWorkSheets = query.getDOArray();
            if(foundWorkSheets.length != 0) {
                WorkSheetImpl[] ws=new WorkSheetImpl[foundWorkSheets.length];
                for (int i=0; i<foundWorkSheets.length; i++) {
                    ws[i]=new WorkSheetImpl(foundWorkSheets[i]);
                }
                return ws;
            } else {
                return null;
            }
        
        } catch(NonUniqueQueryException ex) {
            Enhydra.getLogChannel().write(Logger.DEBUG,
            "Non-unique worksheet found in database: "+ex.getMessage());
            throw new ProjectManagementBusinessException("Non unique worksheet found");
        } catch(DataObjectException ex) {
            throw new ProjectManagementBusinessException("Database error retrieving worksheets: ", ex);
        } catch(QueryException ex) {
            throw new ProjectManagementBusinessException("Query exception retrieving worksheets: ", ex);
        }
    }
    /**
     * The getAllWorkSheetsForEmployeeProjectPair method performs 
     * a database query to return a <CODE>WorkSheet</CODE> objects
     * representing the row in the <CODE>worksheet</CODE> table
     * that matches the employee, project and between date1 and date2
     * (including date1).
     *
     * @param employeeIDs java.util.ArrayList for which we need worksheets.
     * @param projectIDs java.util.ArrayList for which we need worksheets.
     * @param date1 begining date (including date1)
     * @param date2 ending date
     * @return
     *    array of worksheets. null if there isn't a worksheet associated
     *    the employeeIDs, projectIDs and between date1 and date2
     * @exception ProjectManagementBusinessException
     *    if there is a problem retrieving worksheet information.
     */
    public  WorkSheet[] getAllWorksheetsForEmployeeArrayAndProjectArrayBetweenDates(
            java.util.ArrayList employeeIDs, java.util.ArrayList projectIDs, java.sql.Date date1, java.sql.Date date2) 
    throws ProjectManagementBusinessException {            
            
            String employeeID="";
            String projectID="";   
            java.util.Iterator it1=employeeIDs.iterator();
            java.util.Iterator it2=projectIDs.iterator();    
            
           try {
            WorkSheetQuery query = new WorkSheetQuery();
            QueryBuilder mainQuery=query.getQueryBuilder();
            
            QueryBuilder subQuery=new QueryBuilder();
            subQuery.select(PayRateDO.PrimaryKey);
            subQuery.addWhereOpenParen();                    
            boolean firstPass = true;
            while( it1.hasNext() ) {
                employeeID=it1.next().toString();
                if (employeeID !=null && employeeID.length()==0) {
                    employeeID=null;
                }
                if (employeeID != null) {
           EmployeeManagerImpl employeeManager=new EmployeeManagerImpl();
           Employee employee= employeeManager.findEmployeeByID(employeeID);        
                    if ( !firstPass ) { 
                        subQuery.addWhereOr();
                    }
                    firstPass = false;
                    subQuery.addWhere(PayRateDO.Employee,((EmployeeImpl)employee).getDO(),QueryBuilder.EQUAL);
                }
            }
            subQuery.addWhereCloseParen();
            subQuery.addWhereOpenParen();
            firstPass = true;
            while( it2.hasNext() ) {
                projectID=it2.next().toString();
                if (projectID != null && projectID.length()==0) {
                    projectID = null;
                }
                if (projectID != null) {
            ProjectManagerImpl projectManager = new ProjectManagerImpl();
            Project project = projectManager.findProjectByID(projectID);
                    if ( !firstPass ) { 
                        subQuery.addWhereOr();
                    }
                    firstPass = false;
                    subQuery.addWhere(PayRateDO.Project,((ProjectImpl)project).getDO(),QueryBuilder.EQUAL);
                }
            }
            subQuery.addWhereCloseParen();
            mainQuery.addWhereIn(WorkSheetDO.PayRate,subQuery);
            mainQuery.addWhere(WorkSheetDO.WorkingDate,date1,QueryBuilder.GREATER_THAN_OR_EQUAL);
            mainQuery.addWhere(WorkSheetDO.WorkingDate,date2,QueryBuilder.LESS_THAN_OR_EQUAL);
            WorkSheetDO[] foundWorkSheets = query.getDOArray();
             
            if(foundWorkSheets.length != 0) {
                WorkSheetImpl[] ws=new WorkSheetImpl[foundWorkSheets.length];
                try { 
                    for (int i=0; i<foundWorkSheets.length; i++) {
                        ws[i]=new WorkSheetImpl(foundWorkSheets[i]);
                    }
                } catch ( Exception e ) { e.printStackTrace(); }
                return ws;
            } else {
                return null;
            }
                
        } catch(NonUniqueQueryException ex) {
            Enhydra.getLogChannel().write(Logger.DEBUG,
            "Non-unique worksheet found in database: "+ex.getMessage());
            throw new ProjectManagementBusinessException("Non unique worksheet found");
        } catch(DataObjectException ex) {
            throw new ProjectManagementBusinessException("Database error retrieving worksheets: ", ex);
        } catch(QueryException ex) {
            throw new ProjectManagementBusinessException("Query exception retrieving worksheets: ", ex);
        } 
    }         
  public WorkSheet getWorkSheet()
         throws ProjectManagementBusinessException{
         return new WorkSheetImpl();
        }          
    
}
