package projectmanagement.presentation;

import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.appserver.server.Enhydra;
//import com.lutris.xml.xmlc.*;
//import com.lutris.xml.xmlc.html.*;
import com.lutris.logging.*;
import com.lutris.util.KeywordValueException;
import com.lutris.appserver.server.user.User;

import org.enhydra.xml.xmlc.XMLObject;
import org.w3c.dom.*;
import org.w3c.dom.html.HTMLElement;

import java.lang.reflect.*;
import java.util.*;
import java.lang.Throwable;

import projectmanagement.*;
import projectmanagement.spec.employee.Employee;

/**
 * This is the parent Presentation object.  All presentation objects
 * should extend this class.
 *
 * The run method looks for an event parameter and then calls
 * handle<EventName>.  If the "event" Parameter is not defined then
 * the handleDefault() method is called in your child class.
 *
 * @author Sasa Bojanic
 * @version 1.0
 */
public abstract class BasePO implements HttpPresentation {
   private static String AUTHORIZATION_ERROR_PAGE = "/AuthorizationError.po";
   private static String LOGIN_PAGE = "/Login.po";
   private static String EVENT = "event";
   private static String STANDARD_METHOD_PREFIX = "handle";

   /**
    * This is the procedure that is called if there is no "event"
    * HTTP parameter found. It must be overriden by the subclass to
    * do default processing or error checking/handling.
    *
    * @return String The String representation of the HTML or (other format)
    * of the document to be displayed. This method would need to be changed
    * if you wanted to return binary data as well. It returns a String
    * for simplicity right now.
    */
   public abstract XMLObject handleDefault() throws HttpPresentationException;

   /**
    * This method should be implemented in the subclass so that it returns
    * true if this particular request requires the user to be logged
    * in, otherwise false.
    */
   protected abstract int getRequiredAuthLevel();

   /**
    * Saved input and output context, and session data
    */
   protected HttpPresentationComms myComms = null;
   protected ProjectManagementSessionData mySessionData = null;

   /**
    * Gets HttpPresentation object
    *
    * @return The saved comms objects
    * to whichever subclass needs it
    */
   public HttpPresentationComms getComms() {
      return this.myComms;
   }

   /**
    * Gets the session data
    *
    * @return session data
    */
   public ProjectManagementSessionData getSessionData() {
      return this.mySessionData;
   }

   /**
    * Sets the user into the session
    *
    * @param theEmployee, the employee to be set in the session
    * @exception ProjectManagementPresentationException
    */
   public void setUser(Employee theEmployee)
         throws ProjectManagementPresentationException {
      this.getSessionData().setUser(theEmployee);
   }

   /**
    * Gets the user from the session
    *
    * @return the employee object in the session
    */
   public Employee getUser() {
      return this.getSessionData().getUser();
   }

   /**
    * Method to remove the current user from the session
    */
   public void removeUserFromSession() {
      this.getSessionData().removeUser();
   }

   /**
    * This implements the run method in HttpPresentation.
    *
    * @param HttpPresentationComms
    * @exception Exception
    */
   public void run(HttpPresentationComms comms)
      throws Exception {
      // Initialize new or get the existing session data
      initSessionData(comms);

      // Check if the user can access the given page
      checkAuthLevel();

      try {
         // Handle the incoming event request
         handleEvent(comms);
      } catch (Exception ex) {
         throw new Exception("Exception in run "+ex);
      }
   }

   /**
    * Method to get or create the AgSessionData object from the user session
    * This object is saved in the EbrokerPresentation object
    *
    * @param HttpPresentationComms
    * @exception Exception
    */
   protected void initSessionData(HttpPresentationComms comms)
         throws ProjectManagementPresentationException {
      this.myComms = comms;
      try {
         Object obj = comms.sessionData.get(ProjectManagementSessionData.SESSION_KEY);
         // If we found the session data, save it in a private data member
         if(null != obj) {
            this.mySessionData = (ProjectManagementSessionData)obj;
         } else {
            // If no session data was found, create a new session data instance
            this.mySessionData = new ProjectManagementSessionData();
            comms.sessionData.set(ProjectManagementSessionData.SESSION_KEY, this.mySessionData);
         }
      } catch(KeywordValueException ex) {
         writeDebugMsg("Problem getting session data from session: " +
            ex.getMessage());
         throw new ProjectManagementPresentationException("Trouble initializing user",ex);
      }
   }

   /**
    * Return the current authorization level (set during login)
    * @return An int equal to the current authorization level.
    */
   protected int getCurrentAuthLevel ()
         throws ClientPageRedirectException, ProjectManagementPresentationException {
      int accessLevel=0;
//We need  to allow ProjectManagement_pres to be functional , so if the requested url is /ProjectManagement_pres/ we allow user to acsses to page
try {
	String uri = myComms.request.getRequestURI();
      	boolean is= uri.startsWith("/ProjectManagement_pres");
if(is){
   	accessLevel=2 ;
}else{
          accessLevel=getSessionData().getAuthLevel();
        }     
      } catch (Exception ex) {
         throw new ProjectManagementPresentationException("Trouble getting current authorization level",ex);
      }
    
      return accessLevel;
   }

   /**
    * Checks the session data to see if user has the authorization to
    * access the given page. Authorization levels include:
    * UNAUTH_USER   (0) - login not required
    * ORDINARY_USER (1) - requires normal login
    * ADMIN_USER    (2) - requires to login as administrator
    * Redirects to the login page if user is not authorized to access the page.
    */
   protected void checkAuthLevel()
         throws ClientPageRedirectException, ProjectManagementPresentationException {

      int currentAuth=getCurrentAuthLevel();

      try {
         if (currentAuth<getRequiredAuthLevel()) {
            throw new ClientPageRedirectException(AUTHORIZATION_ERROR_PAGE);
         }
      } catch (Exception ex) {
         throw new ProjectManagementPresentationException("Trouble checking for user login status",ex);
      }
   }

   /**
    * Method to call the proper method for the incoming event
    *
    * @param HttpPresentationComms
    * @exception Exception
    */
   public void handleEvent(HttpPresentationComms comms)
         throws Exception {
      String event = comms.request.getParameter(EVENT);

      XMLObject returnHTML = null;

      if (event == null || event.length() == 0) {
         returnHTML = handleDefault();
      }
      else {
         returnHTML = getPageContentForEvent(event);
      }

      comms.response.writeDOM(returnHTML);
   }

   /**
    * Logs user out from the session by setting the usr to null
    * in the session data.
    *
    * @return html document
    * @exception ProjectManagementPresentationException
    */
   public XMLObject handleLogout()
         throws ProjectManagementPresentationException {
      try {
         this.mySessionData=null;
         SessionManager sessionManager=myComms.session.getSessionManager();
         sessionManager.deleteSession(myComms.session);
         throw new ClientPageRedirectException(LOGIN_PAGE);
      } catch (Exception ex) {
         throw new ProjectManagementPresentationException("Trouble logging out user",ex);
      }
    }

   /**
    * If an event parameter is defined then this invokes the method that
    * handles that event.
    *
    * @param event, the incoming event name
    * @exception Exception
    */
   public XMLObject getPageContentForEvent(String event)
         throws Exception {
      try {
         Method method = this.getClass().getMethod(toMethodName(event), null);
         XMLObject thePage = (XMLObject)method.invoke(this, null);
         return thePage;

      } catch(InvocationTargetException ex) {
         // Rethrow the originating exception if as it should be propagated as is
         // It could be a page redirect exception, etc.
         if (ex.getTargetException() instanceof Exception) {
            throw (Exception)ex.getTargetException();
         } else if (ex.getTargetException() instanceof Error) {
            throw (Error)ex.getTargetException();
         } else {
            throw ex;
         }
      } catch(NoSuchMethodException ex) {
         //The method to handle the event does not exist.
         throw new ProjectManagementPresentationException("NO EVENT HANDLER FOUND FOR EVENT: " +
            event, ex);
      } catch(IllegalAccessException ex) {
         //The method to handle the event does not exist.
         throw new ProjectManagementPresentationException("ILLEGAL ACCESS TO EVENT HANDLER (is it public?): " +
            event, ex);
      }
   }

   /**
    * This sets the first letter of the event parameter value in order
    * to adhere to Java method naming conventions.
    *
    * @param String event the incoming name of the event
    * @return String the properly capitalized name
    */
   private String toMethodName(String event) {
      StringBuffer methodName = new StringBuffer(STANDARD_METHOD_PREFIX);
      methodName.append(Character.toUpperCase(event.charAt(0)));

      if (event.length() > 1) {
         methodName.append(event.substring(1));
      }

      return methodName.toString();
   }

   /**
    * Returns the application object associated with the
    * current request.
    *
    * @return the application object.
    */
   public ProjectManagement getApplication() {
      return (ProjectManagement)Enhydra.getApplication();
   }

   /**
    * Method to write a debugging message to the debug log
    * channel when the DEBUG flag is turned on
    *
    * @param msg The message to write to the DEBUG log channel
    */
   public static void writeDebugMsg(String msg) {
      Enhydra.getLogChannel().write(Logger.DEBUG,msg);
   }

   /**
   * Returns true if the given string is null, empty, or contains
   * only the white space(s).
   */
   protected static boolean isNullField (String field) {
      if (field==null) {
         return true;
      }

      if (field.trim().equals("")) {
         return true;
      }
      return false;
   }

   public static boolean areDateFieldsValid (String YYYY,String MM,String DD) {
      if (isNullField(YYYY) || isNullField(MM) || isNullField(DD)) {
         return false;
      }
      Calendar cal=new GregorianCalendar();
      cal.setLenient(false);
      try {
         int yyyy=Integer.valueOf(YYYY).intValue();
         if (yyyy<1900) {
            return false;
         }
         int mm=Integer.valueOf(MM).intValue();
         int dd=Integer.valueOf(DD).intValue();
         cal.set(yyyy,mm-1,dd);
         // if following method throws exception, date is not ok
         cal.getTime();
      } catch (Exception ex) {
         return false;
      }
      return true;
   }
}
